package cologne.eck.calendar_pea.util;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.Collections;

import cologne.eck.calendar_pea.data.PEvent;
import cologne.eck.calendar_pea.gui.event_display.EventButton;
import cologne.eck.tools.UnexpectedValueException;

public class CalendarComposer {

	private final static LocalTime DAY_START = LocalTime.of(0,  0);

	/**
	 * Sort and ArrayList of PEvents for one day and create two list
	 * of EventButtons for this events: 
	 * at index 0 all-day events
	 * at index 1 with-time events
	 * 
	 * @return	two sorted lists of events, 
	 * 			sorted by the start time in this day
	 * 			at index 0 all-day events
	 * 			at index 1 with-time events
	 */
	public static ArrayList<ArrayList<EventButton>> sortEventList(ArrayList<PEvent> eventList, LocalDate date){

		if (eventList == null) {
			return null;
		}
		int listSize = eventList.size();		
		if (listSize == 0) {
			return null;
		}
		//TestLog.o(XXXX.class, "Composer list size: " + listSize);
		ArrayList<ArrayList<EventButton>> list = new ArrayList<ArrayList<EventButton>>(2);		
		ArrayList<EventButton> allDayList = new ArrayList<EventButton>();
		ArrayList<EventButton> withTimeList = new ArrayList<EventButton>();
		LocalDateTime dayStartDateTime = date.atTime(DAY_START);
		
		for (int i = 0; i < listSize; i++) {
			// set startInBetween for all events			
			
			PEvent e = eventList.get(i);			
			if (e.getCalendar().getEventList().size() == 0) {
				new UnexpectedValueException("int", "size of eventList", "is 0:"+ 
						e.getCalendar().getName() + " " + e.getCalendar().getCalUid()).printDescription();
			}
			if (e.isAllDay()) {
				allDayList.add( new EventButton(e, DAY_START));
			} else if (e.isSeveralDays()) {
				if (e.getDateStart().isBefore(dayStartDateTime)) {
					withTimeList.add( new EventButton(e, DAY_START));
				} else {
					withTimeList.add(new EventButton(e, e.getDateStart().toLocalTime()));
				}				
			} else {
				withTimeList.add(new EventButton(e, e.getDateStart().toLocalTime()));
			}
		}		
		Collections.sort(withTimeList);
		Collections.sort(allDayList);
		list.add(allDayList);
		list.add(withTimeList);
		return list;
	}	
}