package cologne.eck.js_cal;

import java.io.File;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.TextStyle;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Locale;

import javax.swing.JOptionPane;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.calendar_pea.control.CalendarControl;
import cologne.eck.calendar_pea.data.PEvent;
import cologne.eck.calendar_pea.data.RecurringPEvent;
import cologne.eck.calendar_pea.gui.LockFrameCalendar;
import cologne.eck.calendar_pea.gui.event_display.EventButton;
import cologne.eck.calendar_pea.gui.event_setting.DatePicker;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;


public class JS_Setter {

/*	public JS_Setter() {
		DatenSetzen.setMonthNames(DatePicker.generateMonthStrings());// local strings
		DatenSetzen.setJsFileName(getJsFileName());
	}*/

	public static void setJsFile(int numberOfDays) {
		
		DatenSetzen.setMonthNames(DatePicker.generateMonthStrings());// local strings
		DatenSetzen.setJsFileName(getJsFileName());
		
		// get list of all shown events
		//ArrayList<PCalendar> calendarList = CalendarControl.getShownCalendarList();

		LocalDateTime dateToSet = LocalDate.now().atStartOfDay();
		
		ArrayList<ArrayList<ArrayList<EventButton>>> eventButtonLists = CalendarControl.getSortedShownEventButtonList(LocalDate.now(), numberOfDays);

		
		// set events for 30 days:
		for (int day = 0; day < numberOfDays; day++) {
			// list of non yearly events of this day
			ArrayList<PEvent> dayList = new ArrayList<PEvent>();
			// list for yearly events of this day
			ArrayList<PEvent> yearlyEvents = new ArrayList<PEvent>();			
			ArrayList<ArrayList<EventButton>> oneDayList = eventButtonLists.get(day);
			if (oneDayList == null) {
				//TestLog.o(XXXX.class, "No events in day "+ dateToSet.plusDays(day));
			} else {
				ArrayList<EventButton> allDayList = oneDayList.get(0);
				ArrayList<EventButton> timeList = oneDayList.get(1);
				if (allDayList != null && allDayList.isEmpty() == false) {
					int allDayListSize = allDayList.size();
					for (int i = 0; i < allDayListSize; i++) {
						PEvent e = allDayList.get(i).getEvent();
						if (e.isRecurring()) {
							RecurringPEvent re = (RecurringPEvent) e;
							if (re.getFrequency() != null && re.getFrequency().equals(ChronoUnit.YEARS)) {
								yearlyEvents.add(re);
							} else {
								dayList.add(re);
							}	
						} else {
							dayList.add(e);
						}					
					}
				}

				if (timeList != null && timeList.isEmpty() == false) {
					int timeListSize = timeList.size();
					for (int i = 0; i < timeListSize; i++) {
						PEvent e = timeList.get(i).getEvent();
						if (e.isRecurring()) {
							RecurringPEvent re = (RecurringPEvent) e;
							if (re.getFrequency() != null && re.getFrequency().equals(ChronoUnit.YEARS)) {
								yearlyEvents.add(re);
							} else {
								dayList.add(re);
							}	
						} else {
							dayList.add(e);
						}
					}
				}
			}
			
	/*		for (int i = 0; i < xe; i++) {
				ArrayList<ArrayList<PEvent>> list = calendarList.get(i).getEventsBetween(dateToSet, dateToSet.plusDays(1));//();

				dayList.addAll(list.get(0));
				dayList.addAll(list.get(1));
				// recurring list: yearly events extra
				int recurringListSize = list.get(2).size();
				
				if (recurringListSize > 0) {
					for (int j = 0; j < recurringListSize; j++) {
						RecurringPEvent re = (RecurringPEvent) list.get(2).get(j);
						if (re.getFrequency().equals(ChronoUnit.YEARS)) {
							yearlyEvents.add(re);
						} else {
							dayList.add(re);
						}					
					}
				}
			}*/
			String firstLine = dateToSet.getDayOfWeek().getDisplayName( 
					TextStyle.SHORT , Locale.getDefault() ).substring(0, 2)
					+ " " 
					+ dateToSet.getDayOfMonth()
					+ " " 
					+ dateToSet.getMonth().getDisplayName(TextStyle.FULL, Locale.getDefault())
					+ "\n";

			String yearly = "";
			if (yearlyEvents.size() > 0) {
				yearly = "*";
				for (int j = 0; j < yearlyEvents.size(); j++) {
					yearly += yearlyEvents.get(j).getSummary() + ", ";
				}
				// cut last , and add newLine
				yearly = yearly.substring(0,  yearly.length() - 2) + "\n";				
			}
			String dayListString = "";
			int dayListSize = dayList.size();
			for (int j = 0;j < dayListSize; j++) {
				String startTime = "";
				LocalTime time = dayList.get(j).getDateStart().toLocalTime();
				if (time.isAfter(LocalTime.of(0, 0))) {
					String minuteString = "" + time.getMinute();
					if (time.getMinute() < 10 ) {
						minuteString = "0" + minuteString;
					}
					startTime = time.getHour() + ":" + minuteString + " ";
				}
				dayListString += startTime + dayList.get(j).getSummary() + "\n";
			}
			String text = firstLine + yearly + dayListString;

			DatenSetzen.tagSetzen(dateToSet.getYear(), text);		
			dateToSet = dateToSet.plusDays(1);
		}
	}
	/**
	 * Get the JavaScript file where events are to be stored
	 * from the js_file_path.txt
	 * 
	 * @return	the JS file,the events are to be stored
	 */
	static String getJsFileName() {
		byte[] fileContent = ReadResources.readExternFile(//System.getProperty("user.dir") + 
				"resources" + File.separator + "js_file_path.txt");
		if (fileContent == null) {
			new UnexpectedValueException("byte[]","fileContent","is null - cannt read file " + "resources" + File.separator + "js_file_path.txt").printDescription();
			return null;
		} else {
			TestLog.o(JS_Setter.class, "js_file_path.txt found");			
		}
		String contentString = new String(fileContent, PeaProperties.getCharset());
		String[] lines = contentString.split("\n");
		for (int i = 0; i < lines.length; i++) {
			File file = new File(lines[i]);
			if (file.exists() && file.canRead()) {
				TestLog.o(JS_Setter.class, "JS file found");
				return lines[i];
			}
		}
		JOptionPane.showMessageDialog(LockFrameCalendar.getFrame(), "js file not found", null, JOptionPane.ERROR_MESSAGE);
		return null;
	}
}
