package cologne.eck.note_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main frame of editor pea.  
 */

import java.awt.*;
import java.awt.font.TextAttribute;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.StyledDocument;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.undo.*;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.StorageProvider;
import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.CloudFilePanel;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.IconManager;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;


@SuppressWarnings("serial")
final class LockFrameEditor extends PeaLockFrame 
			implements 
			WindowListener, // windowClosingCommands
			DocumentListener, // to register if changes in text are saved
			ChangeListener, // for JTabbedPane focus change
			FocusListener,
			ActionListener { 
	
	// this frame
	private static LockFrameEditor frame;
	
	/*
	 * The label for messages and errors
	 */
	private static JLabel messageLabel;

	/**
	 * The tab pane containing one tab for each encrypted
	 * text file. Only one file at the same time is decrypted
	 * and displayed: the file content associated with the 
	 * selected and focused tab. 
	 */
	private static JTabbedPane tabPane;
	
	/** this is the actual focused tab with EditorKit, MouseListener, UndoManager
	 * before the focus is changing
	 */
	private static TabComponent lastFocusTab;
	
	//private static JMenuBar colorBar = null;
	private static JPanel buttonMenuPanel  = null;

	// activate links or set text editable (is set visible)
	protected static JButton setEditableButton = null;
	
	/**
	 * True if the files contains a great number or ratio of links. 
	 * Show the activate / edit button
	 */
	private static boolean isBookmarkFile = false;
	
	// Menu for header if doc is HTML
	private static JMenuBar headerBar = null;
	
	/**
	 * UndoManager, which is always registered to the currently
	 * selected and focused tab (its JTextPane)
	 */
	protected static UndoManager manager;// = new UndoManager();
	//TODO protected static RTFEditorKit kit = null;
	private static HTMLEditorKit kit = null;
	
	// if the content of the current document was modified: true
	private boolean docChangeUnsaved = false;
	
	/**
	 * The associated password dialog, which calls the instance
	 * of this class
	 */
	private static EditorControl dialog;
	



	private LockFrameEditor(EditorControl _dialog, Point location) {

		boolean previousMode = PeaControl.isPlainTextMode();
		// always plain text mode - CloudMenu needs this
		PeaControl.setPlainTextMode(true);
				
		frame = this;
		dialog = _dialog;
		
	    this.setIconImage( PswDialogView.getImage() );
	    this.setTitle("NotebookPEA");	   
	    
	    frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	    
		this.addWindowListener(this);// for windowClosing
				
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		contentPane.setToolTipText("ALTGR + TAB: select file    CTRL + TAB: go to buttons     ESC: go to text");
		this.setContentPane(contentPane);
				
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		LockFrameEditorMenu menu = new LockFrameEditorMenu(this);		
		menuPanel.add(menu);
		menuPanel.setMinimumSize(new Dimension (300, 40));
		menuPanel.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel buttonMenuPanelHolder = new JPanel();
		buttonMenuPanelHolder.setLayout(new BoxLayout(buttonMenuPanelHolder, BoxLayout.X_AXIS));
		
		// TODO appear only for bookmarks

		setEditableButton = new JButton(
				PeaProperties.getVmBridge().translate("make_links_clickable"));
		setEditableButton.addActionListener(menu);
		setEditableButton.setActionCommand("makeLinksClickable");
		setEditableButton.setBackground(new Color(255, 209, 179));
		buttonMenuPanelHolder.add(setEditableButton);
		if (isBookmarkFile == false) {
			setEditableButton.setVisible(false);
		}

		buttonMenuPanelHolder.add(Box.createHorizontalGlue());	
		
		GridLayout grid = new GridLayout(1, 0);
		grid.setVgap(0);
		grid.setHgap(0);
		buttonMenuPanel = new JPanel();
		buttonMenuPanel.setLayout(grid);
		buttonMenuPanel.setMinimumSize(new Dimension(300, 30));	
		buttonMenuPanel.setPreferredSize(new Dimension(600, 30));	
		buttonMenuPanel.setMaximumSize(new Dimension(600, 30));	
		buttonMenuPanel.setToolTipText(
	    	    "Move with TAB, return to text with ESCAPE");
				
		buttonMenuPanel.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(
		KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0 ), "FOCUS_TO_TEXT");
		//KeyStroke.getKeyStroke(KeyEvent.VK_TAB, java.awt.event.InputEvent.ALT_GRAPH_DOWN_MASK ), "SWITCH_FOCUS");
		// WHEN_IN_FOCUSED_WINDOW WHEN_ANCESTOR_OF_FOCUSED_COMPONENT
		AbstractAction rta = new ReturnToTextAction();
		buttonMenuPanel.getActionMap().put("FOCUS_TO_TEXT", rta);
		
		buttonMenuPanelHolder.add(Box.createHorizontalGlue());	
		buttonMenuPanelHolder.add(buttonMenuPanel);	

		contentPane.add(buttonMenuPanelHolder);
		
		// border for buttons to show focus:
		LineBorder border = new LineBorder(Color.LIGHT_GRAY);
				
		JButton nineButton = new JButton();
		nineButton.setAction(new StyledEditorKit.FontSizeAction("" + 9, 9));
		nineButton.setToolTipText(PeaProperties.getVmBridge().translate("font_size") + " " + 9);
		nineButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 9));
		nineButton.setBorder(border);
		nineButton.setContentAreaFilled(false);
		nineButton.addFocusListener(this);
		buttonMenuPanel.add(nineButton);
		
		JButton twelveButton = new JButton();
		twelveButton.setAction(new StyledEditorKit.FontSizeAction("" + 12, 12));
		twelveButton.setToolTipText(PeaProperties.getVmBridge().translate("font_size") + " " + 12);
		twelveButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		twelveButton.setBorder(border);
		twelveButton.setContentAreaFilled(false);
		twelveButton.addFocusListener(this);
		buttonMenuPanel.add(twelveButton);		
		
		JButton fifteenButton = new JButton();
		fifteenButton.setAction(new StyledEditorKit.FontSizeAction("" + 15, 15));
		fifteenButton.setToolTipText(PeaProperties.getVmBridge().translate("font_size") + " " + 14);
		fifteenButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 15));
		fifteenButton.setBorder(border);
		fifteenButton.setContentAreaFilled(false);
		fifteenButton.addFocusListener(this);
		buttonMenuPanel.add(fifteenButton);
		
		JButton boldButton = new JButton();
		boldButton.setAction(new StyledEditorKit.BoldAction());
		boldButton.setText("" + PeaProperties.getVmBridge().translate("bold").charAt(0));
		boldButton.setToolTipText(PeaProperties.getVmBridge().translate("bold"));
		boldButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 12));
		boldButton.setBorder(border);
		boldButton.setContentAreaFilled(false);
		boldButton.addFocusListener(this);
		buttonMenuPanel.add(boldButton);

		JButton italicButton = new JButton();
		italicButton.setAction(new StyledEditorKit.ItalicAction());
		italicButton.setText("" + PeaProperties.getVmBridge().translate("italic").charAt(0));
		italicButton.setToolTipText(PeaProperties.getVmBridge().translate("italic"));
		italicButton.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, 12));
		italicButton.setBorder(border);
		italicButton.setContentAreaFilled(false);
		italicButton.addFocusListener(this);
		buttonMenuPanel.add(italicButton);

		JButton underlineButton = new JButton();
		underlineButton.setAction(new StyledEditorKit.UnderlineAction());
		underlineButton.setText("" + PeaProperties.getVmBridge().translate("underline").charAt(0));
		underlineButton.setToolTipText(PeaProperties.getVmBridge().translate("underline"));		
		Font original = new Font(Font.SANS_SERIF, Font.PLAIN, 12);
		try {
			@SuppressWarnings (value="unchecked")
			Map<TextAttribute, Integer> attributes = (Map<TextAttribute, Integer>) original.getAttributes();
			attributes.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
			underlineButton.setFont(original.deriveFont(attributes));
		} catch (Exception e) {
			// ignore
		}
		underlineButton.setBorder(border);
		underlineButton.setContentAreaFilled(false);
		underlineButton.addFocusListener(this);
		buttonMenuPanel.add(underlineButton);

		JButton leftButton = new JButton();
		leftButton.setAction(new StyledEditorKit.AlignmentAction("", 0));
		leftButton.setToolTipText(PeaProperties.getVmBridge().translate("left"));		
		leftButton.setIcon(IconManager.loadIcon("left-align.png", "left alignment",
				"left", "align selected text left"));
		leftButton.setRolloverIcon(IconManager.loadIcon("left-align-rollover.png", "left alignment",
				"left", "align selected text left"));
		leftButton.setPressedIcon(IconManager.loadIcon("left-align.png", "left alignment",
				"left", "align selected text left"));
		leftButton.setBorder(border);
		leftButton.setContentAreaFilled(false);
		leftButton.addFocusListener(this);
		buttonMenuPanel.add(leftButton);
		
		JButton centerButton = new JButton();
		centerButton.setAction(new StyledEditorKit.AlignmentAction("", 1));
		centerButton.setToolTipText(PeaProperties.getVmBridge().translate("center"));
		centerButton.setIcon(IconManager.loadIcon("center-align.png", "center alignment",
				"center", "align selected text center"));
		centerButton.setPressedIcon(IconManager.loadIcon("center-align.png", "center alignment",
				"center", "align selected text center"));
		centerButton.setRolloverIcon(IconManager.loadIcon("center-align-rollover.png", "center alignment",
				"center", "align selected text center"));
		centerButton.setBorder(border);
		centerButton.setContentAreaFilled(false);
		centerButton.addFocusListener(this);
		buttonMenuPanel.add(centerButton);		
		
		JButton rightButton = new JButton();
		rightButton.setAction(new StyledEditorKit.AlignmentAction("", 2));
		rightButton.setToolTipText(PeaProperties.getVmBridge().translate("right"));
		rightButton.setIcon(IconManager.loadIcon("right-align.png", "right alignment",
				"right", "align selected text right"));
		rightButton.setPressedIcon(IconManager.loadIcon("right-align.png", "right alignment",
				"right", "align selected text right"));
		rightButton.setRolloverIcon(IconManager.loadIcon("right-align-rollover.png", "right alignment",
				"right", "align selected text right"));
		rightButton.setBorder(border);
		rightButton.setContentAreaFilled(false);
		rightButton.addFocusListener(this);
		buttonMenuPanel.add(rightButton);		
		
		
		// Colors:
		// colouMenu is only accessible by keyboard with mnemonic 8
		headerBar = new JMenuBar() ;		
		headerBar.addFocusListener(this);
		headerBar.setBorder(border);
		/*int[] vals = PeaProperties.getPlainPeaColor();
		headerBar.setBackground(new Color(vals[0], vals[1], vals[2]));
		headerBar.setOpaque(true);*/
		
		JMenu headerMenu = new JMenu(" H1 - H6");//PeaProperties.getVmBridge().translate("color"));
		int[] vals = PeaProperties.getPlainPeaColor();
		headerMenu.setBackground(new Color(vals[0], vals[1], vals[2]));
		headerMenu.setMnemonic('H');
		headerMenu.setToolTipText(PeaProperties.getVmBridge().translate("header"));
		headerMenu.setBorder(border);
		//headerMenu.setContentAreaFilled(false);		
		headerMenu.addFocusListener(this);
		headerMenu.setAlignmentX(CENTER_ALIGNMENT);
		//headerMenu.setBackground(Color.WHITE);
		headerMenu.setOpaque(true);
		buttonMenuPanel.add(headerBar);
		headerBar.add(headerMenu);


		JMenuItem h1Item = new JMenuItem("  H1 ");
		h1Item.getAccessibleContext().
		setAccessibleDescription(
				"Clicking this component sets an H1 header.");
		h1Item.addActionListener(this);
		h1Item.setActionCommand("H1");
		headerMenu.add(h1Item);

		JMenuItem h2Item = new JMenuItem("  H2 ");
		h2Item.getAccessibleContext().
		setAccessibleDescription(
				"Clicking this component sets an H2 header.");
		h2Item.addActionListener(this);
		h2Item.setActionCommand("H2");
		headerMenu.add(h2Item);

		JMenuItem h3Item = new JMenuItem("  H3 ");
		h3Item.getAccessibleContext().
		setAccessibleDescription(
				"Clicking this component sets an H3 header.");
		h3Item.addActionListener(this);
		h3Item.setActionCommand("H3");
		headerMenu.add(h3Item);

		JMenuItem h4Item = new JMenuItem("  H4 ");
		h4Item.getAccessibleContext().
		setAccessibleDescription(
				"Clicking this component sets an H4 header.");
		h4Item.addActionListener(this);
		h4Item.setActionCommand("H4");
		headerMenu.add(h4Item);

		JMenuItem h5Item = new JMenuItem("  H5 ");
		h5Item.getAccessibleContext().
		setAccessibleDescription(
				"Clicking this component sets an H5 header.");
		h5Item.addActionListener(this);
		h5Item.setActionCommand("H5");
		headerMenu.add(h5Item);

		JMenuItem h6Item = new JMenuItem("  H6 ");
		h6Item.getAccessibleContext().
		setAccessibleDescription(
				"Clicking this component sets an H6 header.");
		h6Item.addActionListener(this);
		h6Item.setActionCommand("H6");
		headerMenu.add(h6Item);


		// Colors:
		// colouMenu is only accessible by keyboard with mnemonic 8
		JMenuBar colorBar = new JMenuBar() ;		
		colorBar.addFocusListener(this);
		colorBar.setBorder(border);
		
		JMenu colorMenu = new JMenu("");//PeaProperties.getVmBridge().translate("color"));
		colorMenu.setMnemonic(KeyEvent.VK_NUMBER_SIGN);
		colorMenu.setToolTipText(PeaProperties.getVmBridge().translate("color") + "   ( ALT + # )");
		colorMenu.setBorder(border);
		colorMenu.setContentAreaFilled(false);
		colorMenu.addFocusListener(this);
		colorMenu.setIcon(IconManager.loadIcon("colors.png", "select color",
				"colors", "open color chooser"));
		colorMenu.setAlignmentX(CENTER_ALIGNMENT);
		colorMenu.setBackground(Color.WHITE);
		colorMenu.setOpaque(true);
		buttonMenuPanel.add(colorBar);
		colorBar.add(colorMenu);
				
		JMenuItem colorButton0 = new JMenuItem("");
		colorButton0.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to gold.");
		colorButton0.setAction(new StyledEditorKit.ForegroundAction("###",new Color(139,139, 0)));
		colorButton0.setBackground(new Color(139,139, 0));//yellow4
		colorButton0.setForeground(new Color(139,139, 0));
		colorMenu.add(colorButton0);

		JMenuItem colorButton1 = new JMenuItem("");
		colorButton1.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to orange.");
		colorButton1.setAction(new StyledEditorKit.ForegroundAction("###",new Color(205,102,0)));//Color.ORANGE));
		colorButton1.setBackground(new Color(205,102,0));//darkorange2
		colorButton1.setForeground(new Color(205,102,0));
		colorMenu.add(colorButton1);
		
		JMenuItem colorButton2 = new JMenuItem("*");
		colorButton2.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to red.");
		colorButton2.setAction(new StyledEditorKit.ForegroundAction("###",new Color(205,0,0)));
		colorButton2.setBackground(new Color(205,0,0));//red3
		colorButton2.setForeground(new Color(205,0,0));
		colorMenu.add(colorButton2);

		JMenuItem colorButton3 = new JMenuItem("*");
		colorButton3.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to pink.");
		colorButton3.setAction(new StyledEditorKit.ForegroundAction("###",new Color(205,0,205)));
		colorButton3.setBackground(new Color(205,0,205));//magenta3
		colorButton3.setForeground(new Color(205,0,205));//magenta3
		colorMenu.add(colorButton3);
		
		JMenuItem colorButton4 = new JMenuItem("*");
		colorButton4.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to blue.");
		colorButton4.setAction(new StyledEditorKit.ForegroundAction("###",Color.BLUE));
		colorButton4.setBackground(Color.BLUE);
		colorButton4.setForeground(Color.BLUE);
		colorMenu.add(colorButton4);
		
		JMenuItem colorButton5 = new JMenuItem("*");
		colorButton5.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to bright blue.");
		colorButton5.setAction(new StyledEditorKit.ForegroundAction("###",new Color(0,139,139)));
		colorButton5.setBackground(new Color(0,139,139));//cyan4
		colorButton5.setForeground(new Color(0,139,139));//cyan4
		colorMenu.add(colorButton5);
		
		JMenuItem colorButton6 = new JMenuItem("*");
		colorButton6.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to green.");
		colorButton6.setAction(new StyledEditorKit.ForegroundAction("###",new Color(0,139,0)));
		colorButton6.setBackground(new Color(0,139,0));// green4
		colorButton6.setForeground(new Color(0,139,0));// green4
		colorMenu.add(colorButton6);		
		
		JMenuItem colorButton8 = new JMenuItem("*");
		colorButton8.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to gray.");
		colorButton8.setAction(new StyledEditorKit.ForegroundAction("###",Color.DARK_GRAY));
		colorButton8.setBackground(Color.DARK_GRAY);
		colorButton8.setForeground(Color.DARK_GRAY);
		colorMenu.add(colorButton8);		
		
		JMenuItem colorButton9 = new JMenuItem("*");
		colorButton9.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to black.");
		colorButton9.setAction(new StyledEditorKit.ForegroundAction("###",Color.BLACK));
		colorButton9.setBackground(Color.BLACK);
		colorButton9.setForeground(Color.BLACK);
		colorMenu.add(colorButton9);


		// tab are added in PsDialogEditor startDecryption
		// if resetFrame was called, the tabPane is not null
		if (tabPane == null) {
			tabPane = new JTabbedPane();
		}
		// register ESCAPE to return to the text and ALTGR+ TAB to select tab
		tabPane.setToolTipText(
	    	    "Return to text with ESCAPE, select tab with CTRL + ENTER");	
		tabPane.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put( // WHEN_IN_FOCUSED_WINDOW
				KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0 ), "FOCUS_TO_TEXT");
		tabPane.getActionMap().put("FOCUS_TO_TEXT", rta);	
		
		contentPane.add(tabPane);
		
		JPanel messagePanel= new JPanel();
		messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.LINE_AXIS));
		messageLabel = new JLabel();
		messagePanel.add(messageLabel);
		messagePanel.add(Box.createHorizontalGlue());
		contentPane.add(messagePanel);

		this.setLocation(location);
		
		// Notebook needs a minimum width to display tool bar, others than other PEAs
	//	pack();
		int[] dim  = JREProperties.getFrameDim();
		if (dim[0] > PswDialogView.getScreenwidth()) {
			dim[0] = (int) PswDialogView.getScreenwidth();
		}
		if (dim[1] > PswDialogView.getScreenheight()) {
			dim[1] = (int) PswDialogView.getScreenheight();
		}

		this.setSize(new Dimension(dim[0], dim[1]));
		//menu.setRootWindowForCloudMenu();
		
/*		if (((TabComponent) tabPane.getSelectedComponent()).getTextPane().getContentType().equals("text/html")) {
			headerMenu.setVisible(true);
		} else {
			headerMenu.setVisible(false);
		}*/

		// reset mode:
		PeaControl.setPlainTextMode(previousMode);
		docChangeUnsaved = false;
	}
	/**
	 * Get the current instance of ContentUtilities or create a
	 * new one if no instance exists - there is always only
	 * one instance. 
	 * 
	 * @param pswDialog		the associated password dialog
	 * @param loc			the location on screen 
	 * 
	 * @return				the instance of ContentUtilities
	 */
	protected static LockFrameEditor getInstance(EditorControl pswDialog, Point loc) {
		LockFrameEditor frameInstance = null; 
		if (frame == null) {
			frameInstance = new LockFrameEditor(pswDialog, loc);
			LockFrameEditorMenu.setLockFrame(frameInstance);
		} else {
			frameInstance = frame;
		}
		return frameInstance;
	}

	/**
	 * Create a new tab for the given file name and add
	 * it to the JTabbedPane. Unless the tab gets the focus,
	 * the content of the file is not decrypted and displayed. 
	 * 
	 * @param fileNameWithPath	the name of the text file
	 */
	void createNewTab(HashSet<String> set){	

		@SuppressWarnings("unused") // add tab in method, not in constructor...
		TabComponent tab = new TabComponent(tabPane, set);
	}
	
	/**
	 * Manually add ChangeListener after all tab were added, 
	 * set focus to first tab
	 */
	void setChangeListener(){

		if (tabPane.getTabCount() > 0) {
			// check if there is a selected component
			if (tabPane.getSelectedComponent() != null) {
				// try to set focus on selected component
				if (tabPane.getSelectedComponent() instanceof TabComponent) {
					String focusError = changeFocus((TabComponent) tabPane.getSelectedComponent());
					if (focusError != null) { // set focus failed
						// try to set focus on component at index 0
						if (tabPane.getTabCount() > 0 && tabPane.getTabComponentAt(0) instanceof TabComponent) {
							tabPane.setSelectedIndex(0);
							focusError = changeFocus((TabComponent) tabPane.getSelectedComponent());
							if (focusError != null) { // set focus failed
								// try to find any valid tab to set focus
								int tabCount = tabPane.getTabCount();
								TabComponent tab = null;
								// check all tabs:
								for (int i = 0; i < tabCount; i++) {
									Component cTab = tabPane.getComponentAt(i);// the tab title panel
									Component cTabComp = tabPane.getTabComponentAt(i);// the TabComponent
									if (cTab != null && cTabComp != null) {
										if (cTabComp instanceof TabComponent) {
											// valid tab found:
											tab = (TabComponent) tabPane.getComponentAt(i);
											break;
										}
									}
								}
								if (tab == null) { // no tab found:
									new UnexpectedValueException("TabComponent","tab", "is not null - can not set focus on tab").printDescription();
									// close program, otherwise a text pane without function appears
									windowClosingCommands();
								}
								focusError = changeFocus(tab);
								if (focusError != null) { // no valid tab found
									new UnexpectedValueException("String","focusError", "is not null - can not set focus on tab").printDescription();
									windowClosingCommands();
								}
							}
						} else {
							//TestLog.ve(LockFrameEditor.class, "Can not set selected component. tabPane size: " + tabPane.getTabCount(), false);
							if (tabPane.getTabCount() > 0) {
								TestLog.e(LockFrameEditor.class, "Can not set select component: " + tabPane.getTabComponentAt(0).getClass().getSimpleName());
							}
						}
					}
				} else {
					TestLog.ve(LockFrameEditor.class, "not a TabComponent: " + tabPane.getSelectedComponent().getClass().getName(), 5);
				}
			} else {
				tabPane.setSelectedIndex(0);
				changeFocus((TabComponent) tabPane.getSelectedComponent());
			}
		}
		tabPane.addChangeListener(this);
		Component c =  tabPane.getSelectedComponent();//.getTabComponentAt(tabPane.getSelectedIndex());
		if (c instanceof TabComponent) {
			//lastFocusTab = (TabComponent) c;
			setLastFocusTab( (TabComponent) c);
		}
	}	

	@Override
	public void markUnsavedContentChanges() {
		docChangeUnsaved = true;
	}
	@Override
	public void markContentIsSaved() {
		docChangeUnsaved = false;
	}
	@Override
	public boolean isContentUnsaved() {
		return docChangeUnsaved;
	}

	/**
	 * Check if the text of the last selected tab
	 * was modified and not saved manually and 
	 * save the text in this case. 
	 * 
	 * @return	true if content is to be saved
	 */
	public boolean checkUnsavedContentChanges() {
		if (docChangeUnsaved == true) {
			int n = JOptionPane.showConfirmDialog(
				    this,
				    PeaProperties.getVmBridge().translate("save_modifications"),//"Save changes?",
				    PeaProperties.getVmBridge().translate("unsaved_modifications"),//"Unsaved Changes",
				    JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION) {
				HashSet<String> tabSet = lastFocusTab.getFileNames();
				byte[] plainBytes = getPlainBytes();
				byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, // is zeroized
						CipherStuff.getSessionKeyCrypt().getKey(), // keyMaterial is overwritten
						true, null ); // key is stored encrypted
				boolean cloudFileToSave = false;
				for (String nameInTab : tabSet) {					
					ContentUtilities.saveContent(null, null, cipherBytes, nameInTab, true);
					if (nameInTab.contains(CloudControl.getTmpDir())) {
						cloudFileToSave = true;
					}
				}
				// if cloud files: wait until all UploadWorkers started
				// and set running process....
				if (cloudFileToSave == true) {
					// wait a short time, because all UploadWorkers
					// must have initialized and set running processes
					try {
						Thread.sleep(50);
					} catch (InterruptedException e) {
						//e.printStackTrace();
					}
				}
				docChangeUnsaved = false;
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	/**
	 * If a user sets the focus on a new tab manually
	 * or if the focus is set by the program:
	 * Change the title of the new selected tab and
	 * - set EditorKit
	 * - add MouseListener for CopyCutPastePopup
	 * - add DocumentListener to register unsaved changes
	 * - add Redo-Undo-Manager
	 * - register KeyStrokes
	 * and remove all this from the previously focused tab
	 * 
	 * @param newFocusTab		the tab that should get the focus
	 * 
	 * @return an error message if the setting of the focus failed
	 */
	private String changeFocus(TabComponent newFocusTab) {

		if (newFocusTab == null) {
			docChangeUnsaved = false;
			TestLog.e(LockFrameEditor.class, "Missing tab to set focus");
			return "Missing tab to set focus";
		}		
		if (tabPane.getTabCount() == 0 ){
			TestLog.e(LockFrameEditor.class, "tabPane is empty");
			docChangeUnsaved = false;
			return "Empty tab pane";
		}
		// Handle the previously selected tab: 
		if (	lastFocusTab != null 								// exists
				&& (tabPane.indexOfComponent(lastFocusTab) != -1)	// is in tabPane
				) {			// was not closed before
			
			// ask to save text if text was changed and not saved manually:
			checkUnsavedContentChanges();
			
			// set no image and gray background: 
			lastFocusTab.setTabTitle(false);
			
			// get the associated JTextPane:
			JTextPane oldTextPane = lastFocusTab.getTextPane();
			
			// remove DocumentListener:
			oldTextPane.getDocument().removeDocumentListener(this);
			
			// remove Cut-Copy-Paste.Menue: 
			oldTextPane.removeMouseListener(lastFocusTab);

			// remove redo and undo manager:
			oldTextPane.getDocument().removeUndoableEditListener(manager);
			
			oldTextPane.setText("");
			   
			//remove keyStroke: STRG + R, STRG + W
			oldTextPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
			            KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK ));
			oldTextPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
			            KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK ));	
		} else {
			//new UnexpectedValueException("TabComponent", "lastFocusTab", "is missing").printDescription();
		}

		// get the JTextPane of the new tab:
		JTextPane textPane = newFocusTab.getTextPane();		
		
		// set EditorKit:
		/// TODO kit = new RTFEditorKit();
		kit = new HTMLEditorKit();
		textPane.setEditorKit(kit);
		
		// set the text
		String errorMessage = newFocusTab.setText();
		boolean newFocusTabFailed = false;
		if (errorMessage != null) { // decryption failed
			
			newFocusTabFailed = true;
			
			PeaDialog.showMessage(this, newFocusTab.getFileNameWithoutPath() + ": "
			+ PeaProperties.getVmBridge().translate("decryption_failed")
			+ "\n" + errorMessage, PeaProperties.getVmBridge().translate("error")
			, 0);
			
			TestLog.e(LockFrameEditor.class, "decryption failed for new focused tab: " + newFocusTab.getFileNameWithoutPath()
					+ ", " + errorMessage);
			docChangeUnsaved = false;

			// remove file name from list:
			ArrayList<String> list = PeaControl.getAvailableFileNames();
			HashSet<String> tabSet = newFocusTab.getFileNames();
			boolean removedFromSet = false;
			if (tabSet != null && tabSet.isEmpty() == false) {
				for (String nameInSet : tabSet) {
					if (list.contains(nameInSet)) {
						list.remove(nameInSet);
						PeaControl.setAvailableFileNames(list);
					}
					// remove set from model:
					if (removedFromSet == false) {
						removedFromSet = NotebookModel.removeFromMap(nameInSet);
					}
				}
			}
			
			try{
				// remove DocumentListener:
				textPane.getDocument().removeDocumentListener(this);
				
				// remove Cut-Copy-Paste.Menue: 
				if (newFocusTab != null) {
					textPane.removeMouseListener(newFocusTab);
				}

				// remove redo and undo manager:
				textPane.getDocument().removeUndoableEditListener(manager);
				   
				//remove keyStroke: STRG + R, STRG + W
				textPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
				            KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK ));
				textPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
				            KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK ));	
			} catch (Exception e) {
				TestLog.e(LockFrameEditor.class, e.getMessage());
			}
			
			// get the index of the new tab to set the focus:
			int index = tabPane.indexOfComponent(lastFocusTab);

			if (index != -1) { //  in tabPane
				// Handle new selection of tab first: if possible, set to lastFocusTab
				if (lastFocusTab != null 
						//&& lastFocusTab.equals(newFocusTab) == false
						&& (tabPane.indexOfComponent(lastFocusTab) != -1)
						&& lastFocusTab.getFileNameWithoutPath() != null) {
					tabPane.setSelectedComponent(lastFocusTab);
				} else {
					// set focus to first tab
					if (tabPane.getTabCount() > 0) {
						tabPane.setSelectedComponent(tabPane.getComponentAt(0));
					} else {
						PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("no_valid_file_found"), PeaProperties.getVmBridge().translate("error"), 0);
						return PeaProperties.getVmBridge().translate("no_valid_file_found");
					}
				}
				// remove the tab (which is not selected now)
				if (newFocusTab != null &&
						tabPane.getTabCount() > 0 && 
						(tabPane.indexOfComponent(newFocusTab) != -1)){
					try { // if there is no valid tab left, this will throw an IndexOutOfBoundException
						tabPane.remove(newFocusTab);
					} catch (IndexOutOfBoundsException e) {
						// delete the file names: files should never be overwritten
						newFocusTab.setFileName(new HashSet<String>());
						TestLog.e(LockFrameEditor.class, e.toString() + 
								", removing tab failed: " + newFocusTab.getFileNameWithoutPath()
								+ ", " + e.getMessage());
					}					
				} 
			} else {

				// remove the tab (which is not selected now)
				if (newFocusTab != null &&
						tabPane.getTabCount() > 0 && 
						(tabPane.indexOfComponent(newFocusTab) != -1)){
					try { // if there is no valid tab left, this will throw an IndexOutOfBoundException
						tabPane.remove(newFocusTab);
					} catch (IndexOutOfBoundsException e) {
						TestLog.e(LockFrameEditor.class, e.toString() + 
								", removing tab failed: " + newFocusTab.getFileNameWithoutPath()
								+ ", " + e.getMessage());
					}					
				} 
				// return an error message to show that an error appeared
				// this is neccesary, when the change listener is not yet registered in setChangeListener
				return ("lastFocusTab not in tabPane");
			}
		}
		
		// Cut-Copy-Paste.Menue: 
		textPane.addMouseListener(newFocusTab);
		// Undo/Redo: key and menu
		manager = new UndoManager();
		UndoAction undoAction = new UndoAction(manager);
		RedoAction redoAction = new RedoAction(manager);
		textPane.getDocument().addUndoableEditListener(manager);
		   
		//keyStroke: STRG + R, STRG + W
		textPane.registerKeyboardAction(undoAction, KeyStroke.getKeyStroke(
		            KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK ), JComponent.WHEN_FOCUSED);
		textPane.registerKeyboardAction(redoAction, KeyStroke.getKeyStroke(
		            KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK ), JComponent.WHEN_FOCUSED);
		
		// add listener
		textPane.getDocument().addDocumentListener(this);		

		// set tab title with pea icon and green background
		newFocusTab.setTabTitle(true); // true = icon, green

		if (newFocusTabFailed == false) {
			// set the component as lastFocusTab
			lastFocusTab = newFocusTab;
		}
		// DocumentListener should not fire event for text setting
		docChangeUnsaved = false;
		
		//setEditableButton.setVisible(false);
		if (newFocusTab.getTextPane().getContentType().equals("text/html")) {
			headerBar.setVisible(true);			
		} else {
			headerBar.setVisible(false);
		}
		
		return null;
	}
	
	
	//-------------------------override-----------------------------
	// ChangeListener: event if selection changes: 
	// there are some cases where getSelectedIndex() 
	// and getSelectedComponen() are not the same!!!
	@Override
	public void stateChanged(ChangeEvent ce) {	
		
		try { 
			// set the wait cursor: 
			this.setCursor(CursorManager.getWaitCursor());

			if (tabPane.getSelectedComponent() == null) {
				// no tab selected
				if (tabPane.getTabCount() > 0){
						//&& tabPane.getComponent(0) instanceof TabComponent) { // not TabContainer
					// set focus on first tab
					TabComponent newFocus = (TabComponent) tabPane.getComponentAt(0);
					if (newFocus != null &&
							tabPane.indexOfComponent(newFocus) != -1){
						tabPane.setSelectedComponent(newFocus);
					}
				} else {
					// no tab
					TestLog.v(LockFrameEditor.class, "No tab to set focus...");				
				}
				docChangeUnsaved = false;
			} else {
				changeFocus((TabComponent) tabPane.getSelectedComponent());
			}						
		} finally {
			// reset the cursor:
	    	this.setCursor(CursorManager.getDefaultCursor());
		} 
		//((TabComponent)tabPane.getSelectedComponent()).getTextPane() .setCaretPosition(0);
	}
	
	// DocumentListener
	@Override
	public void changedUpdate(DocumentEvent de) {
		if (lastFocusTab.getTextPane().getSelectedText() != null) {
			// only when there is selected text, otherwise this fires always
			docChangeUnsaved = true;
		}
	}
	@Override
	public void insertUpdate(DocumentEvent arg0) {
		docChangeUnsaved = true;	
	}
	@Override
	public void removeUpdate(DocumentEvent arg0) {
		docChangeUnsaved = true;
	}
	
	// ContentUtilities
	void windowClosingCommands() {
	
		// ask to save unsaved modifications of the text
		if (checkUnsavedContentChanges() == true) {
			// do not check twice
			docChangeUnsaved = false;
			// if cloud file is to be saved:
			// doInBackground must set running process before
			// checking if processes are running, 
			// otherwise changes may not be saved
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
				TestLog.e(LockFrameEditor.class, "Sleep thread interrupted...");
			}
		}	

		// check if all files are in path file and ask to store if not
		HashSet<String> availableSet = new HashSet<String>(PeaControl.getAvailableFileNames());
		// add files from model:
		ArrayList <HashSet<String>> setList = NotebookModel.getSetList();
		for (HashSet<String> setInList : setList) {
			availableSet.addAll(setInList);
		}
		if (availableSet.size() > 0) {
			PathFileManager.addFileNamesToPathFile(availableSet, true);
		}
		
		// save properties
		new PropertyHandler().updateAllPropValues();				
		
		// check if cloud processes are still running and ask to exit
		if (CloudControl.getNumberOfRunningCloudProcesses() > 0) {
			String[] options = {PeaProperties.getVmBridge().translate("cancel"),
					PeaProperties.getVmBridge().translate("ok")};					
			
			int result = PeaDialog.showOptionYAxis(this, 
					PeaProperties.getVmBridge().translate("cloud_process_warning")
					+ "\n" + PeaProperties.getVmBridge().translate("terminating_damage"), 
					PeaProperties.getVmBridge().translate("warning"), 
					2,  options, 0);

			if (result == 1) {
				TestLog.v(this.getClass(), "Closing although cloud process is running...");
			} else {
				TestLog.v(this.getClass(), "Upload process is running.... Break exit...");
				return;
			}
		}	

		// clear the text
		if (tabPane.getSelectedComponent() != null) {
			StyledDocument doc = ((TabComponent) tabPane.getSelectedComponent()).getTextPane().getStyledDocument();
			try {
				doc.remove(0, doc.getLength() );
			} catch (BadLocationException e) {
				TestLog.e(LockFrameEditor.class, "LockFrameCalendar: " + e + ", " + e.getLocalizedMessage());
				//e.printStackTrace();
			}			
		}

		// clear the sessionKeys:
		CipherStuff.getSessionKeyCrypt().clearKeys();	
		
		// Close all providers
		Collection<StorageProvider> openProviders = CloudControl.getOpenProviders();
		if (openProviders != null &&  ! openProviders.isEmpty()) {
			for (StorageProvider sp : openProviders) {
				String error = sp.closeSession();
				if (error != null) {
					TestLog.e(LockFrameEditor.class, error);
				}
			}
		}
		
		// delete all temporary backup files of cloud files:
		if (dialog.getEncryptedFileTypePanel() != null) { // after initializing maybe null
			CloudFilePanel cfp = dialog.getEncryptedFileTypePanel().getCloudFilePanel();
			if (cfp != null) {
				String[] providers = cfp.getCloudControl().getUsedProviders();
				if (providers != null) {
					for (String provName : providers) {	
						if (provName != null) {
							String[] filesOfProv = cfp.getCloudControl().getUsedFileNamesOfProvider(provName);
							if (filesOfProv != null) {
								for (String fileName : filesOfProv) {
									if (fileName != null && fileName.length() >0) {
										File fileToDelete = new File(CloudControl.getTmpFileNameForCloudFile(fileName, provName, null));
										if (fileToDelete.exists()) {
											boolean deleteResult = fileToDelete.delete();
											if (deleteResult == false) {
												TestLog.e(LockFrameEditor.class, "Could not delete " + fileToDelete);
											} else {
											}
										} else {
											TestLog.e(LockFrameEditor.class, "File form cloud to delete does not exist " + fileToDelete);
										}
									} else {
										TestLog.e(LockFrameEditor.class, "Could find file to delete from cloud " + provName);
									}
								}
							} else {
								TestLog.e(LockFrameEditor.class, "Could find file to delete from cloud... ");
							}
						} else {
							TestLog.e(LockFrameEditor.class, "Cloud provider is null... ");
						}
					}
				}
			}
		}
		TestLog.o(this.getClass(), "Exit");
		System.exit(0);			
	}

	/**
	 * Get plain text as byte array 
	 * of current text pane
	 * 
	 * @return	the plain text as byte array
	 */
	public byte[] getPlainBytes() {

		JTextPane textPane = lastFocusTab.getTextPane();//((TabComponent) tabPane.getSelectedComponent()).getTextPane();
		String fileNnameLower = lastFocusTab.getFileNameWithoutPath().toLowerCase();
		StyledDocument doc = null;
		if (fileNnameLower.endsWith(".html")) {
			doc = (HTMLDocument) textPane.getStyledDocument();
		} else if (fileNnameLower.endsWith(".rtf")) {
			doc = (DefaultStyledDocument) textPane.getStyledDocument();
		}
		if (doc == null) {
			TestLog.e(LockFrameEditor.class, "no document");
			return null;
		}
		byte[] plainBytes = null;

		// UndoListener can not be serialized.. 
		textPane.getDocument().removeUndoableEditListener(manager);
		if (fileNnameLower.endsWith(".html")) {
			plainBytes = TextHelper.serialize(doc, "html");
		} else if (fileNnameLower.endsWith(".rtf")) {
			plainBytes = TextHelper.serialize(doc, "rtf");
		} 
	    textPane.getDocument().addUndoableEditListener(manager);
		textPane.getDocument().addDocumentListener(this);

		return plainBytes;
	}
	
	@Override
	public void windowClosing(WindowEvent arg0) { // by X-Button
		windowClosingCommands();
	}
	
	@Override
	public void windowActivated(WindowEvent arg0) {
		// Set focus on text, otherwise, focus is an first button in menuButtonPanel...
		JTextPane tp = getFocusedTextPane();
		if (tp != null) {
			tp.requestFocusInWindow();
		} else {
			TestLog.ve(getClass(), "No text pane found");
		}
	}
	
	@Override
	public void windowClosed(WindowEvent arg0) {}
	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}
	@Override
	public void windowOpened(WindowEvent arg0) {}

		
	//--------------------------------------------------------------------------
	// Getter & setter
	
	/**
	 * Get the currently focused JTextPane 
	 * 
	 * @return	the currently focused JTextPane 
	 */
	protected static synchronized JTextPane getFocusedTextPane(){
		return lastFocusTab.getTextPane();
	}
	/**
	 * Get the currently selected tab
	 * 
	 * @return	the currently selected instance of TabComponent
	 */
	protected static synchronized TabComponent getCurrentTab(){
		if (lastFocusTab == null) {
			new UnexpectedValueException("TabComponent", "lastFocusTab", "is null").printDescription();
		}
		return lastFocusTab;
	}
	
	private static synchronized void setLastFocusTab(TabComponent c) {
		LockFrameEditor.lastFocusTab = c;
	}

	/**
	 * Set value for unsaved changes of the currently
	 * selected tabs JTextPane. 
	 * 
	 * @param docChangeUnsaved the docChangeUnsaved to set
	 */
	public void setDocChangeUnsaved(boolean newDocChangeUnsaved) {
		this.docChangeUnsaved = newDocChangeUnsaved;
	}
	
	/**
	 * Get value for unsaved changes of the currently
	 * selected tabs JTextPane. 
	 * 
	 * @return true if there are unsaved changes of the current document
	 */
	public boolean isDocChangeUnsaved() {
		return this.docChangeUnsaved;
	}

	//----------------------------------------------------------------------------------------
	// classes: 
	
	// The Undo action
	private class UndoAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public UndoAction(UndoManager manager) {
	      this.manager = manager;
	    }
	    public void actionPerformed(ActionEvent evt) {
	    	
	    	try {
	  	    	if (manager.canUndo()) {
	  	        manager.undo();
	  	    } else {
		    	JOptionPane.showMessageDialog(frame,
		    			"Cannot undo.");
		    	TestLog.e(LockFrameEditor.class, "LockFrameCalendar: undo action failed");
		    	}
	  	    } catch (CannotUndoException e) {
	  	        Toolkit.getDefaultToolkit().beep();
	  	    }
	    	
	    }
	    private UndoManager manager;
	}
	// The Redo action
	private class RedoAction extends AbstractAction {
		private static final long serialVersionUID = 2L;

		public RedoAction(UndoManager manager) {
	      this.manager = manager;
	    }
	    public void actionPerformed(ActionEvent evt) {
	      try {
	    	  if (manager.canRedo()){
	    		  manager.redo();
	    	  } else {
	    		  TestLog.e(LockFrameEditor.class, "LockFrameCalendar: redo action failed");
	    	  }
	      } catch (CannotRedoException e) {
	        Toolkit.getDefaultToolkit().beep();
	      }
	    }
	    private UndoManager manager;
	}
	
	/**
	 * Keyboard accessibility: 
	 * Return to the current text if ESC is pressed.
	 */
	private class ReturnToTextAction extends AbstractAction {
	    public void actionPerformed(ActionEvent evt) {
	    	//TestLog.e(getClass(), "ESC");
	    	if (lastFocusTab != null && lastFocusTab.isShowing()) {
	    		//TestLog.e(getClass(), "tab");
	    		if (lastFocusTab.getTextPane() != null) {
	    			JTextPane tp = lastFocusTab.getTextPane();
	    			tp.requestFocusInWindow();
	    			tp.setCaretPosition(tp.getSelectionEnd());
	    		} else {
		    		TestLog.ve(getClass(), "no text pane found");
		    	}
	    	} else {
	    		TestLog.ve(getClass(), "no tab found");
	    	}
	    }
	}
	
	@Override
	public String[] getCurrentFileNames() {
		ArrayList<HashSet<String>> setList = NotebookModel.getSetList();
		ArrayList<String> returnList = new ArrayList<String>();
		for (HashSet<String> set : setList) {
			for (String fileName : set) {
				returnList.add(fileName);
			}
		}
		return Converter.arrayListToArray(returnList);
		//return Converter.hashSetToArray(getCurrentTab().getFileNames());
	}
	@Override
	public void setMessage(String message, boolean isError) {
		if(isError == true) {
			messageLabel.setForeground(Color.red);
			messageLabel.setText(message);			
		} else {
			messageLabel.setForeground(Color.black);
			messageLabel.setText(message);
		}
	}
	@Override
	public final void resetFrame() {
		Point loc = frame.getLocationOnScreen();
		Dimension dim = frame.getSize();
		frame.dispose();
		frame = new LockFrameEditor(dialog, loc);
		frame.setSize(dim);		
		frame.setVisible(true);

		if (JREProperties.getMainWindow() instanceof PeaLockFrame) {
			JREProperties.setMainWindow(frame);
		}
	}

	@Override
	public String saveCurrentContentInCloudTempFile(String providerName, String fileNameToStore) {
		
		String orig = null;
		if ( ! fileNameToStore.contains(CloudControl.getTmpDir())) {
			orig = fileNameToStore;
		} 

		String tmpFileName = CloudControl.getTmpFileNameForCloudFile(new File(fileNameToStore).getName(), providerName, orig);
		//System.getProperty("java.io.tmpdir") 
		//+ File.separator + providerName + File.separator + fileNameToStore;
		File file = new File(tmpFileName);
		if (file.exists() == false) {
			try {
				file.getParentFile().mkdirs();
				file.createNewFile();
				TestLog.v(this.getClass(), "Temp file created for cloud " + providerName + ": " + file.getAbsolutePath());
			} catch (IOException e) {
				TestLog.e(LockFrameEditor.class, fileNameToStore + " - " + new ExceptionHelper(e).getInfos());
				return PeaProperties.getVmBridge().translate("error") + "\nCreate tmp file failed";
			}
		}
		
		if (docChangeUnsaved == false) {
			// copy ciphertext from existing file whether backup of cloud file or not
			byte[] cipherText = ReadResources.readExternFile(fileNameToStore);

			if (cipherText == null) {
				return PeaProperties.getVmBridge().translate("error") + "\n(read ciphertext)";
			}

			boolean success = ContentUtilities.saveContent(null, 
					null, cipherText,
					tmpFileName, true);
			if (success == false) {
				TestLog.e(LockFrameEditor.class, fileNameToStore + " - " + "Save calendar in temp file failed");
				return PeaProperties.getVmBridge().translate("error") + "\n(save ciphertext)";
			} else {
				//TestLog.v(this.getClass(), "Calendars saved in temp file " + tmpFileName);
				return null;
			}

		} else {
			byte[] plainBytes = ((LockFrameEditor) PeaControl.getDialog().getLockFrame()).getPlainBytes();
			// encrypt the calendars by session key and store in file
			boolean success = ContentUtilities.saveContent(CipherStuff.getSessionKeyCrypt().getKey(), 
					plainBytes, null,
					tmpFileName, true);
			if (success == false) {
				TestLog.e(LockFrameEditor.class, fileNameToStore + " - " + "Save calendar in temp file failed");
				return PeaProperties.getVmBridge().translate("error") + "\n" + CipherStuff.getErrorMessage();
			} else {
				//TestLog.v(this.getClass(), "Calendars saved in temp file " + tmpFileName);
				return null;
			}
		}

	}
	@Override
	public void setCurrentFileNames(String[] fileNameToSet) {
		new UnexpectedValueException("function","setCurrentFileNames","is not implemented").printDescription();
	}
	@Override
	public void updateModelWithCloudFiles(File[] files, String providerName) {

		for (File f : files) {
			//String fileNameWithoutPath = f.getName();
			String tmpFileName = CloudControl.getTmpFileNameForCloudFile(f.getName(), providerName, f.getAbsolutePath());
			//boolean added = 
			NotebookModel.addFileNameToSet(tmpFileName,f.getAbsolutePath());
		}
	}
	@Override
	public String decryptAndLoadFileToView(String fileName) {
		
		if (fileName == null) {
			new UnexpectedValueException("String", "fileName", "is null").printDescription();
		}
		boolean newTab = false;
		HashSet<String> tabFileNames = NotebookModel.getFileNameSet(fileName);
				//NotebookModel.getFileNames(fileName);
		if (tabFileNames == null || tabFileNames.isEmpty()) { // Download file from cloud
			//new UnexpectedValueException("HashSet<String>", "tabFileNames", "is null or empty for: "+ fileName).printDescription();
			HashSet<String> set = new HashSet<String>();
			set.add(fileName);
			NotebookModel.addFileNameSet(set);
			tabFileNames = set;
			newTab = true;
			// add to CloudFileModel:
			if (fileName.contains(CloudControl.getTmpDir())) {
				dialog.getEncryptedFileTypePanel().getCloudFilePanel().getCloudControl().getCloudFileModel().addUsedFileToProvider(
						CloudControl.getProviderNameFromTmpFile(fileName), 
						CloudControl.getFileNameFromTmpFile(fileName));
			}  			
		}
		// create tab
		createNewTab(tabFileNames);
		// set focus on new tab
		if (newTab == true) {
			Component c = tabPane.getComponent(tabPane.getComponentCount() -1);
			if (c instanceof TabComponent) {
				tabPane.setSelectedComponent(c);
				//changeFocus((TabComponent) c);
			}
		}
		return null;// no error occurred
	}
	
	/**
	 * Get the number of Tabs shown in the JTabbedPane.
	 * 
	 * @return the number of shown tabs
	 */
/*	protected int getNumberOfTabs() {
		return tabPane.getTabCount();
	} */
	
	/**
	 * Get the current JTabbedPane that holds the tabs with texts
	 * 
	 * @return	the JTabbedPane
	 */
	protected static JTabbedPane getTabPane() {
		return tabPane;
	}	

	@Override
	public String saveCurrentContent() {
		try {
			HashSet<String> tabSet = lastFocusTab.getFileNames();
			byte[] plainBytes = getPlainBytes();
			if (plainBytes == null) {
				TestLog.e(LockFrameEditor.class, "No plaintext found ");
				TestLog.ve(LockFrameEditor.class, "No plaintext found ", 5);
				return PeaProperties.getVmBridge().translate("unexpected_error") + " (no plaintext)";
			}
			byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, // is zeroized
					CipherStuff.getSessionKeyCrypt().getKey(), // keyMaterial is overwritten
					true, null ); // key is stored encrypted
			if (cipherBytes == null) {
				TestLog.e(LockFrameEditor.class, "No ciphertext found ");
				TestLog.ve(LockFrameEditor.class, "No ciphertext found ", 5);
				return PeaProperties.getVmBridge().translate("decryption_failed");
			}
			boolean cloudFileToSave = false;
			for (String nameInTab : tabSet) {					
				boolean success = ContentUtilities.saveContent(null, null, cipherBytes, nameInTab, true);
				if (success == false) {
					TestLog.e(LockFrameEditor.class, "Saving content failed ");
					TestLog.ve(LockFrameEditor.class, "Saving content failed " , 5);
				}
				if (nameInTab.contains(CloudControl.getTmpDir())) {
					cloudFileToSave = true;
				}
			}
			// if cloud files: wait until all UploadWorkers started
			// and set running process....
			if (cloudFileToSave == true) {
				// wait a short time, because all UploadWorkers
				// must have initialized and set running processes
				try {
					Thread.sleep(50);
				} catch (InterruptedException e) {
					//e.printStackTrace();
				}
			}
			docChangeUnsaved = false;
		} catch (Exception e) {
			TestLog.e(LockFrameEditor.class, "Saving text failed: " + e.getLocalizedMessage() + " - " + e.toString());
			TestLog.ve(LockFrameEditor.class, "Saving text failed: " + e.toString(),  5);
			return e.getLocalizedMessage();
		}

		this.setDocChangeUnsaved(false);
		return null;
	}
	
	@Override
	public String[] chooseFileNamesToApply() {
		// TODO use to select files for password/algo change
		return null;
	}
	
	@Override
	public boolean changePassword() {
		
		if (checkUnsavedContentChanges() == true) {
			setDocChangeUnsaved(false);
		}
		// TODO MultiFileSelection see ImageLockFrame
		String changeError = changePasswordOrApplyChanges(this, null);
		if (changeError != null) {
			PeaDialog.showMessage(this, changeError, 
					PeaProperties.getVmBridge().translate("error"), 0);
			return false;
		} else {
			TestLog.o(LockFrameEditorMenu.class, "Successfully changed password/algorithms for current file "
					+ LockFrameEditor.getCurrentTab().getFileNameWithoutPath() );
			return true;
		}
	}
	
	@Override
	public void focusGained(FocusEvent e) {
		//TestLog.v(getClass(), "FOCUS: " + e.getSource().getClass().getSimpleName());
		if (e.getSource() instanceof JButton) {
			JButton b = (JButton) e.getSource();
			b.setBorder(new LineBorder(Color.BLACK, 2));			
		}
	}
	
	@Override
	public void focusLost(FocusEvent e) {
		if (e.getSource() instanceof JButton) {
			JButton b = (JButton) e.getSource();
			b.setBorder(new LineBorder(Color.LIGHT_GRAY));
		}
	}
	@Override
	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		if (command.equals("H1")) {
			insertHeader(1);
		} else if (command.equals("H2")) {
			insertHeader(2);
		} else if (command.equals("H3")) {
			insertHeader(3);
		} else if (command.equals("H4")) {
			insertHeader(4);
		} else if (command.equals("H5")) {
			insertHeader(5);
		} else if (command.equals("H6")) {
			insertHeader(6);
		} else {
			TestLog.e(LockFrameEditor.class, "invalid command: " + command);
		}
	}
	/**
	 * Add a header at the caret position
	 * 
	 * @param hierarchy	the header hierarchy (h1 - H6)
	 */
	private void insertHeader(int hierarchy) {
		String header = PeaDialog.showInputDialog(JREProperties.getMainWindow(), 
				PeaProperties.getVmBridge().translate("type") + ": " 
						+ PeaProperties.getVmBridge().translate("header") + " " + hierarchy, 
				null, -1);
				//JOptionPane.showInputDialog("Type header text H" + hierarchy); // TODO translate
		if (header != null){
			JTextPane textPane = getFocusedTextPane();
			HTMLDocument doc = (HTMLDocument)textPane.getDocument();
			HTMLEditorKit editorKit = (HTMLEditorKit)textPane.getEditorKit();
			String text = "<H" + hierarchy + ">" + header + "</H" + hierarchy + ">";
			
			try {
				editorKit.insertHTML(doc, textPane.getCaretPosition(), //doc.getLength(), 
						text, 0, 0, null);
				textPane.requestFocus();
				//textPane.setCaretPosition(textPane.getCaretPosition());

			} catch (BadLocationException e) {
				TestLog.e(LockFrameEditor.class, e.getLocalizedMessage() + " - " + e.toString());
			} catch (IOException e) {
				TestLog.e(LockFrameEditor.class, e.getLocalizedMessage() + " - " + e.toString());
			}
		}
	}

	/**
	 * @param isBookmarkFile the isBookmarkFile to set
	 */
	public static void setBookmarkFile(boolean isBookmarkFile) {
		LockFrameEditor.isBookmarkFile = isBookmarkFile;
	}	
}