package cologne.eck.note_pea;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;


/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Menu of main frame of editor pea.  
 */


import java.awt.event.*;
import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;

import javax.swing.Action;
import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JTextPane;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.StyledEditorKit;


import cologne.eck.all_peas.cloud.gui.CloudMenu;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.PasswordGeneratorDialog;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.menu.HelpMenu;
import cologne.eck.all_peas.gui.menu.PeaMenuBar;
import cologne.eck.all_peas.gui.menu.SettingMenu;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;

@SuppressWarnings("serial")
final class LockFrameEditorMenu extends PeaMenuBar implements ActionListener {

	// private static ResourceBundle PeaProperties.getBundle();
	private static LockFrameEditor lockFrame;// = (JFrame) PeaControl.getDialog().getLockFrame();

	private static CloudMenu cloudMenu;
	
	protected static JMenuItem activateLinksItem;
	
	public LockFrameEditorMenu(LockFrameEditor _lockFrame) {
		
		PswDialogView pdv = PswDialogView.getInstance();

		if (_lockFrame == null) {
			lockFrame = (LockFrameEditor) PeaControl.getDialog().getLockFrame();
			if (lockFrame == null) {
				TestLog.ve(this.getClass(), "Missing lockFrame ", 5);
			}
		} else {
			lockFrame = _lockFrame;
		}
		int fontSize = PeaProperties.getFontSize();//PswDialogView.getFontsize();
		Font buttonFont = new Font(Font.SANS_SERIF, Font.PLAIN, fontSize);

		JMenu fileMenu = new JMenu(PeaProperties.getVmBridge().translate("file"));// File");//\u00FC");
		fileMenu.setFont(buttonFont);
		// fileMenu.setMnemonic(KeyEvent.VK_F);
		fileMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens menu to file operations: save, open, change password, create random password,quit");
		fileMenu.setMnemonic(pdv.getMnemo(this, PeaProperties.getVmBridge().translate("file")));
		this.add(fileMenu);

		JMenuItem saveItem = new JMenuItem(PeaProperties.getVmBridge().translate("save_encrypted"));// save (encrypted)");
		saveItem.setActionCommand("save");
		saveItem.setFont(buttonFont);
		saveItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component saves the current document.");
		//saveItem.setMnemonic(pdv.getMnemo(fileMenu, PeaProperties.getVmBridge().translate("save_encrypted")));		
		saveItem.addActionListener(this);
		fileMenu.add(saveItem);
		
		JMenu convertMenu = new JMenu(PeaProperties.getVmBridge().translate("convert"));// save (encrypted)");
		convertMenu.setFont(buttonFont);
		convertMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component allows to convert the current document.");
		//convertMenu.setMnemonic(pdv.getMnemo(fileMenu, PeaProperties.getVmBridge().translate("convert")));	
		fileMenu.add(convertMenu);
		

		JMenuItem toHtmlItem = new JMenuItem(" " + '\u2794' + "  HTML " + PeaProperties.getVmBridge().translate("format"));
		toHtmlItem.setActionCommand("toHtml");// U+279E
		toHtmlItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component converts the current document in HTML format.");
		toHtmlItem.setFont(buttonFont);
		//toHtmlItem.setMnemonic(pdv.getMnemo(convertMenu, "HTML"));	
		toHtmlItem.addActionListener(this);
		convertMenu.add(toHtmlItem);
		
		JMenuItem toRtfItem = new JMenuItem(" " + '\u2794' + "  RTF " + PeaProperties.getVmBridge().translate("format"));
		toRtfItem.setActionCommand("toRtf");// U+279E
		toRtfItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component converts the current document in RTF format.");
		toRtfItem.setFont(buttonFont);
		//toRtfItem.setMnemonic(pdv.getMnemo(convertMenu, "RTF"));	
		toRtfItem.addActionListener(this);
		convertMenu.add(toRtfItem);
		
		JLabel convertHint = new JLabel("   " + PeaProperties.getVmBridge().translate("formatting_get_lost"));
		convertHint.setFont(buttonFont);
		convertMenu.add(convertHint);

		JMenuItem exportItem = new JMenuItem(PeaProperties.getVmBridge().translate("save_as"));// "Save as...");
		exportItem.setActionCommand("export");
		exportItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component saves the document with custom name.");
		exportItem.setFont(buttonFont);
		//exportItem.setMnemonic(pdv.getMnemo(fileMenu, PeaProperties.getVmBridge().translate("save_as")));	
		exportItem.addActionListener(this);
		fileMenu.add(exportItem);

		JMenuItem importItem = new JMenuItem(PeaProperties.getVmBridge().translate("import_file"));// "open file (txt, rtf or lock file)");
		importItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component imports another text file.");
		importItem.setActionCommand("import");
		importItem.setFont(buttonFont);
		//importItem.setMnemonic(pdv.getMnemo(fileMenu, PeaProperties.getVmBridge().translate("import_file")));					
		importItem.addActionListener(this);
		fileMenu.add(importItem);
		
		JMenuItem newItem = new JMenuItem(PeaProperties.getVmBridge().translate("init_new_file"));
		newItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to initialize a new text file.");
		newItem.setActionCommand("initNew");
		newItem.setFont(buttonFont);
		//newItem.setMnemonic(pdv.getMnemo(fileMenu, PeaProperties.getVmBridge().translate("init_new_file")));					
		newItem.addActionListener(this);
		fileMenu.add(newItem);
		
		
		JMenuItem openEncryptedItem = new JMenuItem(PeaProperties.getVmBridge().translate("open_encrypted_file"));
		openEncryptedItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to open an encrypted text file.");
		openEncryptedItem.setActionCommand("openEncrypted");
		openEncryptedItem.setFont(buttonFont);
		//openEncryptedItem.setMnemonic(pdv.getMnemo(fileMenu, PeaProperties.getVmBridge().translate("open_encrypted_file")));					
		openEncryptedItem.addActionListener(this);
		fileMenu.add(openEncryptedItem);

		JMenuItem randomPswItem = new JMenuItem(PeaProperties.getVmBridge().translate("generate_random_password"));// "Generate
		randomPswItem.getAccessibleContext().
		setAccessibleDescription(
				"Clicking this component creates a radom password in a text field.");																										
		//randomPswItem.setMnemonic(pdv.getMnemo(fileMenu, PeaProperties.getVmBridge().translate("generate_random_password")));						
		randomPswItem.addActionListener(this);
		randomPswItem.setActionCommand("randomPsw");
		fileMenu.add(randomPswItem);

		JMenuItem quitItem = new JMenuItem(PeaProperties.getVmBridge().translate("quit"));// "Quit");
		quitItem.setActionCommand("quit");
		quitItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component exits the program.");
		quitItem.setFont(buttonFont);
		//quitItem.setMnemonic(pdv.getMnemo(fileMenu, PeaProperties.getVmBridge().translate("quit")));	
		quitItem.addActionListener(this);
		fileMenu.add(quitItem);

		JMenu editMenu = new JMenu(PeaProperties.getVmBridge().translate("edit"));// "Edit");
		editMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens edit menu: redo, undo, cut, copy paste, select and default settings.");
		editMenu.setFont(buttonFont);
		editMenu.setMnemonic(pdv.getMnemo(this, PeaProperties.getVmBridge().translate("edit")));	
		this.add(editMenu);

		JMenuItem undoItem = new JMenuItem(PeaProperties.getVmBridge().translate("undo"));// "Undo");
		undoItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component causes undoing the last action.");
		undoItem.setActionCommand("undo");
		undoItem.setFont(buttonFont);
		//undoItem.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("undo")));	
		undoItem.addActionListener(this);
		editMenu.add(undoItem);

		JMenuItem redoItem = new JMenuItem(PeaProperties.getVmBridge().translate("redo"));// "Redo");
		redoItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component causes redoing the last action.");
		redoItem.setActionCommand("redo");
		redoItem.setFont(buttonFont);
		//redoItem.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("redo")));	
		redoItem.addActionListener(this);
		editMenu.add(redoItem);
		

		JMenuItem searchItem = new JMenuItem(PeaProperties.getVmBridge().translate("search"));// "Redo");
		searchItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to insert a string to search for.");
		searchItem.setActionCommand("search");
		searchItem.setFont(buttonFont);
		//searchItem.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("search")));	
		searchItem.addActionListener(this);
		editMenu.add(searchItem);

		JMenuItem cutItem = new JMenuItem(PeaProperties.getVmBridge().translate("cut"));// "Cut");
		cutItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component cuts selected text.");
		cutItem.setActionCommand("cut");
		cutItem.setFont(buttonFont);
		//cutItem.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("cut")));	
		cutItem.addActionListener(this);
		editMenu.add(cutItem);

		JMenuItem copyItem = new JMenuItem(PeaProperties.getVmBridge().translate("copy"));// "Copy");
		copyItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component copies selected text.");
		copyItem.setActionCommand("copy");
		copyItem.setFont(buttonFont);
		//copyItem.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("copy")));	
		copyItem.addActionListener(this);
		editMenu.add(copyItem);

		JMenuItem pasteItem = new JMenuItem(PeaProperties.getVmBridge().translate("paste"));// "Paste");
		pasteItem.setActionCommand("paste");
		pasteItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component pastes text from clipboard.");
		pasteItem.setFont(buttonFont);
		//pasteItem.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("paste")));	
		pasteItem.addActionListener(this);
		editMenu.add(pasteItem);

		JMenuItem selectAllItem = new JMenuItem(PeaProperties.getVmBridge().translate("select_all"));// "Select All");
		selectAllItem.setActionCommand("selectAll");
		selectAllItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects all of the text.");
		selectAllItem.setFont(buttonFont);
		//selectAllItem.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("select_all")));	
		selectAllItem.addActionListener(this);
		editMenu.add(selectAllItem);
				
		activateLinksItem = new JMenuItem(PeaProperties.getVmBridge().translate("make_links_clickable"));// "Select All");
		activateLinksItem.setActionCommand("makeLinksClickable");
		activateLinksItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component activates or deactivates links in HTML text - text is then not editable.");
		activateLinksItem.setFont(buttonFont);
		//activateLinksItem.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("activate_links")));	
		activateLinksItem.addActionListener(this);
		editMenu.add(activateLinksItem);
		
		JMenu settingMenu = new JMenu(PeaProperties.getVmBridge().translate("text_setting"));
		settingMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens menu to set default font color, size and family and background color.");
		settingMenu.setFont(buttonFont);
		settingMenu.setMnemonic(pdv.getMnemo(editMenu, PeaProperties.getVmBridge().translate("text_setting")));	
		editMenu.add(settingMenu);
		
		JMenuItem defaultColorItem = new JMenuItem(PeaProperties.getVmBridge().translate("default_font_color"));// "Select All");
		defaultColorItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to change the default color of the font.");
		defaultColorItem.setActionCommand("defaultFontColor");
		defaultColorItem.setFont(buttonFont);
		defaultColorItem.addActionListener(this);
		//defaultColorItem.setMnemonic(pdv.getMnemo(settingMenu, PeaProperties.getVmBridge().translate("default_font_color")));	
		settingMenu.add(defaultColorItem);		
		
		JMenuItem defaultBackgroundItem = new JMenuItem(PeaProperties.getVmBridge().translate("default_background"));// "Select All");
		defaultBackgroundItem.setActionCommand("defaultBackground");
		defaultBackgroundItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens dialog to change the background of the text area.");
		defaultBackgroundItem.setFont(buttonFont);
		defaultBackgroundItem.addActionListener(this);
		//defaultBackgroundItem.setMnemonic(pdv.getMnemo(settingMenu, PeaProperties.getVmBridge().translate("default_background")));	
		settingMenu.add(defaultBackgroundItem);	
		
		JMenu defaultFamilyItem = new JMenu(PeaProperties.getVmBridge().translate("default_font_family"));// "Select All");
		defaultFamilyItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the default font family: serif, sans serif, mono.");
		defaultFamilyItem.setFont(buttonFont);
		//defaultFamilyItem.setMnemonic(pdv.getMnemo(settingMenu, PeaProperties.getVmBridge().translate("default_font_family")));	
		settingMenu.add(defaultFamilyItem);
		
		String fontNames[] = { "SERIF", "SANS_SERIF", "MONOSPACED" };
		for (String font : fontNames) {
			JMenuItem familyItem = new JMenuItem(font);
			familyItem.setFont(buttonFont);
			familyItem.setActionCommand("defaultFontFamily" + font);
			familyItem.addActionListener(this);
			defaultFamilyItem.add(familyItem);
		}
		
		JMenu defaultSizeItem = new JMenu(PeaProperties.getVmBridge().translate("default_font_size"));// "Select All");
		defaultSizeItem.setFont(buttonFont);
		defaultSizeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the font size from 6 to 24.");
		//defaultSizeItem.setMnemonic(pdv.getMnemo(settingMenu, PeaProperties.getVmBridge().translate("default_font_size")));	
		settingMenu.add(defaultSizeItem);	

		for (int i = 6; i < 25; i++) {
			JMenuItem sizeItem = new JMenuItem("" + i);
			sizeItem.setFont(buttonFont);
			sizeItem.setActionCommand("defaultFontSize" + i);
			sizeItem.addActionListener(this);
			defaultSizeItem.add(sizeItem);
		}	
		
		
		JMenu formatMenu = new JMenu(PeaProperties.getVmBridge().translate("format"));// "Format");
		formatMenu.setFont(buttonFont);
		formatMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens menu to change the format of a selected text.");
		formatMenu.setMnemonic(pdv.getMnemo(this, PeaProperties.getVmBridge().translate("format")));	
		this.add(formatMenu);

		JMenu fontMenu = new JMenu(PeaProperties.getVmBridge().translate("font"));// "Font");
		fontMenu.setFont(buttonFont);
		fontMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the font family: serif, sans serif, mono.");
		//fontMenu.setMnemonic(pdv.getMnemo(formatMenu, PeaProperties.getVmBridge().translate("font")));	
		formatMenu.add(fontMenu);

		//String fontNames[] = { "SERIF", "SANS_SERIF", "MONOSPACED" };
		for (String font : fontNames) {
			fontMenu.add(new StyledEditorKit.FontFamilyAction(font, font));
		}

		JMenu fontStyleMenu = new JMenu(PeaProperties.getVmBridge().translate("font_style"));// "Font style");
		fontStyleMenu.setFont(buttonFont);
		fontStyleMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the font style: bold,italic, underline.");
		//fontStyleMenu.setMnemonic(pdv.getMnemo(formatMenu, PeaProperties.getVmBridge().translate("font_style")));	
		formatMenu.add(fontStyleMenu);

		Action action = new StyledEditorKit.BoldAction();
		action.putValue(Action.NAME, PeaProperties.getVmBridge().translate("bold"));// "Bold");
		fontStyleMenu.add(action);

		action = new StyledEditorKit.ItalicAction();
		action.putValue(Action.NAME, PeaProperties.getVmBridge().translate("italic"));// "Italic");
		fontStyleMenu.add(action);

		action = new StyledEditorKit.UnderlineAction();
		action.putValue(Action.NAME, PeaProperties.getVmBridge().translate("underline"));// "Underline");
		fontStyleMenu.add(action);

		JMenu alignmentMenu = new JMenu(PeaProperties.getVmBridge().translate("alignment"));// "Font style");
		alignmentMenu.setFont(buttonFont);
		alignmentMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to change the text alignment: left, center, right.");
		//alignmentMenu.setMnemonic(pdv.getMnemo(formatMenu, PeaProperties.getVmBridge().translate("alignment")));	
		formatMenu.add(alignmentMenu);

		Action alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getVmBridge().translate("left"), 0);
		alignmentMenu.add(alignmentAction);

		alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getVmBridge().translate("center"), 1);
		alignmentMenu.add(alignmentAction);

		alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getVmBridge().translate("right"), 2);
		alignmentMenu.add(alignmentAction);

		JMenu fontSizeMenu = new JMenu(PeaProperties.getVmBridge().translate("font_size"));// "Font size");
		fontSizeMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the font size from 6 to 32.");
		fontSizeMenu.setFont(buttonFont);
		//fontSizeMenu.setMnemonic(pdv.getMnemo(formatMenu, PeaProperties.getVmBridge().translate("font_size")));	
		formatMenu.add(fontSizeMenu);

		for (int i = 6; i < 33; i++) {
			fontSizeMenu.add(new StyledEditorKit.FontSizeAction("" + i, i));
		}

		JMenuItem fontColorMenu = new JMenuItem(PeaProperties.getVmBridge().translate("font_color"));
		fontColorMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to choose the font color for a selected text.");
		fontColorMenu.setFont(buttonFont);
		//fontColorMenu.setMnemonic(pdv.getMnemo(formatMenu, PeaProperties.getVmBridge().translate("font_color")));	
		fontColorMenu.addActionListener(this);
		fontColorMenu.setActionCommand("fontColor");
		formatMenu.add(fontColorMenu);
		
		cloudMenu = new CloudMenu(pdv.getMnemo(this, PeaProperties.getVmBridge().translate("cloud")));
		//cloudMenu.setMnemonic(pdv.getMnemo(this, PeaProperties.getVmBridge().translate("cloud")));	
		this.add(cloudMenu);
		
		JMenu algoSettingMenu = new SettingMenu(false, pdv.getMnemo(this, PeaProperties.getVmBridge().translate("settings")));// decrpyted mode
		//algoSettingMenu.setMnemonic(pdv.getMnemo(this, PeaProperties.getVmBridge().translate("settings")));	
		this.add(algoSettingMenu);

		JMenu helpMenu = new HelpMenu(pdv.getMnemo(this, PeaProperties.getVmBridge().translate("help")));		
		//helpMenu.setMnemonic(pdv.getMnemo(this, PeaProperties.getVmBridge().translate("help")));	
		this.add(helpMenu);
		

		this.setMaximumSize(this.getPreferredSize());
		
	}

	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();

		if (command.equals("save")) { // saves current content in current file
			String saveError = lockFrame.saveCurrentContent();
			if (saveError != null) {
				PeaDialog.showMessage(lockFrame, saveError, PeaProperties.getVmBridge().translate("error"), 0);
			}

		} else if (command.equals("export")) {
			
			String error = TextHelper.export(this);
			if (error != null) {
				PeaDialog.showMessage(this, error, PeaProperties.getVmBridge().translate("error"), 0);
			}



		} else if (command.equals("import")) { // current content get lost if not saved

			String error = TextHelper.importFile(this, lockFrame);
			if (error != null) {
				PeaDialog.showMessage(this, error, PeaProperties.getVmBridge().translate("error"), 0);
			}
			
		} else if (command.equals("initNew")) { // initialize new file
			
			// to check success of file creation
			int oldListSize = NotebookModel.getSetList().size();

			// initializeNewFiles() uses defaultAlgoParamString:
			// save default algos to reset and set current algos to default
			String originalAlgoParamString = PeaProperties.getDefaultAlgoParamString();
			PeaProperties.setDefaultAlgoParamString(new String(PeaProperties.getAlgoParamString(), AttachmentHandler.getASCIICharset()));
			PeaControl.getDialog().initializeNewFiles();
			// reset default algos
			PeaProperties.setDefaultAlgoParamString(originalAlgoParamString);
			
			// open the new created file
			ArrayList<HashSet<String>> newList = NotebookModel.getSetList();
			if (oldListSize >= newList.size()) {
				TestLog.v(getClass(), "No new file was created...");
				return;
			}
			// open the new file: This is the last in the list
			HashSet<String> newSet = newList.get(newList.size() -1);
			// create tab:
			lockFrame.createNewTab(newSet);
			// Get tab: last component, set focus on this tab
			if (LockFrameEditor.getTabPane() != null 
					&& LockFrameEditor.getTabPane().getComponentCount() > 0) {
				Component c = LockFrameEditor.getTabPane().getComponent(LockFrameEditor.getTabPane().getComponentCount() -1);
				if (c instanceof TabComponent) {
					LockFrameEditor.getTabPane().setSelectedComponent(c);
					//changeFocus((TabComponent) c);
				} else {
					TestLog.ve(getClass(), "No tab found...", 5);
				}
			} else {
				TestLog.ve(getClass(), "Missing tab...", 5);
			}
			
		} else if (command.equals("openEncrypted")) { // open encrypted file
			TestLog.v(getClass(), "Open encrypted file...");
			
			PeaFileChooser pfc = new PeaFileChooser(this);
			pfc.setFileSelectionMode(JFileChooser.FILES_ONLY);
			pfc.setMultiSelectionEnabled(false);
			pfc.setFileFilter(new FileNameExtensionFilter("HTML and RTF files", "rtf", "RTF", "HTML", "html", "htm") );
			pfc.customizeFileChooser(
		    		PeaProperties.getVmBridge().translate("open_file"),
		    		PeaProperties.getVmBridge().translate("open_encrypted_file"), 
		    		false);// no hidden files option
			int returnVal = pfc.showOpenDialog(this);
		    if(returnVal == JFileChooser.APPROVE_OPTION) {
		       System.out.println("You chose to open this file: " +
		            pfc.getSelectedFile().getName());
		       File selectedFile = pfc.getSelectedFile();
		       // check if file is already open:
				if (NotebookModel.fileNameExists(selectedFile.getAbsolutePath()) == true) {
					TestLog.e(LockFrameEditorMenu.class, "File to open is already listed: " + selectedFile.getAbsolutePath());
					PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("file_already_open"), 
							PeaProperties.getVmBridge().translate("error"), 0);
					return;
				} 
				// Create a list
				HashSet<String> newSet = new HashSet<String>();
				newSet.add(selectedFile.getAbsolutePath());
				// create the tab:
				lockFrame.createNewTab(newSet);
				NotebookModel.addFileNameSet(newSet);
				// Get tab: last component, set focus on this tab
				if (LockFrameEditor.getTabPane() != null 
						&& LockFrameEditor.getTabPane().getComponentCount() > 0) {
					Component c = LockFrameEditor.getTabPane().getComponent(LockFrameEditor.getTabPane().getComponentCount() -1);
					if (c instanceof TabComponent) {
						LockFrameEditor.getTabPane().setSelectedComponent(c);
						//changeFocus((TabComponent) c);
					} else {
						TestLog.ve(getClass(), "No tab found...", 5);
					}
				} else {
					TestLog.ve(getClass(), "Missing tab...", 5);
				}
		    }
						
		} else if (command.equals("toHtml")) { // current content get lost if not saved

			TabComponent tab = LockFrameEditor.getCurrentTab();
			if (tab == null) {
				TestLog.ve(LockFrameEditor.class, "Missing tab", 5);
			}
			String error = TextHelper.convertInHTML(this, lockFrame);
			if (error != null) {
				PeaDialog.showMessage(this, error, PeaProperties.getVmBridge().translate("error"), 0);
			}
			
		} else if (command.equals("toRtf")) { // current content get lost if not saved

			TabComponent tab = LockFrameEditor.getCurrentTab();
			if (tab == null) {
				TestLog.ve(LockFrameEditor.class, "Missing tab", 5);
			}
			String error = TextHelper.convertInRTF(this, lockFrame);
			if (error != null) {
				PeaDialog.showMessage(this, error, PeaProperties.getVmBridge().translate("error"), 0);
			}

		} else if (command.equals("fontColor")) {

			Color color = JColorChooser.showDialog(this,
					// PeaControl.getDialog().getLockFrame(),
					PeaProperties.getVmBridge().translate("font_color"), Color.BLACK);

			if (color != null) {
				// TestLog.v(this.getClass(), color);
				// set color
				SimpleAttributeSet attr = new SimpleAttributeSet();
				StyleConstants.setForeground(attr, color);
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				textPane.setCharacterAttributes(attr, false);
				//LockFrameEditor.setDocChangeUnsaved(true);
			}

		} else if (command.equals("randomPsw")) {
			PasswordGeneratorDialog dialog = new PasswordGeneratorDialog(JREProperties.getMainWindow());
			dialog.setIconImage(PswDialogView.getImage());
			dialog.setLocationRelativeTo(this);
			dialog.setVisible(true);

		} else if (command.equals("quit")) {
			try {
				//((LockFrameEditor) PeaControl.getDialog().getLockFrame()).checkUnsavedTextChanges();
				((LockFrameEditor) PeaControl.getDialog().getLockFrame()).windowClosingCommands();

				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				StyledDocument doc = textPane.getStyledDocument();
				try {
					doc.remove(0, doc.getLength());
				} catch (BadLocationException e) {
					// don't show message...
					TestLog.e(LockFrameEditorMenu.class, "LockFrameEditorMenu (Quit): \n" + "Can't remove text from Textpane, \n"
							+ e.getMessage() + ", " + e.toString());
				}
				System.exit(0);
			} catch (Exception e) {
				int result = PeaDialog.showQuestion(this, e.getLocalizedMessage() 
						+ "\n   " + PeaProperties.getVmBridge().translate("quit") + "?", 
						PeaProperties.getVmBridge().translate("error"), 0);
				if (result == 0) {
					System.exit(0);
				}
			}
		} else if (command.equals("undo")) {
			if (LockFrameEditor.manager.canUndo()) {
				LockFrameEditor.manager.undo();
			}
		} else if (command.equals("redo")) {
			if (LockFrameEditor.manager.canRedo()) {
				LockFrameEditor.manager.redo();
			}
			
		} else if (command.equals("search")) {
			// open input dialog
			String searchText = PeaDialog.showInputDialog(this, 
					PeaProperties.getVmBridge().translate("search_for"), 
					PeaProperties.getVmBridge().translate("search"), -1);
			if (searchText == null || searchText.length() == 0) {
				return;
			}
			int searchTextLen = searchText.length();
			//TestLog.v(this.getClass(), "search for " + searchText);

			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			// remove highlighter from previous search
			textPane.getHighlighter().removeAllHighlights();
			Document doc = textPane.getDocument();
			
			String tabText = null;
			int docLen = doc.getLength();
			if (docLen == 0) {
				TestLog.e(LockFrameEditorMenu.class, "No text to search in...");
				return;
			}
			try {
				tabText = doc.getText(0, docLen);
			} catch (BadLocationException e1) {
				TestLog.e(LockFrameEditor.class, new ExceptionHelper(e1).getInfos());
				//e1.printStackTrace();
				return;
			}
			

			int pos= 0;
			int endPos = 0;
			DefaultHighlighter.DefaultHighlightPainter highlightPainter = 
			        new DefaultHighlighter.DefaultHighlightPainter(Color.YELLOW);
			// Note: The hightlight is removes by mouse click in TabComponent MouseListener
			while (endPos < docLen) {
				pos = tabText.indexOf(searchText, endPos);
				if (pos == -1) {
					break;
				}
				endPos = pos + searchTextLen;
				// highlight
				try {
					textPane.getHighlighter().addHighlight(pos, endPos, 
							highlightPainter);
				} catch (BadLocationException e) {
					TestLog.e(LockFrameEditor.class, new ExceptionHelper(e).getInfos());
					TestLog.e(LockFrameEditorMenu.class, "While searching for " + searchText);
				}
			}
			
		} else if (command.equals("cut")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.cut();
		} else if (command.equals("copy")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.copy();
		} else if (command.equals("paste")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.paste();			
			// Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
			// TransferHandler transferHandler =
			// LockFrameCalendar.textPane.getTransferHandler();
			// transferHandler.importData(LockFrameCalendar.textPane,
			// clipboard.getContents(null));
		} else if (command.equals("selectAll")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			if (textPane != null) {
				textPane.selectAll();
			}
			
		} else if (command.startsWith("makeLinksClickable")) { 
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			if (textPane != null) { // only for HTML
				if (textPane.isEditable() && textPane.getContentType().equals("text/html")) {
					textPane.setEditable(false);		
					//textPane.setFocusable(true);
					// change editable button:
					LockFrameEditor.setEditableButton.setText(
							PeaProperties.getVmBridge().translate("make_text_editable"));
					LockFrameEditor.setEditableButton.setActionCommand("makeTextEditable");
					// set menmonic: remove old mnemonic, set new
					if (LockFrameEditor.setEditableButton.getParent() != null
							&& LockFrameEditor.setEditableButton.getParent().getParent() != null) {
						// remove current mnemonic:
						PswDialogView.getView().removeMnemonic(
								LockFrameEditor.setEditableButton.getParent().getParent(), 
								(char) LockFrameEditor.setEditableButton.getMnemonic());
						// remove current mnemonic:
						LockFrameEditor.setEditableButton.setMnemonic(PswDialogView.getView().getMnemo(
								LockFrameEditor.setEditableButton.getParent().getParent(), 
								PeaProperties.getVmBridge().translate("make_text_editable")));
					}
					LockFrameEditor.setEditableButton.setVisible(true);
					
				} else {
					textPane.setEditable(true);
					LockFrameEditor.setEditableButton.setVisible(false);
				}
			}
		} else if (command.startsWith("makeTextEditable")) { 
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			if (textPane != null) { // only for HTML
				if (textPane.isEditable() == false && textPane.getContentType().equals("text/html")) {
					textPane.setEditable(true);				
					
					LockFrameEditor.setEditableButton.setText(
							PeaProperties.getVmBridge().translate("make_links_clickable"));
					LockFrameEditor.setEditableButton.setActionCommand("makeLinksClickable");
					// set menmonic:
					if (LockFrameEditor.setEditableButton.getParent() != null
							&& LockFrameEditor.setEditableButton.getParent().getParent() != null) {
						// remove current mnemonic:
						PswDialogView.getView().removeMnemonic(
								LockFrameEditor.setEditableButton.getParent().getParent(), 
								(char) LockFrameEditor.setEditableButton.getMnemonic());
						// set new mnemonic:
						LockFrameEditor.setEditableButton.setMnemonic(PswDialogView.getView().getMnemo(
								LockFrameEditor.setEditableButton.getParent().getParent(), 
								PeaProperties.getVmBridge().translate("make_links_clickable")));
					}
					LockFrameEditor.setEditableButton.setVisible(true);
					
				} else {
					textPane.setEditable(true);
					LockFrameEditor.setEditableButton.setVisible(false);
				}
			}
			
		} else if (command.startsWith("defaultFontFamily")) {
			String family = command.substring(17, command.length());
			if (family.equals("SERIF")) {
				NotebookSetting.setFontFamily(Font.SERIF);
			} else 	if (family.equals("SANS_SERIF")) {
				NotebookSetting.setFontFamily(Font.SANS_SERIF);
			} else 	if (family.equals("MONOSPACED")) {
				NotebookSetting.setFontFamily(Font.MONOSPACED);
			} else {
				new UnexpectedValueException("family", "String", "does not match font family constants").printDescription();
			}
			// Set the default settings: 
			MutableAttributeSet attributes = LockFrameEditor.getFocusedTextPane().getInputAttributes();
			StyleConstants.setFontFamily(attributes, NotebookSetting.getFontFamily());
	        // get the document
	        StyledDocument document = LockFrameEditor.getFocusedTextPane().getStyledDocument();
	        // Replace the style at end of document
	        document.setCharacterAttributes(document.getLength(), 1, attributes, true);
			
		} else if (command.startsWith("defaultFontSize")) {
			String sizeString = command.substring(15, command.length());
			int defaultSize = NotebookSetting.getFontSize();
			try {
				defaultSize = Integer.parseInt(sizeString);
			} catch (Exception e) {
				new UnexpectedValueException("sizeString", "String", "can not be parsed: " + sizeString).printDescription();
			}			
			NotebookSetting.setFontSize(defaultSize);
			// Set the default settings: 
			MutableAttributeSet attributes = LockFrameEditor.getFocusedTextPane().getInputAttributes();;
	        StyleConstants.setFontSize(attributes, NotebookSetting.getFontSize()); 
	        StyledDocument document = LockFrameEditor.getFocusedTextPane().getStyledDocument();
	        // Replace the style at end of document
	        document.setCharacterAttributes(document.getLength(), 1, attributes, true);
		} else if (command.equals("defaultFontColor")) {

			Color color = JColorChooser.showDialog(this,
			PeaProperties.getVmBridge().translate("default_font_color"), Color.BLACK);

			if (color != null) {
				NotebookSetting.setFontColor(color);
				MutableAttributeSet attributes = LockFrameEditor.getFocusedTextPane().getInputAttributes();
		        StyleConstants.setForeground(attributes, NotebookSetting.getFontColor());
		        // get the document
		        StyledDocument document = LockFrameEditor.getFocusedTextPane().getStyledDocument();
		        // Replace the style at end of document
		        document.setCharacterAttributes(document.getLength(), 1, attributes, true);
			}
		} else if (command.equals("defaultBackground")) {
			Color color = JColorChooser.showDialog(this,
					// PeaControl.getDialog().getLockFrame(),
					PeaProperties.getVmBridge().translate("default_background"), NotebookSetting.getTextBackground());

			if (color != null) {
				NotebookSetting.setTextBackground(color);

				UIManager.put("TextPane[Enabled].backgroundPainter", LockFrameEditor.getCurrentTab().new BackgroundPainter() );// {
		/*		    @Override
				    public void paint(Graphics2D g, JComponent comp, int width, int height) {
				        g.setColor(NotebookSetting.getTextBackground());
				        g.fillRect(0, 0, width, height);
				    }
				}); */
				UIDefaults defaults = new UIDefaults();
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				textPane.putClientProperty("Nimbus.Overrides", defaults);
				textPane.putClientProperty("Nimbus.Overrides.InheritDefaults", true);
				textPane.setBackground(NotebookSetting.getTextBackground());
			}
		} else {
			TestLog.e(LockFrameEditorMenu.class, "invalid command in editor menu: " + command);
		}
	}
	


/*	protected void setRootWindowForCloudMenu(Window win) {
		cloudMenu.setRooTWindow(win);
	}*/
	protected static void setLockFrame(LockFrameEditor lfe) {
		if (lfe != null) {
			lockFrame = lfe;
		}
	}
}