package cologne.eck.note_pea;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Color;
import java.awt.Font;
import java.util.Properties;

import cologne.eck.tools.UnexpectedValueException;


/*
 * This class contains variables and their Getters & Setters 
 * that might be changed by users
 */
class NotebookSetting {// implements ClipboardOwner {
	
	private static int fontStyle = Font.PLAIN;// BOLD, ITALIC
	
	private static String fontFamily = Font.SANS_SERIF;// SERIF, MONOSPACED
	
	private static int fontSize = 12;
	
	private static Color textBackground = new Color(231, 231, 231);//Color.WHITE;
	
	private static Color fontColor = Color.BLACK;

	// to check, if current clipboard content is set from this PEA
	private static boolean clipboardContentIsFromPEA = false;	
/*	static {
		
		Toolkit.getDefaultToolkit().getSystemClipboard().addFlavorListener(new FlavorListener() { 
            @Override 
            public void flavorsChanged(FlavorEvent e) { 

            //    clipboardContentIsFromPEA = false;
TestLog.v(NotebookSetting.class,  "clip changed");
            }
        }); 
	}*/
	
	/**
	 * Get the default font style from settings 
	 * (PLAIN = 0, BOLD = 1,ITALIC = 2)
	 * 
	 * @return	the default font style
	 */
	protected static int getFontStyle() {
		return fontStyle;
	}
	/**
	 * Set the default font style from settings 
	 * (PLAIN, BOLD,ITALIC)
	 * 
	 * @param fontStyle	the default font style
	 */
	protected static void setFontStyle(int newFontStyle) {
		if (newFontStyle >= 0 && newFontStyle <= 2) {
			NotebookSetting.fontStyle = newFontStyle;
		} else {
			new UnexpectedValueException("newFontStyle", "int", "does not match styles (0-2): " + newFontStyle).printDescription();
		}
	}

	/**
	 * Get the default font family from settings 
	 * (SANS_SERIF, SERIF, MONOSPACED)
	 * 
	 * @return	the default font family
	 */
	protected static String getFontFamily() {
		return fontFamily;
	}
	/**
	 * Set the default font family from settings 
	 * (SANS_SERIF, SERIF, MONOSPACED)
	 * 
	 * @param fontFamily	the default font family
	 */
	protected static void setFontFamily(String newFontFamily) {
		if (newFontFamily.equals(Font.SANS_SERIF)
				|| newFontFamily.equals(Font.SERIF)
				|| newFontFamily.equals(Font.MONOSPACED)) {
			NotebookSetting.fontFamily = newFontFamily;
			EditorControl.getDialog().updatePeaPropValues(new Properties());
		} else {
			new UnexpectedValueException("newFontFamily", "String", "does not match families: " + newFontFamily).printDescription();
		}
	}

	/**
	 * Get the default font size from settings 
	 * 
	 * @return	the default font size
	 */
	protected static int getFontSize() {
		return fontSize;
	}
	/**
	 * Set the default font size from settings 
	 * 
	 * @param fontSize	the default font size
	 */
	protected static void setFontSize(int fontSize) {
		NotebookSetting.fontSize = fontSize;
		EditorControl.getDialog().updatePeaPropValues(new Properties());
	}

	/**
	 * Get the background color of the text pane
	 * 
	 * @return	the background color of the text pane
	 */
	protected static Color getTextBackground() {
		return textBackground;
	}
	/**
	 * Set the background color of the text pane
	 * 
	 * @param textBackground	the background color of the text pane
	 */
	protected static void setTextBackground(Color textBackground) {
		NotebookSetting.textBackground = textBackground;
	}

	/**
	 * Get the default font color
	 * 
	 * @return	the default font color
	 */
	protected static Color getFontColor() {
		return fontColor;
	}
	/**
	 * Set the default font color
	 * 
	 * @param fontColor	the default font color
	 */
	protected static void setFontColor(Color fontColor) {
		NotebookSetting.fontColor = fontColor;
		EditorControl.getDialog().updatePeaPropValues(new Properties());
	}
	/**
	 * @return the clipboardContentIsFromPEA
	 */
	public static boolean isClipboardContentIsFromPEA() {
		return clipboardContentIsFromPEA;
	}
	/**
	 * @param clipboardContentIsFromPEA the clipboardContentIsFromPEA to set
	 */
	public static void setClipboardContentIsFromPEA(boolean clipboardContentIsFromPEA) {
		NotebookSetting.clipboardContentIsFromPEA = clipboardContentIsFromPEA;
	}
/*	@Override
	public void lostOwnership(Clipboard arg0, Transferable arg1) {
		// TODO Auto-generated method stub
		TestLog.v(getClass(), "lost");
		
	}
	*/
	
	//================== GETTERS & SETTERS ================//
	
	/**
	 * Set the language bundle for Calendar Lock PEA
	 * 
	 * @param newBundle	the language bundle to use
	 */
/*	public static void setResourceBundle(ResourceBundle newBundle) {
		CalendarSetting.calTranslate = newBundle;
	} */
	/**
	 * Get the current language bundle. If no bundle
	 * was set before, it is set with default language
	 * 
	 * @return		the current language bundle
	 */
/*	public final static ResourceBundle getResourceBundle(){
		if (CalendarSetting.calTranslate == null) { // set default language
			try {
				EditorControl.setLanguagesBundle(null);
			} catch (MalformedURLException e) {
				TestLog.e(XXXX.class, "Language support failed: "  + e.toString() + "\n  " + e.getMessage());
				e.printStackTrace();
			}
		}
		return CalendarSetting.calTranslate;
	}*/

}
