package cologne.eck.note_pea;

import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.Desktop;
import java.io.IOException;

import javax.swing.JTextPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import cologne.eck.tools.TestLog;

@SuppressWarnings("serial")
class PeaTextPane extends JTextPane implements HyperlinkListener{
	
	public PeaTextPane() {
		this.addHyperlinkListener(this);
	}
	
	@Override
	public void copy() {
		super.copy();
		if (LockFrameEditor.getFocusedTextPane() != null
				&& LockFrameEditor.getFocusedTextPane().getContentType() != null) {
			if (LockFrameEditor.getFocusedTextPane().getContentType().equals("text/html") ) {
				// register, that clipboard is set from PEA, does not work for rtf
				NotebookSetting.setClipboardContentIsFromPEA(true);
			}
		}
	}
	@Override
	public void cut() {
		super.cut();
		if (LockFrameEditor.getFocusedTextPane() != null
				&& LockFrameEditor.getFocusedTextPane().getContentType() != null) {
			if (LockFrameEditor.getFocusedTextPane().getContentType().equals("text/html") ) {
				// register, that clipboard is set from PEA, does not work for rtf
				NotebookSetting.setClipboardContentIsFromPEA(true);
			}
		}
	}
	@Override
	public void paste() {
		
		JTextPane textComponent = LockFrameEditor.getFocusedTextPane();		
		Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
		
		if (clip.isDataFlavorAvailable(DataFlavor.stringFlavor) ){
			
			Transferable t = clip.getContents(this);
			DataFlavor[] flavors = t.getTransferDataFlavors();
			DataFlavor fl = DataFlavor.selectBestTextFlavor(flavors);
			// 1. Check appropriate mime type
			if (textComponent.getContentType().equals("text/html") ){
				if(  fl.getMimeType().equals("text/html; class=java.io.Reader")) {
					//TestLog.v(CopyCutPastePopup.class, "Paste inside PEA");
					super.paste(); 
					return;
				}
			} else if( textComponent.getContentType().equals("text/rtf")) {// fl.getMimeType().equals("application/x-java-serialized-object; class=java.lang.String")) {
				//TestLog.v(PeaTextPane.class, "Paste inside PEA");
				super.paste(); 
				return;
			}
			// 2. Fallback: Check if clipboard content was set from PEA
			// This does only work once
			if (NotebookSetting.isClipboardContentIsFromPEA() == true
					&& textComponent.getContentType().equals("text/html")) {
				//TestLog.v(PeaTextPane.class,"Paste from PEA...");
				// default paste
				super.paste(); 
				NotebookSetting.setClipboardContentIsFromPEA(false);
				return;
			}

			// 3.Fallback: From other applications: Insert plain text
			//TestLog.v(PeaTextPane.class, "Insert plain text...");
			// try another way to insert					
			String data = "";
			try {
				// inserts plain text
				data = (String) clip.getData(DataFlavor.stringFlavor);
			} catch (HeadlessException e) {
				TestLog.ve(PeaTextPane.class, e.getMessage() + " - " + e.toString(), 5);
			} catch (UnsupportedFlavorException e) {
				TestLog.ve(PeaTextPane.class, e.getMessage() + " - " + e.toString(), 5);
			} catch (IOException e) {
				TestLog.ve(PeaTextPane.class, e.getMessage() + " - " + e.toString(), 5);
			} 
			textComponent.replaceSelection(data);
		} // TODO images
	}
	@Override
	public void hyperlinkUpdate(HyperlinkEvent e) {
		
		if(e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            if(Desktop.isDesktopSupported()) {
                try {
//System.out.println(e.getURL());
                    Desktop.getDesktop().browse(e.getURL().toURI());
                }
                catch (Exception e1) {
                    e1.printStackTrace();
                }
            }
        }		
	}
}
