package cologne.eck.note_pea;

import java.awt.Component;
import java.awt.Cursor;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.HashSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.EditorKit;
import javax.swing.text.StyledDocument;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.rtf.RTFEditorKit;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.WriteResources;
import cologne.eck.tools.Zeroizer;

class TextHelper {
	
	private static final String openError = 
			PeaProperties.getVmBridge().translate("failed")+ ": " +
			PeaProperties.getVmBridge().translate("open_file");	
	private static final String exportError = 
			PeaProperties.getVmBridge().translate("failed")+ ": " +
			PeaProperties.getVmBridge().translate("save_as");
	private static final String convertError = 
			PeaProperties.getVmBridge().translate("failed")+ ": " +
			PeaProperties.getVmBridge().translate("convert");
	
	
	
	// Helper functions:
	private final static JTextPane loadRtfStringToPane(String rtfString) {
		if (rtfString == null) {
			TestLog.ve(TextHelper.class, "Missing rtf string to convert", 5);
			return null;
		}
		JTextPane pane = new JTextPane();
	    pane.setContentType("text/rtf"); // also application/rtf
	    EditorKit kitRtf = pane.getEditorKitForContentType("text/rtf");	    
	    try {
			kitRtf.read(new StringReader(rtfString), pane.getDocument(), 0);
			//kitRtf.write(writer, p.getDocument(), 0, p.getDocument().getLength());
		} catch (FileNotFoundException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (IOException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (BadLocationException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (Exception e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}
	    kitRtf = null;
	    TestLog.v(TextHelper.class, "Set rtf string to text pane: " + pane.getDocument().getLength());
	    return pane;
	}
	/**
	 * Check if an String is a bookmark file. Checks if there are 
	 * more than 16 links or if the ratio between lines and links is < 4. 
	 * 
	 * @param htmlString	the string to check
	 * 
	 * @return	is handled as bookmark file
	 */
	protected final static boolean checkBookmarkType(String htmlString) {
		// search for pattern <a href=
		// lines / linkNumber < 4 -> bookmark file
		Matcher m = Pattern.compile("<a href=").matcher(htmlString);
		int links = 0;
		while (m.find() ) {
			links++;
			if (links > 16) {
				return true;
			}
		}
		m = Pattern.compile("\r\n|\r|\n").matcher(htmlString);
		int lines = 1;
		while (m.find()) {
			lines ++;
		}
		//TestLog.v(TextHelper.class, "lines: " + lines + ", links: " + links);

		if (links <= 0) {
			return false;
		}
		if (lines / links < 4) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Load a HTML string to a text pane
	 * 
	 * @param htmlString	the string to load
	 * 
	 * @return	a new created text pane with the string loaded
	 */
	private final static JTextPane loadHtmlStringToPane(String htmlString) {
		if (htmlString == null) {
			TestLog.ve(TextHelper.class, "Missing html string to convert", 5);
			return null;
		}
		JTextPane pane = new JTextPane();
	    pane.setContentType("text/html;charset=UTF-8");//setContentType("text/html"); 
	    EditorKit htmlKit = pane.getEditorKitForContentType("text/html");	    
	    try {
			htmlKit.read(new StringReader(htmlString), pane.getDocument(), 0);
			//kitRtf.write(writer, p.getDocument(), 0, p.getDocument().getLength());
		} catch (FileNotFoundException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (IOException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (BadLocationException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (Exception e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}
	    htmlKit = null;
	    TestLog.v(TextHelper.class, "Set html string to text pane: " + pane.getDocument().getLength());
	    return pane;
	}	
	private final static String getHtmlStringFromPane(JTextPane pane) {
		if (pane == null) {
			TestLog.ve(TextHelper.class, "Missing pane string for conversion", 5);
			return null;
		}
		EditorKit kitHtml = pane.getEditorKitForContentType("text/html");
		Writer writer = new StringWriter();
		
//		FontSize fontSize = new FontSize( (Float)attributes.get(TextAttribute.SIZE) );
		try {
			kitHtml.write(writer, pane.getDocument(), 0, pane.getDocument().getLength());
			TestLog.v(TextHelper.class, "Successfully got html string from text pane. ");
		} catch (IOException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (BadLocationException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (Exception e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}
		String result = writer.toString();
		TestLog.v(TextHelper.class, "Got html string from text pane: " + result.length());
		return result;
	}
	private final static String getRtfStringFromPane(JTextPane pane) {
		if (pane == null) {
			TestLog.ve(TextHelper.class, "Missing pane string for conversion", 5);
			return null;
		}
		EditorKit rtfKit = pane.getEditorKitForContentType("text/rtf");
		//Writer writer = new StringWriter();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			//rtfKit.write(writer, pane.getDocument(), 0, pane.getDocument().getLength());
			rtfKit.write(baos, pane.getDocument(), 0, pane.getDocument().getLength());
			TestLog.v(TextHelper.class, "Successfully got RTF string from text pane. ");
		} catch (IOException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (BadLocationException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (Exception e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}
		String result = baos.toString();
		TestLog.v(TextHelper.class, "Got rtf string from text pane: " + result.length());
		return result;
	}
	
	private final static DefaultStyledDocument getRTFDocumentFromString(String rtfString) {
		if (rtfString == null) {
			TestLog.ve(TextHelper.class, "Missing string for conversion", 5);
			return null;
		}
	    StringReader stringReader = new StringReader(rtfString);
	    RTFEditorKit rtfKit = new RTFEditorKit();
	    DefaultStyledDocument rtfDoc = (DefaultStyledDocument) rtfKit.createDefaultDocument();
	    try {
			rtfKit.read(stringReader, rtfDoc, 0);
		} catch (IOException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (BadLocationException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}	    
	    return rtfDoc;
	}
	private final static HTMLDocument getHTMLDocumentFromString(String htmlString) {
		if (htmlString == null) {
			TestLog.ve(TextHelper.class, "Missing string for conversion", 5);
			return null;
		}
	    StringReader stringReader = new StringReader(htmlString);
	    HTMLEditorKit htmlKit = new HTMLEditorKit();
	    HTMLDocument htmlDoc = (HTMLDocument) htmlKit.createDefaultDocument();
	    try {
			htmlKit.read(stringReader, htmlDoc, 0);
		} catch (IOException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (BadLocationException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}	    
	    return htmlDoc;
	}
/*	private final static String getRtfStringFromDocument(DefaultStyledDocument rtfDoc) {
		if (rtfDoc == null) {
			TestLog.ve(TextHelper.class, "Missing doc for conversion", 5);
			return null;
		}
		RTFEditorKit rtfKit = new RTFEditorKit();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			rtfKit.write(baos, rtfDoc, 0, rtfDoc.getLength());
		} catch (IOException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (BadLocationException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}  catch (Exception e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}	
		String result = baos.toString();
		return result;
	}
	private final static String getHtmlStringFromDocument(HTMLDocument htmlDoc) {
		if (htmlDoc == null) {
			TestLog.ve(TextHelper.class, "Missing doc for conversion", 5);
			return null;
		}
		HTMLEditorKit rtfKit = new HTMLEditorKit();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			rtfKit.write(baos, htmlDoc, 0, htmlDoc.getLength());
		} catch (IOException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		} catch (BadLocationException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}  catch (Exception e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage() + " - " + e.toString(), 5);
		}	
		String result = baos.toString();
		return result;
	}
	*/
	
	// Converter functions:
	/**  
	 * Convert a RTF file
	 * into a HTML string in the form "<html>..."
	 * 
	 * @param file	a file in RTF format
	 * 
	 * @return	A string in HTML format ("<html><head>...")
	 */
	private final static String convertRTFtoHTML(File file) {
		byte[] content = ReadResources.readExternFile(file.getAbsolutePath());
		String rtfString = new String(content, PeaProperties.getCharset());
		return convertRTFtoHTML(rtfString);
	}
	/** 
	 * Convert a HTML file
	 * into a RTF string 
	 * 
	 * @param file	a file in HTML format
	 * 
	 * @return	A string in RTF format 
	 */
	private final static String convertHTMLtoRTF(File file) {
		byte[] content = ReadResources.readExternFile(file.getAbsolutePath());
		String htmlString = new String(content, Charset.forName("ASCII"));
		return convertHTMLtoRTF(htmlString);
	}
	/**  
	 * Convert a RTF string in the form "{\\rtf1\\ansi..."
	 * into a HTML string in the form "<html>..."
	 * 
	 * @param rtfString	a string in RTF format ("{\\rtf1\\ansi...")
	 * 
	 * @return	A string in HTML format ("<html><head>...")
	 */
	private final static String convertRTFtoHTML(String rtfString) {
		
		
		
		TestLog.v(TextHelper.class, "Convert rtf string to html...");		
		JTextPane pane = loadRtfStringToPane(rtfString);    
	    return getHtmlStringFromPane(pane);
	}

	/**  
	 * Convert a HTML string in the form "<html>..."
	 * into a RTF string in the form "{\\rtf1\\ansi..."
	 * 
	 * @param htmlString	HTML string in the form "<html>..."
	 * 
	 * @return	RTF string in the form "{\\rtf1\\ansi..."
	 */
	private final static String convertHTMLtoRTF(String htmlString) {
		String newLineMarker = "#XXX#";
		TestLog.v(TextHelper.class, "Convert html string to rtf...");	
		// handle the line problem:
		htmlString = htmlString.replaceAll("<br>", newLineMarker);
		htmlString = htmlString.replaceAll("<br/>", newLineMarker);
		htmlString = htmlString.replaceAll("<br />", newLineMarker);
		htmlString = htmlString.replaceAll("</p>", newLineMarker);
		htmlString = htmlString.replaceAll("<p.*?>", "");
		
	    // font color problem: reset color, this adds a space character
	    htmlString = htmlString.replaceAll("</font>", "</font>" + "<font color=\"#000000\">&#8203;</font>"); //  &#8203; &nbsp;
		
		JTextPane pane = loadHtmlStringToPane(htmlString);    
	    String rtfString =  getRtfStringFromPane(pane);
	    rtfString = rtfString.replaceAll(newLineMarker, "\\\\par ");	    

	    return rtfString;
	}

	
	/**
	 * Convert a HTML bookmark file using Netscape Bookmark File Format
	 * into a readable HTML file with links
	 * 
	 * @param file	the bookmark file to read
	 * 
	 * @return	the HTML string or null
	 */
	private final static String convertBookmarksToHTML(File file) {

		StringBuilder sb = new StringBuilder();
		sb.append("<html>\n");
		// Open the file
		FileInputStream fstream = null;
		BufferedReader br = null;
		try {
			fstream = new FileInputStream(file);
			br = new BufferedReader(new InputStreamReader(fstream));
		} catch (FileNotFoundException e) {
			TestLog.ve(TextHelper.class, e.getLocalizedMessage(), 5);
			return null;
		}	
		if (br != null) {
			String line = null;

			// to skip the first part <!DOCTYPE...
			boolean startRead = false;
			// line by line:
			try {
				int headerType = 1;

				while ((line = br.readLine()) != null)   {
					if (startRead == true) {
						line = line.replaceAll("<DT>", "").trim();
						if (line.startsWith("<H3")) { // header H3
							int startIndex = line.indexOf('>');
							if (startIndex > 0) {
								line = "<H" + headerType + line.substring(startIndex);// delete all tags inside <H3>
								line = line.replace("</H3>", "</H" + headerType + ">");
								sb.append(line + "\n");
							}						

						} else if (line.startsWith("<H1")) { // header H1
							sb.append(line + "\n");

						} else if (line.startsWith("<A HREF=")) { // link
							int startIndex = line.indexOf('"', line.indexOf('"') + 1);
							String firstPart = line.substring(0, startIndex + 1); // include "
							String secondPart = line.substring(  line.indexOf('>') );
							sb.append(firstPart + secondPart + "<br>\n");

						} else if (line.equals("</DL><p>")) { // end
							headerType--;						
						} else if (line.equals("<DL><p>")) { // start
							headerType++;
						}
						sb.append("\n");
					} else {
						if (line.trim().startsWith("<META HTTP-EQUIV=")) {
							// start to read from next line on:
							startRead = true;
						}
					}
				}
			} catch (IOException e1) {
				TestLog.ve(TextHelper.class, e1.getLocalizedMessage(), 5);
				return null;
			}
			sb.append("</html>");
			//Close the input stream
			try {
				fstream.close();
			} catch (IOException e) {
				TestLog.ve(TextHelper.class, e.getLocalizedMessage(), 5);
				return null;
			}
		}
		return new String(sb);
		/*		
		byte[] content = ReadResources.readExternFile(file.getAbsolutePath());
		String bookmarkString = new String(content, PeaProperties.getCharset());
		// replace the start with html and add html end tag
		int startPos = bookmarkString.indexOf("charset=UTF-8\">") + "charset=UTF-8\">".length();
		bookmarkString = "<html>" + bookmarkString.substring(startPos) + "</html>";

		// replace DT with br:
		bookmarkString = bookmarkString.replaceAll("<br>", "<DT>");
		 */		
	}

	
	/**
	 * Serialize a DefaultStyledDocument with a RTFEditorKit or HTMLEditorKit. 
	 * Note: Till Java 9, the alignment property in RTF is not saved.
	 * 
	 * @param dsd	the document
	 * @param extension	String html or rtf
	 * 
	 * @return		the document as an array of bytes
	 */
	final static byte[] serialize(StyledDocument dsd, String extension)  {
		if (dsd == null) {
			TestLog.ve(TextHelper.class, "Missing document ", 5);
			return null;
		}
		if (extension == null) {
			TestLog.ve(TextHelper.class, "Missing extension ", 5);
			return null;
		}
		EditorKit kit = null;
		if (extension.equals("html")) {
			kit = new HTMLEditorKit();
		} else if (extension.equals("rtf")) {
			kit =  new RTFEditorKit();
		}
		else {
			TestLog.ve(TextHelper.class, "Invalid extension: " + extension);
			return null;
		}
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			kit.write( out,  dsd, 0, dsd.getLength() );
			out.close();
		} catch (BadLocationException e) {
			TestLog.e(TextHelper.class, "Converter: " + e);
			e.printStackTrace();
		} catch (IOException e) {
			TestLog.e(TextHelper.class, "Converter: " + e);
			e.printStackTrace();
		}  
	    return out.toByteArray();
	}
	/**
	 * Convert a serialized DefaultStyledDocument as an array
	 * of bytes into a DefaultStyledDocument. 
	 * Note: Till Java 9, the alignment property is not included.
	 * 
	 * @param data	the serialized document as an array of bytes
	 * @param extension		String rtf or html
	 * 
	 * @return		a StyledDocument
	 */
	final static StyledDocument deserialize(byte[] data, String extension)  {
		if (data == null) {
			TestLog.ve(TextHelper.class, "Missing data", 5);
			return null;
		}
		if (extension == null) {
			TestLog.ve(TextHelper.class, "Missing extension", 5);
			return null;			
		}

		EditorKit kit = null;
		StyledDocument dsd = null;
		if (extension.equals("rtf")) {
			kit =  new RTFEditorKit();
			dsd = new DefaultStyledDocument();
		} else if (extension.equals("html")) {
			kit = new HTMLEditorKit();
			dsd = new HTMLDocument();
		}
		ByteArrayInputStream in = new ByteArrayInputStream(data);
		if (kit != null && in != null) {
			try {
				kit.read(in, dsd, 0);
				in.close();
			} catch (BadLocationException e) {
				TestLog.e(TextHelper.class, "Deserialization failed:  "+ e.getLocalizedMessage() + " - " + e.toString());
				return null;
			} catch (IOException e) {
				TestLog.e(TextHelper.class, "Deserialization failed:  " + e.getLocalizedMessage() + " - " + e.toString());
				return null;
			}		
		} else {
			TestLog.e(TextHelper.class, "Deserialization failed: Missing variables",5);
			return null;
		}
	    return dsd;
	}
	
	final static String importFile(Component component, LockFrameEditor lockFrame) {
		
		if (lockFrame == null) {
			lockFrame = (LockFrameEditor) PeaControl.getDialog().getLockFrame();		
			if (lockFrame == null) {
				TestLog.e(TextHelper.class, "Null value", 5);
				return openError + "\n" + PeaProperties.getVmBridge().translate("unexpected_error");
			}
		}
		Object[] possibleOptions = { // "open encrypted lock file",
				"txt (text/plain) ", 
				"rtf (text/rtf) ",
				"html (text/html) ",
				"bookmark file (html)" // TODO language
				};
		Object selectedValue = JOptionPane.showInputDialog(component,
				PeaProperties.getVmBridge().translate("open_what_kind_of_file")
				+"\n(" + PeaProperties.getVmBridge().translate("formatting_get_lost") + ")", // "Open what kind of file? \n",
				null, JOptionPane.PLAIN_MESSAGE, null, possibleOptions, possibleOptions[0]);
		FileFilter filter = null;
		File file = null;
		String fileName = null;
		String importFormat = "";
		boolean isBookmarkFile = false;
		if (selectedValue == null) { // cancel
			TestLog.v(TextHelper.class, "Import selection cancelled...");
			return null;
		} else if (selectedValue.equals(possibleOptions[0])) {
			filter = new TxtFilter();			
			TestLog.v(TextHelper.class, "Import txt file...");
		} else if (selectedValue.equals(possibleOptions[1])) {
			filter = new RtfFilter();
			importFormat = "rtf";
			TestLog.v(TextHelper.class, "Import rtf file...");
		} else if (selectedValue.equals(possibleOptions[2])) {
			filter = new HTMLFilter();
			importFormat = "html";
			TestLog.v(TextHelper.class, "Import html file...");
		} else if (selectedValue.equals(possibleOptions[3])) {
			filter = new HTMLFilter();
			importFormat = "html";
			TestLog.v(TextHelper.class, "Import html bookmark file...");
			isBookmarkFile = true;
		}

		final PeaFileChooser fc = new PeaFileChooser(component);
		fc.setFileFilter(filter);
		int returnVal = fc.showOpenDialog(component);
		// PeaControl.getDialog().getLockFrame());
		if (returnVal == PeaFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
			if (file == null) {
				TestLog.ve(TextHelper.class, "Null value...", 5);
				return null;
			}
			fileName = file.getAbsolutePath();
			if (fileName == null) {
				TestLog.ve(TextHelper.class, "Null value...", 5);
				return null;
			}

		} else { // cancel option
			TestLog.v(TextHelper.class, "Export cancelled...");
			return null;
		}
		
		if (fileName.toLowerCase().endsWith(".txt")) {
			importFormat = "txt";
		} else if (fileName.toLowerCase().endsWith(".rtf")) {
			importFormat = "rtf";
		} else if (fileName.toLowerCase().endsWith(".html") 
				|| fileName.toLowerCase().endsWith(".htm") 
				|| fileName.toLowerCase().endsWith(".shtml") ) {
			importFormat = "html";
		} else {
			TestLog.ve(TextHelper.class, "Unknown extension: " + fileName.toLowerCase());
			importFormat = "XXX";
		}
				
		try {
			// check if file is encrypted:
			int fileIdLen = Attachments.getFileIdentifierSize();
			byte[] end = null;
			try {
				end = Attachments.getEndBytesOfFile( 
						fileName, 
						fileIdLen);			
			} catch (IOException e) {
				TestLog.e(TextHelper.class, new ExceptionHelper(e).getInfos() + " - " + fileName);
				return openError + "\n" + e.getLocalizedMessage();
			} catch (Exception e) {
				TestLog.e(TextHelper.class, new ExceptionHelper(e).getInfos() + " - " + fileName);
				return openError + "\n" + e.getLocalizedMessage();
			}
			if (end != null) {
				// check file identifier: This might be a randomly chosen one if key file property
				byte[] fileId = new byte[fileIdLen];
				System.arraycopy(end, end.length - fileIdLen, fileId, 0, fileIdLen);
				if (  Arrays.equals(fileId, Attachments.getFileIdentifier())) {
					TestLog.e(TextHelper.class, "Encrypted file: " + fileName);
					return openError + "\n" + PeaProperties.getVmBridge().translate("encrypted_file") + " " + fileName;
				}
				if (  Arrays.equals(fileId, AttachmentHandler.getVersion1ID() )) {
					TestLog.e(TextHelper.class, "Encrypted file: " + fileName);
					return openError + "\n" + PeaProperties.getVmBridge().translate("encrypted_file") + " " + fileName
							+ "\n(PEA version 1)";
				}
				if(  Arrays.equals(fileId, AttachmentHandler.getVersion2ID() )) {
					TestLog.e(TextHelper.class, "Encrypted file: " + fileName);
					return openError + "\n" + PeaProperties.getVmBridge().translate("encrypted_file") + " " + fileName
							+ "\n(PEA version 2)";
				}
			}

			// warning: long execution time
			if (file.length() > 1048576) { // 1 MiB
				JOptionPane.showMessageDialog(component,
						// PeaControl.getDialog().getLockFrame(),
						"Large file may cause long execution time. ",
						PeaProperties.getVmBridge().translate("warning"), JOptionPane.WARNING_MESSAGE);
				// set the wait cursor:
				lockFrame.setCursor(CursorManager.getWaitCursor());
			}
			// avoid too large files to encrypt
			if (file.length() > 8388608 * 16) { // 128 MiB limit to avoid memory errors				
				// reset the cursor:
				lockFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				return openError + "\nSelected file is too large";
			}
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			Document paneDoc = textPane.getDocument();
			if (paneDoc == null) {
				TestLog.e(TextHelper.class, "Null value", 5);
				return openError + "\n" + PeaProperties.getVmBridge().translate("unexpected_error"); 
			}
			StyledDocument doc = null;
			if (paneDoc instanceof StyledDocument) {
				doc = (StyledDocument) paneDoc;
			} else {
				TestLog.e(TextHelper.class, "Invalid document", 5);
				return openError + "\n" + PeaProperties.getVmBridge().translate("unexpected_error"); 
			}			
			int docLen = doc.getLength();

			// <<<<<<<<<< text file >>>>>>>>>>>>>
			if (importFormat.equals("txt")) {				
				byte[] textBytes = ReadResources.readExternFile(fileName);
				if (textBytes != null) {
					char[] textChars = Converter.bytes2chars(textBytes);
					if (textChars != null) {
						try {
							doc.insertString(docLen, new String(textChars), null);
							lockFrame.setDocChangeUnsaved(true);
						} catch (BadLocationException e) {
							TestLog.e(TextHelper.class, 
									"E (Open txt): \n" + e.toString() + ", " + e.getMessage());
							return openError + "\n" + e.getLocalizedMessage();
						}
						Zeroizer.zero(textChars);				
					} else {
						TestLog.e(TextHelper.class, "Null value", 5);
						return openError + "\n" + PeaProperties.getVmBridge().translate("unexpected_error"); 
					}
				} else {
					if (file.length() > 0) {
						TestLog.e(TextHelper.class, "Null value", 5);
						return openError + "\n" + PeaProperties.getVmBridge().translate("unexpected_error"); 
					}
				}
				
			// <<<<<<<<<< rtf or html file >>>>>>>>>>>>>
			} else if (importFormat.equals("rtf") || importFormat.equals("html")) {

				String paneFormat = LockFrameEditor.getCurrentTab().getPaneFormat();
				TestLog.v(TextHelper.class, "pane format: " + paneFormat);
				String newContent = null;

				if (importFormat.equals("rtf")){
					// check if content contains OLE, warn and ask to cancel
					//kit = new RTFEditorKit();
					byte[] encoded = null;
					try {
						encoded = Files.readAllBytes(file.toPath());
					} catch (IOException e1) {
						TestLog.ve(TextHelper.class, e1.toString() + ": " + e1.getLocalizedMessage());
						return openError + "\n" +e1.getLocalizedMessage();
					}
					if (encoded == null) {
						new UnexpectedValueException("byte[]", "encoded", "is null").printDescription();
						return openError;
					}
					String content = new String(encoded, StandardCharsets.US_ASCII);//PeaProperties.getCharset()); 
					int index = content.indexOf("{\\object");
					if (index != -1) {
						// Warning: 
						int res = PeaDialog.showQuestion(component,
								"This file contains the object linking and embedding (OLE) capability"
										+ "which allows embedding of malicious code. \n"
										+ "Do you want to open them anyway?", 
										PeaProperties.getVmBridge().translate("warning"), -1);
						if (res != 0) {
							TestLog.ve(TextHelper.class, "Import cancelled");
							return null;
						}
					}
					if (paneFormat.equals("html") ) {
						TestLog.v(TextHelper.class, "Convert rtf to html...");
						newContent = convertRTFtoHTML(file);
						if (newContent == null) {
							TestLog.ve(TextHelper.class, "Convertion failed", 5);
							return openError + "\n" + "(Convertion failed)";
						}
						importFormat = "html";
					} else {
						TestLog.v(TextHelper.class, "No convertion required...");
						newContent = new String(ReadResources.readExternFile(fileName), PeaProperties.getCharset());
					}
				} else if (importFormat.equals("html")) {
					if (paneFormat.equals("rtf") ) {
						if (isBookmarkFile == true) {
							TestLog.ve(TextHelper.class, "Unsuitable pane format: Use HTML file to import bookmarks");
							return openError + "\n" + "(Use html file to import bookmars)";
						}
						TestLog.v(TextHelper.class, "Convert html to rtf...");
						newContent = convertHTMLtoRTF(file);
						if (newContent == null) {
							TestLog.ve(TextHelper.class, "Convertion failed", 5);
							return openError + "\n" + "(Convertion failed)";
						}
						importFormat = "rtf";
					} else {
						if (isBookmarkFile == true) {
							newContent = convertBookmarksToHTML(file);
						} else {
							TestLog.v(TextHelper.class, "No convertion required...");
							newContent = new String(ReadResources.readExternFile(fileName), PeaProperties.getCharset());
						}
					}
				}
				TestLog.v(TextHelper.class, "Insert text...");
				if (importFormat.equals("rtf")) {
					StringReader reader = null;
					//FileInputStream in;
					try {
						EditorKit kit = textPane.getEditorKitForContentType("text/rtf");
						//in = new FileInputStream(fileName);
						reader = new StringReader(newContent);
						//kit.read(in, doc, docLen);//Exception in thread "AWT-EventQueue-1" java.lang.ClassCastException: java.lang.String cannot be cast to javax.swing.text.html.HTML$Tag
						kit.read(reader, doc, docLen);
						reader.close();
						lockFrame.setDocChangeUnsaved(true);
					} catch (FileNotFoundException e) {
						TestLog.ve(TextHelper.class, " (Open): \n" + e.toString() + ", " + e.getMessage(), 3);
						return openError + "\n" +e.getLocalizedMessage();
					} catch (IOException e) {
						TestLog.ve(TextHelper.class, " (Open): \n" + e.toString() + ", " + e.getMessage(), 3);
						return openError + "\n" +e.getLocalizedMessage();
					} catch (BadLocationException e) {
						TestLog.ve(TextHelper.class, " (Open): \n" + e.toString() + ", " + e.getMessage(), 3);
						return openError + "\n" +e.getLocalizedMessage();
					} catch (Exception e) {
						TestLog.ve(TextHelper.class, " (Open): \n" + e.toString() + ", " + e.getMessage(), 3);
						return openError + "\n" +e.getLocalizedMessage();
					}
				} else if (importFormat.equals("html")){//kit instanceof HTMLEditorKit) {
					EditorKit kit = textPane.getEditorKitForContentType("text/html");
					StringReader reader = null;

					try {
						reader = new StringReader(newContent);
						//in = new FileInputStream(fileName);
						//kit.read(in, doc, docLen);//Exception in thread "AWT-EventQueue-1" java.lang.ClassCastException: java.lang.String cannot be cast to javax.swing.text.html.HTML$Tag
						kit.read(reader, doc, docLen);
						reader.close();
						lockFrame.setDocChangeUnsaved(true);
					} catch (FileNotFoundException e) {
						TestLog.ve(TextHelper.class, " (Open): \n" + e.toString() + ", " + e.getMessage(), 3);
						return openError + "\n" +e.getLocalizedMessage();
					} catch (IOException e) {
						TestLog.ve(TextHelper.class, " (Open): \n" + e.toString() + ", " + e.getMessage(), 3);
						return openError + "\n" +e.getLocalizedMessage();
					} catch (BadLocationException e) {
						TestLog.ve(TextHelper.class, " (Open): \n" + e.toString() + ", " + e.getMessage(), 3);
						return openError + "\n" +e.getLocalizedMessage();
					} catch (Exception e) {
						TestLog.ve(TextHelper.class, " (Open): \n" + e.toString() + ", " + e.getMessage(), 3);
						return openError + "\n" +e.getLocalizedMessage();
					}
				}

			// <<<<<<<<<< unknown file format >>>>>>>>>>>>>
			} else {
				TestLog.ve(TextHelper.class, "Unknown format: " + importFormat);
				String[] options = {PeaProperties.getVmBridge().translate("cancel"), 
						PeaProperties.getVmBridge().translate("ignore_and_go_on")};
				int res = PeaDialog.showOptions(component, 
						PeaProperties.getVmBridge().translate("unsuitable_format") + ": "
								+ fileName, 
								PeaProperties.getVmBridge().translate("warning"), 
								0, 2, options, 0);
				if (res == 1) { // read as text
					byte[] textBytes = ReadResources.readExternFile(fileName);
					if (textBytes != null) {
						char[] textChars = Converter.bytes2chars(textBytes);
						try {
							doc.insertString(docLen, new String(textChars), null);
							lockFrame.setDocChangeUnsaved(true);
						} catch (BadLocationException e) {
							TestLog.e(TextHelper.class, 
									"E (Open ): \n" + e.toString() + ", " + e.getMessage());
							return openError + "\n" + e.getLocalizedMessage();
						} catch (Exception e) {
							TestLog.e(TextHelper.class, 
									"E (Open ): \n" + e.toString() + ", " + e.getMessage());
							return openError + "\n" + e.getLocalizedMessage();
						}
						Zeroizer.zero(textChars);
					}
				} else {
					TestLog.v(TextHelper.class, "Import of unknown file cancelled");
					return null;
				}
			}

		} finally {
			// reset the cursor:
			if (lockFrame == null) {
				lockFrame = (LockFrameEditor) PeaControl.getDialog().getLockFrame();
			}
			lockFrame.setCursor(CursorManager.getDefaultCursor());
		}
		return null;

	}
	
	final static String export(Component component) {
		Object[] possibleOptions = { 
				"txt (text/plain) - " + PeaProperties.getVmBridge().translate("unencrypted"), // "A. save as			
				"rtf (text/rtf) - " + PeaProperties.getVmBridge().translate("unencrypted"),//PeaProperties.getVmBridge().translate("plaintext_file_rtf"),
				"html (text/html) - " + PeaProperties.getVmBridge().translate("unencrypted")};// PeaProperties.getVmBridge().translate("html")};// "B. save as plain text in rich text
															// format (unencrypted)"};
		Object selectedValue = JOptionPane.showInputDialog(component,
				PeaProperties.getVmBridge().translate("choose_how_to_save")
				+"\n(" + PeaProperties.getVmBridge().translate("formatting_get_lost") + ")", // "Choose how you want to save the file:
																			// \n",
				PeaProperties.getVmBridge().translate("choose"), // "Choose...",
				JOptionPane.PLAIN_MESSAGE, null, possibleOptions, possibleOptions[0]);
		FileFilter filter = null;
		String fileName = null;
		String filePath = null;
		
		String exportFormat = "";
		if (selectedValue == null) { // cancel
			return null;
		} else if (((String) selectedValue).startsWith("txt")) {
			filter = new TxtFilter();
			exportFormat = "txt";
			TestLog.v(TextHelper.class, "Export as txt...");
		} else if (((String) selectedValue).startsWith("rtf")) {
			filter = new RtfFilter();
			exportFormat = "rtf";
			TestLog.v(TextHelper.class, "Export as rtf...");
		} else if (((String) selectedValue).startsWith("html")) {
			filter = new HTMLFilter();
			exportFormat = "html";
			TestLog.v(TextHelper.class, "Export as html...");
		}
		final PeaFileChooser fc = new PeaFileChooser(component);
		fc.setFileFilter(filter);
		int returnVal = fc.showSaveDialog((JFrame) PeaControl.getDialog().getLockFrame());
		if (returnVal == PeaFileChooser.APPROVE_OPTION) {
			fileName = fc.getSelectedFile().getName();
			filePath = fc.getSelectedFile().getPath();

		} else { // cancel option
			return null;
		}

		// WriteRessources...
		if (exportFormat.equals("txt")) {
			String text = null;
			try {
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				text = textPane.getDocument().getText(0, textPane.getDocument().getLength());
			} catch (BadLocationException e) {
				TestLog.e(TextHelper.class, "(Save as txt): \n" + e.toString() + ", " + e.getMessage());
				return exportError + "\n" +e.getLocalizedMessage();
			}
			if (text == null) {
				text = "";
			}
			byte[] textBytes = text.getBytes(PeaProperties.getCharset());// .Converter.chars2bytes(text.toCharArray());
			if (!fileName.endsWith(".txt")) {
				filePath = filePath + ".txt";
			}
			if (new File(filePath).exists()) {
				TestLog.v(TextHelper.class, "Warning existing file will be overwritten");
				int n = JOptionPane.showConfirmDialog(component,
						// PeaControl.getDialog().getLockFrame(),
						PeaProperties.getVmBridge().translate("overwrite_existing_file"),
						PeaProperties.getVmBridge().translate("warning"), JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.WARNING_MESSAGE);
				if (n != JFileChooser.APPROVE_OPTION) { // cancel, close
					TestLog.v(TextHelper.class, "Export cancelled");
					return null;
					// System.exit(0);
				}
			}
			boolean success = WriteResources.write(textBytes, filePath, null);
			if (success == false) {
				TestLog.e(TextHelper.class, " (Save as txt): \n");
				return exportError + "\n" +PeaProperties.getVmBridge().translate("read_write_failed");

			}
			
			
		} else if (exportFormat.equals("rtf")
				|| exportFormat.equals("html")) {

			if (exportFormat.equals("rtf") && ( ! filePath.toLowerCase().endsWith(".rtf"))) { // avoid double extension
				filePath = filePath + ".rtf";
			} else if (exportFormat.equals("html") && (  ! filePath.toLowerCase().endsWith(".html"))) {
				filePath = filePath + ".html";
				TestLog.v(TextHelper.class, "Add extension html");
			}
			if (new File(filePath).exists()) {
				TestLog.v(TextHelper.class, "Warning existing file will be overwritten");
				int n = JOptionPane.showConfirmDialog(component,
						// PeaControl.getDialog().getLockFrame(),
						PeaProperties.getVmBridge().translate("overwrite_existing_file"),
						PeaProperties.getVmBridge().translate("warning"), JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.WARNING_MESSAGE);
				if (n != JFileChooser.APPROVE_OPTION) { // cancel, close
					TestLog.v(TextHelper.class, "Export cancelled");
					return null;
					// System.exit(0);
				}
			}
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			
			StyledDocument doc = textPane.getStyledDocument();
			EditorKit kit = null;
			if (filter instanceof RtfFilter) {
				if (LockFrameEditor.getCurrentTab().getPaneFormat().equals("html")) {
					// must replace line breaks...
					TestLog.v(TextHelper.class, "Export from HTML to RTF might be incomplete...");
					//String htmlString = getHtmlStringFromDocument((HTMLDocument) textPane.getDocument());//getRtfStringFromPane(textPane);
					String htmlString = getHtmlStringFromPane(textPane);//
					WriteResources.writeText(convertHTMLtoRTF(htmlString), filePath);
					return null;
					//doc = new DefaultStyledDocument();
					//
				} else {
					kit = new RTFEditorKit();
				}
			} else if (filter instanceof HTMLFilter) {
				kit = new HTMLEditorKit();//textPane.getEditorKitForContentType("text/html");//new HTMLEditorKit();
				//doc = new HTMLDocument();
			}
			BufferedOutputStream out;
			try {
				out = new BufferedOutputStream(new FileOutputStream(filePath));
				if (kit != null) {
					kit.write(out, doc, doc.getStartPosition().getOffset(), doc.getLength());
				} else {
					TestLog.v(TextHelper.class, "Missing kit...", 5);
				}
				out.close();
				TestLog.v(TextHelper.class, "Successfully exported");
			} catch (FileNotFoundException e) {
				TestLog.e(TextHelper.class, "EditorLockFrame Menu (Save ): \n" + e.toString() + ", " + e.getMessage());
				return exportError + "\n" + e.getLocalizedMessage();
			} catch (IOException e) {
				TestLog.e(TextHelper.class, "EditorLockFrame Menu (Save): \n" + e.toString() + ", " + e.getMessage());
				return exportError + "\n" + e.getLocalizedMessage();
			} catch (BadLocationException e) {
				TestLog.e(TextHelper.class, "EditorLockFrame Menu (Save ): \n" + e.toString() + ", " + e.getMessage());
				return exportError + "\n" + e.getLocalizedMessage();
			}
		}
		return null;
	}
	
	
	final static String convertInHTML(Component c, LockFrameEditor lockFrame) {
		TabComponent tab = LockFrameEditor.getCurrentTab();
		if (tab == null) {
			TestLog.ve(TextHelper.class, "Missing tab", 5);
			return convertError;
		}
		if (lockFrame == null) {
			TestLog.ve(TextHelper.class, "Missing lockFrame ", 5);
			return convertError;
		}
		if (tab.getPaneFormat().equals("html")) {
			return "Already HTML";
		}
		
		// change and check file names:
		HashSet<String> fileNames = tab.getFileNames();
		HashSet<String> htmlFileNames = new HashSet<String>();
		for (String rtfFileName : fileNames) {
			if (rtfFileName != null) {
				String extension = "";
				int dotIndex = rtfFileName.lastIndexOf('.');
				if (dotIndex > 0) {
					extension = rtfFileName.substring(dotIndex+1);
				} 
				String htmlFileName = null;
				if (extension.toLowerCase().equals("html")) {
					TestLog.ve(TextHelper.class, "Already html file: " + rtfFileName, 5);
					continue;
				} else {
					if (dotIndex == -1) { // there is not dot
						dotIndex = rtfFileName.length() - 1;
					}
					htmlFileName = rtfFileName.substring(0,  dotIndex + 1 ) + "html";
				}			
				if (new File(htmlFileName).exists()) {
					TestLog.v(TextHelper.class, "file exists");
					return convertError + "\n" + "File already exists";
					/*
				int n = JOptionPane.showConfirmDialog(c,
						// PeaControl.getDialog().getLockFrame(),
						PeaProperties.getVmBridge().translate("overwrite_existing_file"),
						PeaProperties.getVmBridge().translate("warning"), JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.WARNING_MESSAGE);
				if (n != JFileChooser.APPROVE_OPTION) { // cancel, close
					TestLog.v(TextHelper.class, "Export cancelled");
					continue;
					// System.exit(0);
				} else {
					htmlFileNames.add(htmlFileName);
				}*/
				}
				htmlFileNames.add(htmlFileName);
			} else {
				TestLog.v(TextHelper.class, "Null file name", 5);
			}
		}
		
		if (htmlFileNames.isEmpty()) {
			TestLog.v(TextHelper.class, "No file name", 5);
			return null;
		}
		
		JTextPane textPane = tab.getTextPane();
		if (textPane == null) {
			TestLog.ve(TextHelper.class, "Missing text pane ", 5);
			return convertError;
		}
		StyledDocument doc = textPane.getStyledDocument();
		if (doc == null) {
			TestLog.ve(TextHelper.class, "Missing document ", 5);
			return convertError;
		}		
		
		//String rtfString = getRtfStringFromPane(textPane);
		
		String htmlString = getHtmlStringFromPane(textPane);
		if (htmlString == null) {
			TestLog.ve(TextHelper.class, "Convert to html failed", 5);
			return convertError + "\n(HTML)";
		}
		HTMLDocument htmlDoc = getHTMLDocumentFromString(htmlString);
		if (htmlDoc == null) {
			TestLog.ve(TextHelper.class, "Getting doc failed", 5);
			return convertError + "\n(DOC)";
		}
		TestLog.v(TextHelper.class, "Successfully converted to HTML");		
		
		// save as html file; 
		byte[] plainContent = serialize(htmlDoc, "html");
		if (plainContent == null) {
			TestLog.ve(TextHelper.class, "serialization failed", 5);
			return convertError + "\n(Serialization)";
		}
		// encrypt:
		byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainContent, null, true, null);
		if (cipherBytes == null) {
			TestLog.ve(TextHelper.class, "encryption failed", 5);
			return convertError + "\n(Encryption)";
		}
		for (String newFileName : htmlFileNames) {
			if (newFileName != null) {
				WriteResources.write(cipherBytes, newFileName, null);
			} else {
				TestLog.ve(TextHelper.class, "Null file name", 5);	
			}
		}
		
		// TODO remove old tab, create new tab
		// Change Model
		String addError = NotebookModel.addFileNameSet(htmlFileNames);
		if (addError != null) {
			TestLog.ve(TextHelper.class, "Adding failed: " + addError, 5);
			return convertError + "\n" + addError;
		}
		// add tab:
		String loadError =  lockFrame.decryptAndLoadFileToView(Converter.hashSetToArray(htmlFileNames)[0]);
		if (loadError != null) {
			return convertError + "\n" +  loadError;
		}
/*		JTabbedPane tabPane = ((JTabbedPane) tab.getParent());
//TODO
		TabComponent newFocusTab = (TabComponent) tabPane.getComponent(tabPane.getComponentCount() - 1);
		String changeFocusError = ((LockFrameEditor)lockFrame).changeFocus(newFocusTab); 
		
		// save
		String saveError = lockFrame.saveCurrentContent();
		if (saveError != null) {
			return saveError;
		}*/
		return null;

	}
	
	final static String convertInRTF(Component c, LockFrameEditor lockFrame) {
		TabComponent tab = LockFrameEditor.getCurrentTab();
		if (tab == null) {
			TestLog.ve(TextHelper.class, "Missing tab", 5);
			return convertError;
		}
		if (lockFrame == null) {
			TestLog.ve(TextHelper.class, "Missing lockFrame ", 5);
			return convertError;
		}
		if (tab.getPaneFormat().equals("rtf")) {
			return "Already RTF";
		}
		
		// change and check file names:
		HashSet<String> fileNames = tab.getFileNames();
		HashSet<String> rtfFileNames = new HashSet<String>();
		for (String hFileName : fileNames) {
			if (hFileName != null) {
				String extension = "";
				int dotIndex = hFileName.lastIndexOf('.');
				if (dotIndex > 0) {
					extension = hFileName.substring(dotIndex+1);
				} 
				String rFileName = null;
				if (extension.toLowerCase().equals("rtf")) {
					TestLog.ve(TextHelper.class, "Already rtf file: " + hFileName, 5);
					continue;
				} else {
					if (dotIndex == -1) { // there is not dot
						dotIndex = hFileName.length() - 1;
					}
					rFileName = hFileName.substring(0,  dotIndex + 1 ) + "rtf";
				}			
				if (new File(rFileName).exists()) {
					TestLog.v(TextHelper.class, "file exists");
					return convertError + "\n" + "File already exists";
				}
				rtfFileNames.add(rFileName);
			} else {
				TestLog.v(TextHelper.class, "Null file name", 5);
			}
		}
		
		if (rtfFileNames.isEmpty()) {
			TestLog.v(TextHelper.class, "No file name", 5);
			return null;
		}
		
		JTextPane textPane = tab.getTextPane();
		if (textPane == null) {
			TestLog.ve(TextHelper.class, "Missing text pane ", 5);
			return convertError;
		}
		StyledDocument doc = textPane.getStyledDocument();
		if (doc == null) {
			TestLog.ve(TextHelper.class, "Missing document ", 5);
			return convertError;
		}		
		

/*		
x		String rString = getRtfStringFromPane(textPane);
		if (rString == null) {
			TestLog.ve(TextHelper.class, "Convert to rtf failed", 5);
			return convertError + "\n(RTF)";
		}
		DefaultStyledDocument rDoc = getRTFDocumentFromString(rString);
		if (rDoc == null) {
			TestLog.ve(TextHelper.class, "Getting doc failed", 5);
			return convertError + "\n(DOC)";
		} */
		String hString = getHtmlStringFromPane(textPane);
		if (hString == null) {
			TestLog.ve(TextHelper.class, "Convert to rtf failed", 5);
			return convertError + "\n(RTF)";
		}
		String rString = convertHTMLtoRTF(hString);
		DefaultStyledDocument rDoc = getRTFDocumentFromString(rString);
		if (rDoc == null) {
			TestLog.ve(TextHelper.class, "Getting doc failed", 5);
			return convertError + "\n(DOC)";
		}
		
		
		
		
		
		TestLog.v(TextHelper.class, "Successfully converted to RTF");		
		
		// save as rtf file; 
		byte[] plainContent = serialize(rDoc, "rtf");
		if (plainContent == null) {
			TestLog.ve(TextHelper.class, "serialization failed", 5);
			return convertError + "\n(Serialization)";
		}
		// encrypt:
		byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainContent, null, true, null);
		if (cipherBytes == null) {
			TestLog.ve(TextHelper.class, "encryption failed", 5);
			return convertError + "\n(Encryption)";
		}
		for (String newFileName : rtfFileNames) {
			if (newFileName != null) {
				WriteResources.write(cipherBytes, newFileName, null);
			} else {
				TestLog.ve(TextHelper.class, "Null file name", 5);	
			}
		}
		
		// TODO remove old tab, create new tab
		// Change Model
		String addError = NotebookModel.addFileNameSet(rtfFileNames);
		if (addError != null) {
			TestLog.ve(TextHelper.class, "Adding failed: " + addError, 5);
			return convertError + "\n" + addError;
		}
		// add tab:
		String loadError =  lockFrame.decryptAndLoadFileToView(Converter.hashSetToArray(rtfFileNames)[0]);
		if (loadError != null) {
			return convertError + "\n" +  loadError;
		}
/*		JTabbedPane tabPane = ((JTabbedPane) tab.getParent());
//TODO
		TabComponent newFocusTab = (TabComponent) tabPane.getComponent(tabPane.getComponentCount() - 1);
		String changeFocusError = ((LockFrameEditor)lockFrame).changeFocus(newFocusTab); 
		
		// save
		String saveError = lockFrame.saveCurrentContent();
		if (saveError != null) {
			return saveError;
		}*/
		return null;

	}


	
	// --------------------------------
	// inner classes
	//
	static class RtfHtmlFilter extends FileFilter { // for encrypted files only
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".rtf")
					|| f.getName().toLowerCase().endsWith(".html");
		}
		public String getDescription() {
			return "HTML or rich text format" + " (*.rtf), (*.html)";
		}
	};
	private static class TxtFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")
					|| f.getName().toLowerCase().endsWith(".text");
		}
		public String getDescription() {
			return "text" + " (*.txt)";
		}
	};

	private static class RtfFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".rtf");
		}
		public String getDescription() {
			return "rich text format" + " (*.rtf)";
		}
	};
	
	private static class HTMLFilter extends FileFilter { 
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".html")
					|| f.getName().toLowerCase().endsWith(".htm")
					|| f.getName().toLowerCase().endsWith(".shtml");
		}
		public String getDescription() {
			return "HTML format" + " (*.html)";
		}
	};
}
