package cologne.eck.peafactory.control;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Calls the Jar Compiler to compile the modified files with
 * new settings and new random values. 
 */


import java.io.IOException;
import java.util.Locale;


//import com.sun.tools.javac.main.JavaCompiler;


import javax.tools.Diagnostic;
import javax.tools.DiagnosticListener;
import javax.tools.JavaCompiler;
import javax.tools.JavaFileObject;
import javax.tools.StandardJavaFileManager;
import javax.tools.ToolProvider;


//import com.sun.tools.javac.Main;

//import com.sun.tools.javac.main.Main;

//import org.eclipse.jdt.internal.compiler.tool.EclipseCompiler;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.tools.TestLog;


class FileCompiler {	
	
    private static class MyDiagnosticListener implements DiagnosticListener<JavaFileObject> {

		@Override
		public void report(Diagnostic<? extends JavaFileObject> diagnostic) {
			 
            TestLog.o(FileCompiler.class, "Line Number->" + diagnostic.getLineNumber());
            TestLog.o(FileCompiler.class, "code->" + diagnostic.getCode());
            TestLog.o(FileCompiler.class, "Message->"
                               + diagnostic.getMessage(Locale.ENGLISH));
            TestLog.o(FileCompiler.class, "Source->" + diagnostic.getSource());
            TestLog.o(FileCompiler.class, " ");			
		}
    }

    /**
     * Compile two files: setting.java and start.java. 
     * Use system Java compiler if available, 
     * otherwise included OpenJDK javac
     * 
     * @param javaFileNames	the file names to compile with absolute path
     * 
     * @return null if compilation was successful, an error message otherwise
     */
    protected String compile(String [] javaFileNames) {

		// get Compiler: package ecj-3.7.jar
    	// until version 0.3.0-beta:		JavaCompiler compiler = new EclipseCompiler();// org.eclipse.jdt.internal.compiler.tool.EclipseCompiler
		
		// Java compiler for platform if available
		JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();

		if (compiler == null) {
			TestLog.e(FileCompiler.class, "No system compiler found...");
			for (int i=0; i< javaFileNames.length;i++) {
				//TestLog.o(FileCompiler.class, javaFileNames[i]);
				String[] args = new String[] {
						javaFileNames[i]
				};
				// Use compiler from tools.jar inside peafactory
				int compileStatus = com.sun.tools.javac.Main.compile(args); // com.sun.tools.javac.Main
				if (compileStatus != 0) {
					TestLog.e(FileCompiler.class, "compilation failed for " + javaFileNames[i] + compileStatus, 5);
					return "compilation failed for " + javaFileNames[i] + ", status: " + compileStatus;
				} else {
					TestLog.o(FileCompiler.class, "Successfully compiled " + javaFileNames[i]);
				}
			}
			return null;
		}
		
        // for compilation diagnostic message processing on compilation WARNING/ERROR
        MyDiagnosticListener c = new MyDiagnosticListener();
		
        StandardJavaFileManager fileManager = compiler.getStandardFileManager(c, null, PeaProperties.getCharset());
 
        // java files -> file objects
        Iterable<? extends JavaFileObject> fileObjects =  fileManager.getJavaFileObjects(javaFileNames);

        try {
            JavaCompiler.CompilationTask task = compiler.getTask(null, fileManager, c, null, null, fileObjects);
            task.call();	
			fileManager.close();
		} catch (RuntimeException e) {
			TestLog.e(FileCompiler.class, "FileCompiler: " + e.toString(), 5);	
			return e.getLocalizedMessage() + "\n" + e.toString();
		} catch (IOException e) {
			TestLog.e(FileCompiler.class, "FileCompiler: Can not close StandardJavaFileManager " + e.toString(), 5);
			return e.getLocalizedMessage() + "\n" + e.toString();
		}  
        return null;
	} 
}
