package cologne.eck.peafactory.control;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Program flow of generation the jar file (Password Encrypted Archive). 
 */

import java.io.File;

import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.peafactory.gui.MainView;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.Zeroizer;


public class JarStuff {	

	private static String jarFileName;// set in MainView, get in FileModifier
	private static String labelText;// set in MainView, get in FileModifier

	private static final String fileToCompile1 = "settings" + File.separator + "PeaSettings.java";// file to be modified: 
	private static final String fileToCompile2 = "start" + File.separator + "Start.java";// file to be modified: 


	/**
	 * Create the jar file: 
	 * write start.java and settings.java
	 * compile start.java and settings.java
	 * and pack the jar file. 
	 * 
	 * @return true for success, false otherwise
	 */
	protected final boolean generateJarFile() {//byte[] keyMaterial) {
		
		// Type of PEA: image, text, file...
		//DataType dataType = DataType.getCurrentType();		
		
		//=======================
		// generate java files:
		FileModifier.generateStart();		// class to start the PEA with required memory
		FileModifier.generatePeaSettings();	// all settings and Getter/Setter for the PEA

		MainView.progressBar.setValue(45);
		MainView.progressBar.paint(MainView.progressBar.getGraphics());		
		
		//=====================================
		// compile with new variables
		//			
		// java files to compile
		String javaFileNames[] = {fileToCompile1, fileToCompile2};	
		FileCompiler compiler = new FileCompiler();
		String compileError = compiler.compile(javaFileNames);
		if (compileError != null) {
			TestLog.ve(JarStuff.class, "Compilation failed", 5);
			PeaDialog.showMessage(MainView.getInstance(null), "Compilation failed...\n" + compileError, "Error", 0);
			return false;
		}
		MainView.progressBar.setValue(60);
		MainView.progressBar.paint(MainView.progressBar.getGraphics());		
		
		//=======================================
        // Create jar archive with compiled files
		CreateJarFile.createJarFile();
		
		// overwrite random values: overwrite files with 0
		if (PeaFactory.isKeyFileProperty() == true) {
			Zeroizer.zeroFile("settings" + File.separator + "PeaSettings.class");
			Zeroizer.zeroFile("settings" + File.separator + "PeaSettings.java");
		}

		MainView.progressBar.setValue(100);
		MainView.progressBar.paint(MainView.progressBar.getGraphics());
		return true;
	}
	
	
	protected final static String getJarFileName() {
		return jarFileName;
	}
	public final static void setJarFileName(String _jarFileName) {
		jarFileName = _jarFileName;
	}
	protected final static String getLabelText() {
		return labelText;
	}
	public final static void setLabelText(String _labelText) {
		labelText = _labelText;
	}
}
