package cologne.eck.peafactory.control;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */


import java.awt.EventQueue;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.*;

import javax.swing.JOptionPane;

import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREBridge;
import cologne.eck.peafactory.control.data_type.DataType;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.peafactory.crypto.RandomStuff;
import cologne.eck.peafactory.crypto.kdf.CatenaKDF;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.peafactory.data.FactoryPropertyHandler;
import cologne.eck.peafactory.data.PeaFactoryProperties;
import cologne.eck.peafactory.gui.MainView;
import cologne.eck.peafactory.gui.ProjectSelection;
import cologne.eck.tools.Zeroizer;
import cologne.eck.tools.Comparator;
import cologne.eck.tools.Converter;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;

import org.bouncycastle.crypto.BlockCipher;
import org.bouncycastle.crypto.Digest;
import org.bouncycastle.crypto.digests.Blake2bDigest;
import org.bouncycastle.crypto.engines.ThreefishEngine;

public final class PeaFactory {

	// i18n: 	
	private static String language; 
	private static Locale currentLocale;
	private static Locale defaultLocale = Locale.getDefault();
	private static ResourceBundle languageBundle;
	
	private static String os = null;

	private static MainView frame;
	
	private static String version = "Peafactory version 0.4-beta";// 
	private static int PEA_VERSION = 2;
	
	private final static String packagePathName = "cologne" + File.separator + "eck" + File.separator + "peafactory";
	
	// default algos:
	private static final KeyDerivation DEFAULT_KDF = new CatenaKDF();
	private static final Digest DEFAULT_HASH = new Blake2bDigest();
	private static final BlockCipher DEFAULT_CIPHER = new ThreefishEngine(512);
	
	/*
	 * Properties of the producing PEA:
	 */
	// create new random bytes for programRandomBytes and fileIdentifier
	private static boolean keyFileProperty = true;// TODO set true
	// create new random bytes for programRandomBytes and fileIdentifier
	// and store them to use the same values for the next PEAs
	private static boolean keepRandomBytes = false;
	// create a start script for Unix and store along with the PEA 
	private static boolean unixScript = false;
	
	// Use the previously stored random bytes for programRandomBytes and fileIdentifier
	private static boolean useKeptRandomBytes = false;
	private static byte[] keptRandomBytes = null;
	private static byte[] keptFileIdentifier = null;
	private static String insertedKey = null;
	private static boolean testMode = false;
	
	// 129 byte string to initialize the salt: 
	private final static String FIXED_STRING = "PEAFACTORY - PRODUCTION OF PASSWORD ENCRYPTED ARCHIVES - FIXED STRING USED TO INITIALIZE THE SALT FOR THE KEY DERIVATION FUNCTION";
	private final static String FIXED_FILE_ID_STRING = "PFACTORY";

	
	public static void main(String[] args) throws MalformedURLException {		
		
		os = System.getProperty("os.name");
		
		if (args.length > 0) {
			if (args[0] != null) {
				if(args[0].equals("-r")){
					TestLog.o(PeaFactory.class, "Rescue mode only works for PEAs, not for PeaFactory...");
				} else if(args[0].equals("-t")){
					TestLog.o(PeaFactory.class, "#####---TEST MODE---#####");
					PeaProperties.setWorkingMode("-t");
					testMode = true;
				} else {
					TestLog.o(PeaFactory.class, "Unknown argument: " + args[0]);
				}
			}
		}
		if (testMode == true) {			
			//new TestMode().runInTestMode();
			TestLog.v(PeaFactory.class, version);
		}
		
		// i18n: 
	    language = defaultLocale.getLanguage(); // get default language
	    currentLocale = new Locale(language); // set default language as language

	    //File file = new File("src" + File.separator + "config" + File.separator + "i18n");
	    File file = new File("config" + File.separator + "i18n"); 
	    URL[] urls = {file.toURI().toURL()};  
	    ClassLoader loader = new URLClassLoader(urls);  
	    languageBundle = ResourceBundle.getBundle("LanguagesBundle", currentLocale, loader);
	    
	    PeaProperties.setVmBridge(new JREBridge());
    
	    // DEFAULT HASH FUNCTION: 
	    if (HashStuff.getHashAlgo() == null) { // needed for RandomPasswordDialog
	    	HashStuff.setHashAlgo( DEFAULT_HASH );
	    }	    
	    
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {		
					TestLog.v(PeaFactory.class, packagePathName + "\nOS: " + os);
					new FactoryPropertyHandler().readAndSetAllPropValues();
					PswDialogView.setUI(false);					
					// select the kind of pea
					ProjectSelection proj = new ProjectSelection(PeaFactoryProperties.getFrameLocation());
					proj.setVisible(true);								
					frame = MainView.getInstance(proj.getLocationOnScreen());
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
	
	public final static void createFile() {
		
		String peaFileName;
		String peaPath;

		try {
			// indicate that program is working:
			MainView.showWaitCursor();

		if (testMode == true){
			TestLog.o(PeaFactory.class, "key file property: " + PeaFactory.isKeyFileProperty());
			TestLog.o(PeaFactory.class, "should keep random bytes for next PEA: " + keepRandomBytes);
			TestLog.o(PeaFactory.class, "is compatible with previous PEA: " + PeaFactory.isUseKeptRandomBytes());
		}

		
		//=====================
		// initialize (order!)
		//
	    if (HashStuff.getHashAlgo() == null) {
	    	HashStuff.setHashAlgo( DEFAULT_HASH );// default
	    	//HashStuff.setHMacAlgo( DEFAULT_HASH );// default
	    }
	    if (PeaFactory.isKeyFileProperty() == true){ // unique values
	    	
	    	if (keepRandomBytes == true) { // keep values for next PEA 		
	    		if (useKeptRandomBytes == true){ // use values from previous PEA
	    			// check for program bug in test mode:
	    			if (testMode == true) {
	    				Comparator.checkNullVector(keptRandomBytes);
	    			}
	    			Attachments.setProgramRandomBytes(keptRandomBytes);
	    		} else { 
	    			if (insertedKey != null){
	    				// use manually inserted key
	    				Attachments.setProgramRandomBytes(Converter.hex2bytes(insertedKey));
	    			} else {
	    				// create new values
	    				Attachments.generateAndSetProgramRandomBytes();
	    			}
	    		}
	    		// store values in variable:
	    		keptRandomBytes = Attachments.getProgramRandomBytes();
	    		
	    		if (useKeptRandomBytes == true){ // use values from previous PEA
	    			Attachments.setFileIdentifier(keptFileIdentifier);
	    		} else { // create new values
	    			Attachments.generateAndSetFileIdentifier();	
	    		}		    	
	    		// store values in variable:
		    	keptFileIdentifier = Attachments.getFileIdentifier();
		    	// set value to register in next PEA
		    	useKeptRandomBytes = true;
		    	
	    	} else {
	    		if (keptRandomBytes != null) {
	    			Zeroizer.zero(keptRandomBytes);
	    			keptRandomBytes = null;
	    			keptFileIdentifier = null;
	    		}
    			if (insertedKey != null){
    				// use manually inserted key
    				Attachments.setProgramRandomBytes(Converter.hex2bytes(insertedKey));
    			} else {
    				// create new values
    				Attachments.generateAndSetProgramRandomBytes();
    			}
		    	Attachments.generateAndSetFileIdentifier();	
		    	useKeptRandomBytes = false;
	    	}

	    } else { // default values
	    	
	    	useKeptRandomBytes = false;
    		if (keptRandomBytes != null) {
    			Zeroizer.zero(keptRandomBytes);
    			keptRandomBytes = null;
    		}
	    	// use default values for fileIdentifier and programRandomBytes
	    	// calculate default salt and update via setAttachedSalt: 
	    	// xor the salt with attachedSalt (will be attached to the content)
	    	Attachments.setFileIdentifier(FIXED_FILE_ID_STRING.getBytes(PeaProperties.getCharset())); // 8 byte
	    	String saltString = FIXED_STRING.substring(0, KeyDerivation.getSaltSize() );
	    	Attachments.setProgramRandomBytes(saltString.getBytes(PeaProperties.getCharset()));
	    }	    
	    //Help.printBytes("programRandomBytes",Attachments.getProgramRandomBytes());
	    
	    // new
	    byte[] attachedSalt = new RandomStuff().createRandomBytes(Attachments.getProgramRandomBytesSize());
	    if (attachedSalt == null) {
	    	new UnexpectedValueException("byte[]", "attachedSalt","is null").printDescription();
	    }
	    if ( KeyDerivation.getKdf() == null) {
	    	new UnexpectedValueException(" KeyDerivation", " KeyDerivation.getKdf()","is null").printDescription();
	    }
	    KeyDerivation.setSalt(attachedSalt);
	    
	    // DEFAULT CIPHER ALGO: 
	    if (CipherStuff.getCipherAlgo() == null) {
	    	CipherStuff.setCipherAlgo( DEFAULT_CIPHER );
	    }
	    // DEFAULT KEY DERIVATION FUNCTION
	    if (KeyDerivation.getKdf() == null) {
	    	KeyDerivation.setKdf( DEFAULT_KDF );
	    }

		MainView.progressBar.setValue(35);
		MainView.progressBar.paint(MainView.progressBar.getGraphics());
			
		//=================
		// generate jar file:
		//
		//long start = System.currentTimeMillis(); // Startpunkt
		
		// Create algo param string from current settings:
		String algoParams = AlgoParamHandler.writeString(false);

		TestLog.v(PeaFactory.class, "Algo-Param-String:" + algoParams);
		// set algo param string in pea.properties as default:
		new PropertyHandler().setOneProperty("default_algo_param", algoParams);

		boolean success = new JarStuff().generateJarFile();
		if (success == false) {
			MainView.resetCursor();
			MainView.updateFrame(MainView.getFrameLocation());
			return;
		}
		peaFileName = JarStuff.getJarFileName().substring(0, (JarStuff.getJarFileName().length() - 4)); // extract ".jar"
		peaPath = System.getProperty("user.dir") + File.separator + "peas";
		
	} finally {
		MainView.resetCursor();
	}
		
		// show informations about created pea:
		int input = JOptionPane.showConfirmDialog(frame,
				languageBundle.getString("new_pea_info") + "\n"
				+ DataType.getCurrentType().getDescription() + "\n\n"
				+ languageBundle.getString("new_pea_type_info") + " "
				+ languageBundle.getString("file") + " \n "
				+ "    - " + peaFileName + " -\n"
				+ languageBundle.getString("new_pea_location_info") + "\n "
				+ "   " + peaPath + "\n\n"
				+ languageBundle.getString("open_directory") + "\n ",
			"Pea info",
				    JOptionPane.YES_NO_OPTION);
		
		if (input == JOptionPane.YES_OPTION) { // 0 
			
			String error = OperatingSystemStuff.openFileManager(peaPath);
			
			if (error != null) {
				JOptionPane.showMessageDialog(frame,
						languageBundle.getString("desktop_error")
						+ "\n\n  " + error,
						languageBundle.getString("error"),
						JOptionPane.ERROR_MESSAGE);
			}
		} else { // do not open folder peas:
			//
		}
	}
	
	//=========================
	// Helper functions:
	
	// set new language, locale, ResourceBundle
	public final static void setI18n(String newLanguage){
        
        // get ResourceBundle
	    File file = new File("config" + File.separator + "i18n");  
	    URL url = null;
		try {
			url = file.toURI().toURL();
		} catch (MalformedURLException e) {
			TestLog.e(PeaFactory.class, "PeaFactory: Missing Language files.", 5);
		}  
	    URL[] urls = {url};
	    ClassLoader loader = new URLClassLoader(urls);  
	    ResourceBundle newLanguageBundle = ResourceBundle.getBundle("LanguagesBundle", new Locale(newLanguage), loader);  

		// set variables:
	    language = newLanguage;
	    currentLocale = new Locale(newLanguage);
	    languageBundle = newLanguageBundle;

		// remember to reset
		String jarName = MainView.getJarName();
		String labelText =  MainView.getLabelText();
		MainView.updateFrame(MainView.getFrameLocation());
		// reset labels:
		MainView.setJarName(jarName);
		MainView.setLabelText(labelText);
	}

	// ==============================================================================================
	// Getter & Setter
	//
	public final static String getLanguage() {
		return language;
	}
	public final static ResourceBundle getLanguagesBundle(){
		return languageBundle;
	}
	/*public final static String getPackagePath() {
		return packagePathName;
	}*/
	public final static boolean isFrameInstantiated() {
		if (frame == null) {
			return false;
		} else {
			return true;
		}
	}

	/*public final static String getOS() {
		return os;
	}*/
	public final static MainView getFrame() {
		return frame;
	}
	/**
	 * @return the version
	 */
/*	protected static String getVersion() {
		return version;
	} */

	/**
	 * Check if the next to produce PEA will be compatible with the current PEA. 
	 * Compatible PEAs do reuse the previously stored random values. 
	 * 
	 * @return the keepRandomBytes	true, if next to produce PEA will be compatible with the current PEA. 
	 */
	public static boolean isKeepRandomBytes() {
		return keepRandomBytes;
	}

	/**
	 * Set compatibility of the next PEA, that means reusing of the random bytes 
	 * of the current PEA for the next to produce PEA. 
	 * 
	 * @param keepRandomBytes the keepRandomBytes to set
	 * 					true: the next to produce PEA will be compatible with the current PEA. 
	 * 					false: next PEA will be incompatible, random values will be cleared.
	 */
	public static void setKeepRandomBytes(boolean keepRandomBytes) {
		PeaFactory.keepRandomBytes = keepRandomBytes;
	}

	/**
	 * Check if existing random bytes should be reused. These random
	 * bytes had been stored previously for reusing. 
	 * 
	 * @return	true if random bytes should be reused
	 */
	public static boolean isUseKeptRandomBytes() {
		return useKeptRandomBytes;
	}
	/**
	 * Set reusing existing random bytes. These random
	 * bytes had been stored previously for reusing. 
	 * 
	 * @param _useKeptRandomBytes	true: reuse, otherwise clear the random bytes
	 */
	public static void setUseKeptRandomBytes(boolean _useKeptRandomBytes) {
		useKeptRandomBytes = _useKeptRandomBytes;
	}

	public static void setInsertedKey(String keyAsHex) {
		insertedKey = keyAsHex;
	}

	/**
	 * @return the unixScript
	 */
	public static boolean isUnixScript() {
		return unixScript;
	}

	/**
	 * @param unixScript the unixScript to set
	 */
	public static void setUnixScript(boolean _unixScript) {
		PeaFactory.unixScript = _unixScript;
	}

	/**
	 * @return the keyFileProperty
	 */
	public static boolean isKeyFileProperty() {
		return keyFileProperty;
	}

	/**
	 * @param keyFileProperty the keyFileProperty to set
	 */
	public static void setKeyFileProperty(boolean _keyFileProperty) {
		PeaFactory.keyFileProperty = _keyFileProperty;
	}

	/**
	 * @return the pEA_VERSION
	 */
	static int getPEA_VERSION() {
		return PEA_VERSION;
	}

	/**
	 * @param pEA_VERSION the pEA_VERSION to set
	 */
	/*public static void setPEA_VERSION(int pEA_VERSION) {
		PEA_VERSION = pEA_VERSION;
	}*/
}