package cologne.eck.peafactory.gui;

import java.awt.Color;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Settings for Peas
 */


import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.peafactory.control.PeaFactory;
import cologne.eck.tools.TestLog;

@SuppressWarnings("serial")
class GeneralPeaSettings extends JDialog 
		implements ActionListener, DocumentListener {	
	
	private GeneralPeaSettings peaSettings;
	
	// check boxes for properties: 
	private JCheckBox keyFilePropertyCheck;
	private JCheckBox compatibilityCheck;
	private JCheckBox defaultCheck;	
	private JCheckBox scriptCheck;
	
	private JTextArea keyArea; // to show the key file property as hex string
	
	private ResourceBundle languageBundle;
	
	public GeneralPeaSettings() {
		
		super(PeaFactory.getFrame() );

		peaSettings = this;	

		// i18n
		languageBundle = PeaFactory.getLanguagesBundle();
		
		this.setIconImage(MainView.getImage());
		
		this.setTitle(languageBundle.getString("general_pea_settings"));

		JPanel contentPane = (JPanel) peaSettings.getContentPane();//new JPanel();
		contentPane.setBorder(PeaBorderFactory.createBorder(false));
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel infoPanel = new JPanel();
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
		JTextArea infoArea = new JTextArea(languageBundle.getString("general_pea_infos"));
		infoArea.setWrapStyleWord(true);
		infoArea.setLineWrap(true);
		infoArea.setEditable(false);
		infoPanel.add(Box.createHorizontalStrut(10));
		infoPanel.add(infoArea);
		infoPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoPanel);
		
		contentPane.add(Box.createVerticalStrut(10));		
		
		JPanel tipPanel = new JPanel();
		tipPanel.setLayout(new BoxLayout(tipPanel, BoxLayout.X_AXIS));
		JLabel tipLabel1 = new JLabel(languageBundle.getString("this_session"));
		tipLabel1.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		tipPanel.add(tipLabel1);
		tipPanel.add(Box.createHorizontalGlue());
		
		contentPane.add(tipPanel);		
		
		contentPane.add(Box.createVerticalStrut(20));
		
		JPanel boundPanel = new JPanel();
		
		boundPanel.setLayout(new BoxLayout(boundPanel, BoxLayout.X_AXIS));
		keyFilePropertyCheck = new JCheckBox(languageBundle.getString("key_file_pea"), false);
		if (PeaFactory.isKeyFileProperty() == true) {
			keyFilePropertyCheck.setSelected(true);
		} else {
			keyFilePropertyCheck.setSelected(false);
		}
		keyFilePropertyCheck.addActionListener(this);
		keyFilePropertyCheck.setActionCommand("keyFilePropertyCheck");

		boundPanel.add(keyFilePropertyCheck);
		boundPanel.add(Box.createHorizontalGlue());
		
		contentPane.add(boundPanel);		
		
		JPanel infoBoundPanel = new JPanel();
		infoBoundPanel.setLayout(new BoxLayout(infoBoundPanel, BoxLayout.X_AXIS));
		JTextArea infoBoundArea = new JTextArea(languageBundle.getString("key_file_pea_infos"));
		infoBoundArea.setWrapStyleWord(true);
		infoBoundArea.setLineWrap(true);
		infoBoundArea.setEditable(false);
		
		infoBoundPanel.add(Box.createHorizontalStrut(20));
		infoBoundPanel.add(infoBoundArea);
		infoBoundPanel.add(Box.createHorizontalGlue());
		
		contentPane.add(infoBoundPanel);
		
		JPanel insertKeyPanel = new JPanel();
		insertKeyPanel.setLayout(new BoxLayout(insertKeyPanel, BoxLayout.LINE_AXIS));
		JLabel label = new JLabel(languageBundle.getString("existing_key") + ": ");
		insertKeyPanel.add(Box.createHorizontalStrut(20));
		insertKeyPanel.add(label);
		insertKeyPanel.add(Box.createHorizontalGlue());
		contentPane.add(insertKeyPanel);
		JPanel keyPanel = new JPanel();
		keyPanel.setLayout(new BoxLayout(keyPanel, BoxLayout.LINE_AXIS));
		keyArea = new JTextArea();
		keyArea.setColumns(Attachments.getProgramRandomBytesSize() / 2);
		keyArea.getDocument().addDocumentListener(this);
		keyPanel.add(Box.createHorizontalStrut(20));
		keyPanel.add(keyArea);
		contentPane.add(keyPanel);

		contentPane.add(Box.createVerticalStrut(10));
		
		
		JPanel defaultPanel = new JPanel();
		defaultPanel.setLayout(new BoxLayout(defaultPanel, BoxLayout.X_AXIS));
		defaultCheck = new JCheckBox(languageBundle.getString("default_pea"), false);
		if (PeaFactory.isKeyFileProperty() == false) {
			defaultCheck.setSelected(true);
		} else {
			defaultCheck.setSelected(false);
		}
		defaultCheck.addActionListener(this);
		defaultCheck.setActionCommand("default");
		
		defaultPanel.add(defaultCheck);
		defaultPanel.add(Box.createHorizontalGlue());
		contentPane.add(defaultPanel);
		
		JPanel infoDefaultPanel = new JPanel();
		infoDefaultPanel.setLayout(new BoxLayout(infoDefaultPanel, BoxLayout.X_AXIS));
		JTextArea infoDefaultArea = new JTextArea(languageBundle.getString("info_default_pea"));
		infoDefaultArea.setWrapStyleWord(true);
		infoDefaultArea.setLineWrap(true);
		infoDefaultArea.setEditable(false);
		
		infoDefaultPanel.add(Box.createHorizontalStrut(20));
		infoDefaultPanel.add(infoDefaultArea);
		infoDefaultPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoDefaultPanel);		
		contentPane.add(Box.createVerticalStrut(10));
		
		
		JPanel compatibilityPanel = new JPanel();
		compatibilityPanel.setLayout(new BoxLayout(compatibilityPanel, BoxLayout.X_AXIS));
		compatibilityCheck = new JCheckBox(languageBundle.getString("compatible_pea"), false);
		compatibilityCheck.setSelected(false);
		compatibilityCheck.addActionListener(this);
		compatibilityCheck.setActionCommand("nextPeaCompatible");
		
		compatibilityPanel.add(compatibilityCheck);
		compatibilityPanel.add(Box.createHorizontalGlue());
		contentPane.add(compatibilityPanel);
		
		JPanel infoCompatibilityPanel = new JPanel();
		infoCompatibilityPanel.setLayout(new BoxLayout(infoCompatibilityPanel, BoxLayout.X_AXIS));
		JTextArea infoCompatibilityArea = new JTextArea(languageBundle.getString("info_compatible_pea"));
		infoCompatibilityArea.setWrapStyleWord(true);
		infoCompatibilityArea.setLineWrap(true);
		infoCompatibilityArea.setEditable(false);
		
		infoCompatibilityPanel.add(Box.createHorizontalStrut(20));
		infoCompatibilityPanel.add(infoCompatibilityArea);
		infoCompatibilityPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoCompatibilityPanel);		
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel scriptPanel = new JPanel();
		scriptPanel.setLayout(new BoxLayout(scriptPanel, BoxLayout.X_AXIS));
		scriptCheck = new JCheckBox(languageBundle.getString("script_check"), false);
		if (PeaFactory.isUnixScript() == true) {
			scriptCheck.setSelected(true);
		} else {
			scriptCheck.setSelected(false);	
		}
		scriptCheck.addActionListener(this);
		scriptCheck.setActionCommand("unixScript");
		
		scriptPanel.add(scriptCheck);
		scriptPanel.add(Box.createHorizontalGlue());
		contentPane.add(scriptPanel);
		
		JPanel infoScriptPanel = new JPanel();
		infoScriptPanel.setLayout(new BoxLayout(infoScriptPanel, BoxLayout.X_AXIS));
		JTextArea infoScriptArea = new JTextArea(languageBundle.getString("info_script_check"));
		infoScriptArea.setWrapStyleWord(true);
		infoScriptArea.setLineWrap(true);
		infoScriptArea.setEditable(false);
		
		infoScriptPanel.add(Box.createHorizontalStrut(20));
		infoScriptPanel.add(infoScriptArea);
		infoScriptPanel.add(Box.createHorizontalGlue());

		contentPane.add(infoScriptPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		JButton okButton = new JButton("ok");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(okButton);
		
		contentPane.add(buttonPanel);
		
		peaSettings.setSize(new Dimension(550,550));
		//peaSettings.setLocation(PeaFactory.getFrame().getLocation());
		peaSettings.setLocation(MainView.getFrameLocation());
		//peaSettings.pack();
	}

	@Override
	public void actionPerformed(ActionEvent ape) {
		String com = ape.getActionCommand();
		
		if (com.equals("ok")) {
		
			String projectLabelText = "";

			if (defaultCheck.isSelected()) {
				projectLabelText = languageBundle.getString("default_pea");
				PeaFactory.setKeyFileProperty(false);
				PeaFactory.setKeepRandomBytes(false);
				PeaFactory.setUseKeptRandomBytes(false);

			} else if (compatibilityCheck.isSelected() ) {
				projectLabelText = languageBundle.getString("only_compatible_with_previous_pea");
				
				PeaFactory.setKeyFileProperty(true);
				
				// PeaFactoty should keep random bytes for next PEA
				// while keeping these bytes, MainView.isCompatibleWithPreviousPEA is set in PeaFactory
				PeaFactory.setKeepRandomBytes(true);

			} else if (keyFilePropertyCheck.isSelected() ) {
				projectLabelText = languageBundle.getString("key_file_pea");
				
				PeaFactory.setKeyFileProperty(true);
				PeaFactory.setKeepRandomBytes(compatibilityCheck.isSelected());
				
				// set key from keyArea
				if (keyFilePropertyCheck.isSelected()) {

					String keyAsHex = keyArea.getText();
					if (keyAsHex != null && keyAsHex.length() > 0) {
						if (keyAsHex.length() != (Attachments.getProgramRandomBytesSize() * 2)){
							TestLog.e(GeneralPeaSettings.class, "Wrong key size for manually added key: " + keyAsHex.length()
							+ " - must be " + (Attachments.getProgramRandomBytesSize() * 2));
							PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("key")
									+ " " + PeaProperties.getVmBridge().translate("failed") + " (length)", 
									PeaProperties.getVmBridge().translate("error"), 0);
							return;
						}
						//TestLog.o(XXXX.class, "set manually inserted key...");
						PeaFactory.setInsertedKey(keyAsHex);
					}
				}
			}
			
			if (scriptCheck.isSelected() ) {
				PeaFactory.setUnixScript(true);
			} else {
				PeaFactory.setUnixScript(false);
			}
			
			peaSettings.setVisible(false);
			peaSettings.dispose();
			
			// update project label:
			MainView.getInstance(null).updateLabelText(projectLabelText);
			 
		// Check boxes logic
		} else if ( com.startsWith("default")) {	
			
			if (defaultCheck.isSelected()) {
				keyFilePropertyCheck.setSelected(false);
				compatibilityCheck.setSelected(false);
			} else {
				keyFilePropertyCheck.setSelected(true);
			}
			
		} else if ( com.startsWith("keyFilePropertyCheck")) {	
			
			if (keyFilePropertyCheck.isSelected()) {
				//blankCheck.setSelected(false);
				defaultCheck.setSelected(false);
			} else {
				defaultCheck.setSelected(true);
			}
		
		} else if ( com.startsWith("nextPeaCompatible")) {	
		
			if (compatibilityCheck.isSelected()) {				
				keyFilePropertyCheck.setSelected(true);
				//blankCheck.setSelected(false);
				defaultCheck.setSelected(false);
			} 		
		}
	}
	
	private void checkKeyString (String keyString) {
		if (keyString.length() == (Attachments.getProgramRandomBytesSize() * 2)) { // length fits
			if ( keyString.matches("^[0-9a-fA-F]+$") ) { // is HEX
				keyFilePropertyCheck.setSelected(true);
				defaultCheck.setSelected(false);
				keyArea.setBorder(null);
			} else {
				TestLog.e(GeneralPeaSettings.class, "Is not HEX String");
			}
		} else {
			keyArea.setBorder(new LineBorder(Color.RED, 1));
		}
	}

	@Override
	public void changedUpdate(DocumentEvent arg0) {}

	@Override
	public void insertUpdate(DocumentEvent de) {
		String keyString = keyArea.getText();
		if (keyString != null) {
			checkKeyString(keyString);
		}
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		String keyString = keyArea.getText();
		if (keyString != null) {
			checkKeyString(keyString);
		}		
	}
}
