package cologne.eck.peafactory.gui;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main frame for the production of any Password Encryption Archive (pea).
 */

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.*;
import java.util.ResourceBundle;

import javax.swing.*;
import javax.swing.border.LineBorder;


import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.peafactory.control.JarStuff;

import cologne.eck.peafactory.control.PeaFactory;
import cologne.eck.peafactory.control.data_type.DataType;
import cologne.eck.peafactory.data.FactoryPropertyHandler;
import cologne.eck.peafactory.data.PeaFactoryProperties;
import cologne.eck.tools.EntropyPool;
import cologne.eck.tools.KeyRandomCollector;
import cologne.eck.tools.MouseRandomCollector;
import cologne.eck.tools.TestLog;





@SuppressWarnings("serial")
public final class MainView extends JFrame implements ActionListener {

	// static instance
	private static MainView frame; 

	// screen: 
	private static Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	private static double screenWidth = screenSize.getWidth();
	private static double screenHeight = screenSize.getHeight();
	
	private JLabel projectLabel; // Notebook, Image...

	private static JTextField jarNameField;
	private static JTextField labelTextField;	

	public static JProgressBar progressBar;
	
	private static String language; 
	private static ResourceBundle languageBundle;
	
	private static Image image = new ImageIcon("resources" + File.separator + "pea-lock.png").getImage();
	//private static Image cursorImage = new ImageIcon("resources" + File.separator + "wait_cursor.png").getImage();
	
	//private static Cursor waitCursor = Toolkit.getDefaultToolkit().createCustomCursor(cursorImage,
    //        new Point(0, 0), "wait_cursor");			
	
	private MainView (Point location) {
		
		frame = this;
		this.setTitle("Pea Factory");
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.addWindowListener( new WindowAdapter(){
			public void windowClosing(WindowEvent we) {
				// close program if ManView is not instantiated or not visible (program start)
				// else: close this dialog
					new FactoryPropertyHandler().updateAllPropValues();
					TestLog.v(getClass(), "Exit...");
					System.exit(0);
		}});
		//this.setBackground(PswDialogView.getPeaColor());
		
		language = PeaFactory.getLanguage();
		languageBundle = PeaFactory.getLanguagesBundle();
				
		DataType dataType = DataType.getCurrentType();

		//Image icon = new ImageIcon("resources" + File.separator + "pea-lock.png").getImage();
	    this.setIconImage(image);

		JPanel contentPane = (JPanel)this.getContentPane();
		contentPane.setBorder(PeaBorderFactory.createBorder(true));//(new EmptyBorder(5, 15, 5, 15));
		// use frame to collect random values: 
		contentPane.addMouseMotionListener(new MouseRandomCollector() );
		JScrollPane frameScrollPane = new JScrollPane(contentPane);
		this.setContentPane(frameScrollPane);
		
		BoxLayout boxLayout = new BoxLayout(contentPane, BoxLayout.Y_AXIS);
		contentPane.setLayout(boxLayout);
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));		
		// Menu:
		Menu menu = new Menu(languageBundle);
		JLabel languageLabel = new JLabel(language + "  ");
		LanguageMenu languageMenu = new LanguageMenu();
		languageMenu.setToolTipText(languageBundle.getString("tooltip_language"));

		menuPanel.add(menu);		
		menuPanel.add(Box.createHorizontalGlue() );
		menuPanel.add(languageLabel);
		menuPanel.add(languageMenu);

		contentPane.add(menuPanel);
		
		JPanel projectLabelPanel = new JPanel();
		projectLabelPanel.setLayout(new BoxLayout(projectLabelPanel, BoxLayout.X_AXIS));
		projectLabel = new JLabel();
		// show the type of the PEA: notebook, image...
		projectLabel.setText(dataType.getDescription());

		// show property of the PEA:
		if (PeaFactory.isKeyFileProperty() == false) {
			updateLabelText(languageBundle.getString("default_pea"));
		} else {
			if (PeaFactory.isUseKeptRandomBytes() == true) {
				updateLabelText(languageBundle.getString("only_compatible_with_previous_pea"));
			} else {
				updateLabelText(languageBundle.getString("key_file_pea"));
			}
		}
		projectLabel.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 14));
		projectLabel.setBorder(new LineBorder(Color.GRAY, 1));
		projectLabelPanel.add(projectLabel);
		projectLabelPanel.add(Box.createHorizontalGlue());
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(projectLabelPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel jarNamePanel = new JPanel();		
		jarNamePanel.setLayout(new BoxLayout(jarNamePanel, BoxLayout.Y_AXIS));
		
		JLabel jarNameLabel = new JLabel(languageBundle.getString("jar_name_label"));
		jarNameLabel.setPreferredSize(new Dimension(300,30));
		jarNamePanel.add(jarNameLabel);
		
		jarNameField = new JTextField(30);
		jarNameField.setToolTipText(languageBundle.getString("tooltip_jarname"));
		// collect random values from key events:
		jarNameField.addKeyListener(new KeyRandomCollector() );

		jarNamePanel.add(jarNameField);
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(jarNamePanel);
	
		
		JPanel labelTextPanel = new JPanel();		
		labelTextPanel.setLayout(new BoxLayout(labelTextPanel, BoxLayout.Y_AXIS));		
		JLabel abovePswfieldLabel = new JLabel(languageBundle.getString("above_pswfield_label"));
		abovePswfieldLabel.setPreferredSize(new Dimension(300,30));
		labelTextPanel.add(abovePswfieldLabel);		
		labelTextField = new JTextField(30);
		labelTextField.setToolTipText(languageBundle.getString("tooltip_text_above"));
		// collect random values from key events:
		labelTextField.addKeyListener(new KeyRandomCollector() );
		labelTextPanel.add(labelTextField);
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(labelTextPanel);

		
		JPanel newProjectPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
		JButton newProjectButton = new JButton(languageBundle.getString("new_project"));
		newProjectButton.setActionCommand("newProject");
		newProjectButton.addActionListener(this);
		newProjectPanel.add(newProjectButton);
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(newProjectPanel);

		
		JPanel createFilePanel = new JPanel();
		createFilePanel.setLayout(new BoxLayout(createFilePanel, BoxLayout.X_AXIS));
		
		progressBar = new JProgressBar(0,100);
		progressBar.setValue(0);
		progressBar.setStringPainted(false);
		progressBar.setPreferredSize(new Dimension(200, 30));
		progressBar.setBorderPainted(false);// not visible if no task
		createFilePanel.add(progressBar);
		createFilePanel.add(Box.createHorizontalStrut(50));		

		
		JButton createFileButton = new JButton(languageBundle.getString("create_jarfile_button"));
		createFileButton.setActionCommand("create");
		createFileButton.addActionListener(this);
		createFileButton.setToolTipText(languageBundle.getString("tooltip_create"));
		createFilePanel.add(createFileButton);
		contentPane.add(Box.createVerticalStrut(10));
		contentPane.add(createFilePanel);

		if (location == null) {
			setLocation(PeaFactoryProperties.getFrameLocation());		
		} else {
			setLocation(location);
		}
	
		int prefHeight = (int) this.getLayout().preferredLayoutSize(contentPane).getHeight();
		int prefWidth = (int) this.getLayout().preferredLayoutSize(contentPane).getWidth();

		// size of frame depends on screen size: 
		if ( (prefHeight + 120) > screenHeight) { // width + 50, because of scrollpanes' width
			prefHeight = (int) screenHeight - 130;
		}
		if ( (prefWidth + 130) > screenWidth) {
			prefWidth = (int) screenWidth - 150;
		}
		// set JScrollPane size for frame:
		frameScrollPane.setPreferredSize(new Dimension(prefWidth + 50, prefHeight + 50));
	}
	/**
	 * Get the current static instance or create a new one. 
	 * 
	 * @param _location	the location or null (location from properties)
	 * 
	 * @return	the current or new frame
	 */
	public final static MainView getInstance(Point _location) {
		if (frame == null || frame.isShowing() == false) {
			if (_location == null) {
				_location = PeaFactoryProperties.getFrameLocation();
			}
			frame = new MainView(_location);
			return frame;
		} else {
			//TestLog.e(XXXX.class, "MainView: existing instance.");
			return frame;
		}
	}



	@Override
	public void actionPerformed(ActionEvent ape) {
		String com = ape.getActionCommand();
		
		if (com.equals("newProject")) {
			
			Point location = PeaFactoryProperties.getFrameLocation();
			if (frame != null) {
				location = frame.getLocationOnScreen();
			}
			
			ProjectSelection proj = new ProjectSelection(location);
			//proj.setLocation( (int)this.getLocation().getX() + 100, (int)this.getLocation().getY() + 60 );
			proj.setVisible(true); 

		} else if ( com.equals("create")) {	
		
			EntropyPool.getInstance().stopCollection();
			
			if (jarNameField.getText() == null) {
				JarStuff.setJarFileName("default.jar");				
			} else if (jarNameField.getText().length() > 0) {
				String jarText = jarNameField.getText();
				//jarText = jarText.replaceAll("[^a-zA-Z0-9.-]", "_");
				jarText = jarText.replaceAll("[:\\\\/*\"?|<>' $%!#&{}@´`+=();]", "_");
				JarStuff.setJarFileName(jarText + ".jar");
			} else {
				JarStuff.setJarFileName("default.jar");
			}
			//TestLog.o(XXXX.class, "MainView - actionPerformed, jarTextField: "+ jarNameField.getText());			
			//TestLog.o(XXXX.class, "MainView - actionPerformed, jarFileName: "+ JarStuff.getJarFileName());
			
			String labelText = labelTextField.getText();
			labelText = deleteDangerousChars(labelText);
			
			JarStuff.setLabelText(labelText );

			progressBar.setBorderPainted(true);
			progressBar.setStringPainted(true);

			PeaFactory.createFile();
					
			Point location = PeaFactoryProperties.getFrameLocation();
			if (frame != null) {
				location = frame.getLocationOnScreen();
				frame.dispose(); 
				frame = null;
			}				

			ProjectSelection proj = new ProjectSelection(location);
			//proj.setLocation( (int)this.getLocation().getX() + 100, (int)this.getLocation().getY() + 60 );
			proj.setVisible(true); 			
		}	
	}
	
	private String deleteDangerousChars(String text) {
		
		// these characters would allow to run code..!
		text = text.replace('"', ' ');
		text = text.replace('\'', ' ');		
		text = text.replace(';', ',');

		// these characters would cause problems for file names
		text = text.replace('\\', '-'); // windows
		text = text.replace(" ", "-"); // unix 
		text = text.replace('/', '-'); // windows and unix
		text = text.replace('!', '-'); // unix: bash problems
		
		
		// replace escape sequences: these characters will cause compiling problems
		text = text.replace('\t',  ' ');
		text = text.replace('\b', ' ');		
		text = text.replace('\n',  ' ');
		text = text.replace('\r', ' ');
		text = text.replace('\f',  ' ');
		
		return text;
	}

	/**
	 * Dispose the existing frame and create a new one
	 * depending on the new settings. 
	 */
	public final static void updateFrame(Point _location) {
		
		if (frame != null) {
			//_location = frame.getLocationOnScreen();
			frame.setVisible(false);
			frame.dispose(); 
			//frame = null;
		}			
		frame = MainView.getInstance(_location);
		frame.pack();
		frame.setVisible(true);
	}

	/**
	 * The label describing the PEA containing a general PEA description 
	 * is appended with a more detailed description
	 * previous detailed description is overwritten
	 *  
	 * @param newText	values are: languageBundle.getString("default_pea"), 
	 * 					languageBundle.getString("only_compatible_with_previous_pea"),
	 * 					languageBundle.getString("key_file_pea")
	 */
	protected final void updateLabelText(String newText) {
		String labelText =  projectLabel.getText();
		if (labelText.contains(" - ") == true) {
			String firstPart = projectLabel.getText().substring(0, projectLabel.getText().indexOf(" - "));
			projectLabel.setText(firstPart + " - " + newText);
		} else {
			projectLabel.setText(labelText + " - " + newText);
		}
	}
	
	//==========================================
	// Getter & Setter


	/**
	 * Get the current location of the frame
	 * 
	 * @return	the location as point
	 */
	public final static Point getFrameLocation() {
		return frame.getLocation();
	}

	/**
	 * Checks if the instance of MainView is actually displayed or not
	 * 
	 * @return	true if displayed
	 */
	protected final static boolean isDisplayed() {// used in ProjectSelection
		if (frame == null) {
			return false; 
		} else if ( frame.isVisible() == false ) {
			return false;
		} else {
			return true;
		}
	}
	/**
	 * Get the PEA image to use as frame icon. 
	 * 
	 * @return	the PEA image
	 */
	public final static Image getImage() {
		return image;
	}

	/**
	 * Display the wait cursor to indicate that
	 * program is working
	 */
	public static void showWaitCursor() {
		frame.setCursor(CursorManager.getWaitCursor());
	}
	/**
	 * Reset the cursor to the default cursor
	 */
	public static void resetCursor() {
		// reset the cursor:
		frame.setCursor(CursorManager.getDefaultCursor());
	}

	/**
	 * Get the user typed jar name
	 * 
	 * @return	the user typed jar name
	 */
	public static String getJarName() {
		if (jarNameField != null) {
			return jarNameField.getText();
		} else {
			return "";
		}
	}
	/**
	 * Set the name of the jar archive
	 * 
	 * @param _jarName	the name for the jar archive to create
	 */
	public static void setJarName(String _jarName) {
		if (jarNameField != null) {
			jarNameField.setText( _jarName);
		}
	}
	/**
	 * Get the user typed text (hint)
	 * 
	 * @return	the user typed text (hint)
	 */
	public static String getLabelText() {
		if (labelTextField != null) {
			return labelTextField.getText();
		} else {
			return "";
		}
	}
	/**
	 * Set the text (password hint) for the PEA to create
	 * 
	 * @param _labelText	the text (hint)
	 */
	public static void setLabelText(String _labelText) {
		if (labelTextField != null) {
			labelTextField.setText( _labelText);
		}
	}
}
