package cologne.eck.peafactory.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.ResourceBundle;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Menu of MainView. 
 * All setting for pea production. 
 */

//import javax.swing.JMenuBar;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.filechooser.FileNameExtensionFilter;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.PasswordGeneratorDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.menu.PeaMenuBar;
import cologne.eck.peafactory.control.PeaFactory;
import cologne.eck.peafactory.control.CreateJarFile;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.peafactory.crypto.kdf.Argon2KDF;
import cologne.eck.peafactory.crypto.kdf.BcryptKDF;
import cologne.eck.peafactory.crypto.kdf.CatenaKDF;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.peafactory.crypto.kdf.PomeloKDF;
import cologne.eck.peafactory.crypto.kdf.ScryptKDF;
import cologne.eck.peafactory.gui.kdf_settings.Argon2Setting;
import cologne.eck.peafactory.gui.kdf_settings.BcryptSetting;
import cologne.eck.peafactory.gui.kdf_settings.CatenaSetting;
import cologne.eck.peafactory.gui.kdf_settings.PomeloSetting;
import cologne.eck.peafactory.gui.kdf_settings.ScryptSetting;
import cologne.eck.tools.PDFConverter;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.ZipStuff;

import org.bouncycastle.crypto.engines.AESEngine;
import org.bouncycastle.crypto.engines.DSTU7624Engine;
import org.bouncycastle.crypto.engines.LEAEngine;
import org.bouncycastle.crypto.engines.RC6Engine;
import org.bouncycastle.crypto.engines.Shacal2Engine;
import org.bouncycastle.crypto.engines.TwofishEngine;
import org.bouncycastle.crypto.engines.SerpentEngine;
import org.bouncycastle.crypto.engines.ThreefishEngine;
import org.bouncycastle.crypto.digests.Blake2bDigest;
import org.bouncycastle.crypto.digests.Blake3Digest;
import org.bouncycastle.crypto.digests.SHA3Digest;
import org.bouncycastle.crypto.digests.WhirlpoolDigest;//512 bit
import org.bouncycastle.crypto.digests.SkeinDigest;//512
import org.bouncycastle.crypto.digests.SHA512Digest;//512




@SuppressWarnings("serial")
class Menu extends PeaMenuBar implements ActionListener {

	private static int securityLevel = 3;//standard 1-5
	
	private ResourceBundle languageBundle;// = PeaFactory.getLanguagesBundle();
	
	// register if warning message was already shown (show only once)
	private static boolean compatibilityWarningShown = false;
	
	private JDialog peaLangDialog;
	
	protected Menu(ResourceBundle _languageBundle) {
	
		languageBundle = _languageBundle;

//		setBorder(new EtchedBorder() );
		setMinimumSize(new Dimension(200, 50));

		JMenu menu = new JMenu(languageBundle.getString("menu"));
		menu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		menu.setMnemonic(KeyEvent.VK_M);
		this.add(menu);
		
		JMenuItem newItem = new JMenuItem(languageBundle.getString("new_project"));
		newItem.setActionCommand("newProject");
		newItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		newItem.setMnemonic(KeyEvent.VK_N);
		newItem.addActionListener(this);
		menu.add(newItem);
		
		JMenuItem quitItem = new JMenuItem(languageBundle.getString("quit"));
		quitItem.setActionCommand("quit");
		quitItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		quitItem.setMnemonic(KeyEvent.VK_B);
		quitItem.addActionListener(this);
		menu.add(quitItem);
		
		//================== Setting ======================================

		JMenu setMenu = new JMenu(languageBundle.getString("settings"));
		setMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setMenu.addMenuListener(new MenuListener(){
			@Override
			public void menuCanceled(MenuEvent e) {}
			@Override
			public void menuDeselected(MenuEvent e) {}
			@Override
			public void menuSelected(MenuEvent e) {
				
				if (PeaFactory.isKeepRandomBytes() == true && compatibilityWarningShown == false) {
				
					int n = JOptionPane.showConfirmDialog(
						    MainView.getInstance(MainView.getFrameLocation()),
						    languageBundle.getString("compatibility_warning"),
						    "Warning",
						    JOptionPane.OK_CANCEL_OPTION);
					if (n == JOptionPane.OK_OPTION){
						PeaFactory.setKeepRandomBytes(false);
						PeaFactory.setUseKeptRandomBytes(false);
						MainView.getInstance(MainView.getFrameLocation()).updateLabelText(languageBundle.getString("key_file_pea"));						
						compatibilityWarningShown = true;
					} else { // cancel, close
						return;
					}
				}				
			}			
		});
		setMenu.setMnemonic(KeyEvent.VK_S);
		this.add(setMenu);
		

		JMenuItem setPeaSettings = new JMenuItem(languageBundle.getString("general_pea_settings"));
		setPeaSettings.setActionCommand("generalPeaSettings"); 
		setPeaSettings.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setPeaSettings.setMnemonic(KeyEvent.VK_T);
		setPeaSettings.addActionListener(this);
		setMenu.add(setPeaSettings);
		
		JMenuItem setPeaLang = new JMenuItem(languageBundle.getString("set_language_of_pea")); 
		setPeaLang.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setPeaLang.setMnemonic(KeyEvent.VK_S);
		setPeaLang.addActionListener(this);
		setPeaLang.setActionCommand("setPeaLang"); 
		setMenu.add(setPeaLang);		

/*		JMenuItem pswGeneratorItem = new JMenuItem(languageBundle.getString("psw_generator"));
		pswGeneratorItem.setActionCommand("psw_generator");
		pswGeneratorItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		pswGeneratorItem.setMnemonic(KeyEvent.VK_P);
		pswGeneratorItem.addActionListener(this);
		setMenu.add(pswGeneratorItem);*/
		
		// = = = = = =  Level = = = = = = = = = =  
		
		JMenu setLevel = new JMenu(languageBundle.getString("set_security_level")); 
		setLevel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setLevel.setMnemonic(KeyEvent.VK_L);
		setMenu.add(setLevel);		

		JMenuItem setThoughtless = new JMenuItem(languageBundle.getString("thoughtless"));
		setThoughtless.setActionCommand("setThoughtless"); 
		setThoughtless.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setThoughtless.setMnemonic(KeyEvent.VK_T);
		setThoughtless.addActionListener(this);
		setLevel.add(setThoughtless);
		
		JMenuItem setLow = new JMenuItem(languageBundle.getString("low"));
		setLow.setActionCommand("setLow"); 
		setLow.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setLow.setMnemonic(KeyEvent.VK_L);
		setLow.addActionListener(this);
		setLevel.add(setLow);

		JMenuItem setStandard = new JMenuItem(languageBundle.getString("standard"));
		setStandard.setActionCommand("setStandard"); 
		setStandard.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setStandard.setMnemonic(KeyEvent.VK_S);
		setStandard.addActionListener(this);
		setLevel.add(setStandard);

		JMenuItem setHigh = new JMenuItem(languageBundle.getString("high"));
		setHigh.setActionCommand("setHigh"); 
		setHigh.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setHigh.setMnemonic(KeyEvent.VK_H);
		setHigh.addActionListener(this);
		setLevel.add(setHigh);

		JMenuItem setParanoid = new JMenuItem(languageBundle.getString("paranoid"));
		setParanoid.setActionCommand("setParanoid"); 
		setParanoid.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setParanoid.setMnemonic(KeyEvent.VK_P);
		setParanoid.addActionListener(this);
		setLevel.add(setParanoid);
		
		// = = = = = =  Crypto = = = = = = = = = =  

		JMenu cryptMenu = new JMenu(languageBundle.getString("set_crypt")); 
		cryptMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		cryptMenu.setMnemonic(KeyEvent.VK_C);
		setMenu.add(cryptMenu);
		
		// =   =   =   =   =  KDF =   =   =   =   =   = 
		
		JMenu setKeyDerivation = new JMenu(languageBundle.getString("set_kdf")); 
		setKeyDerivation.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setKeyDerivation.setMnemonic(KeyEvent.VK_K);
		cryptMenu.add(setKeyDerivation);		
		
		JMenuItem setArgon2i = new JMenuItem(languageBundle.getString("set_argon2i"));
		setArgon2i.setActionCommand("setArgon2i"); 
		setArgon2i.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setArgon2i.setMnemonic(KeyEvent.VK_I);
		setArgon2i.addActionListener(this);
		setKeyDerivation.add(setArgon2i);	
		
		JMenuItem setArgon2d = new JMenuItem(languageBundle.getString("set_argon2d"));
		setArgon2d.setActionCommand("setArgon2d"); 
		setArgon2d.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setArgon2d.setMnemonic(KeyEvent.VK_D);
		setArgon2d.addActionListener(this);
		setKeyDerivation.add(setArgon2d);	
		
		JMenuItem setArgon2id = new JMenuItem(languageBundle.getString("set_argon2id"));
		setArgon2id.setActionCommand("setArgon2id"); 
		setArgon2id.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setArgon2id.setMnemonic(KeyEvent.VK_D);
		setArgon2id.addActionListener(this);
		setKeyDerivation.add(setArgon2id);	
		
		
		
		JMenuItem setDragonfly = new JMenuItem(languageBundle.getString("set_dragonfly"));
		setDragonfly.setActionCommand("setDragonfly"); 
		setDragonfly.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setDragonfly.setMnemonic(KeyEvent.VK_D);
		setDragonfly.addActionListener(this);
		setKeyDerivation.add(setDragonfly);		
		
		JMenuItem setButterfly = new JMenuItem(languageBundle.getString("set_butterfly"));
		setButterfly.setActionCommand("setButterfly"); 
		setButterfly.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setButterfly.setMnemonic(KeyEvent.VK_U);
		setButterfly.addActionListener(this);
		setKeyDerivation.add(setButterfly);		
		
		JMenuItem setScrypt = new JMenuItem(languageBundle.getString("set_scrypt"));
		setScrypt.setActionCommand("setScrypt"); 
		setScrypt.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setScrypt.setMnemonic(KeyEvent.VK_S);
		setScrypt.addActionListener(this);
		setKeyDerivation.add(setScrypt);		
				
		JMenuItem setPomelo = new JMenuItem(languageBundle.getString("set_pomelo") 
				+ " (" + PeaProperties.getVmBridge().translate("experimental") + ")");
		setPomelo.setActionCommand("setPomelo"); 
		setPomelo.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setPomelo.setMnemonic(KeyEvent.VK_P);
		setPomelo.addActionListener(this);
		setKeyDerivation.add(setPomelo);	
		
		JMenuItem setBcrypt = new JMenuItem(languageBundle.getString("set_bcrypt"));
		setBcrypt.setActionCommand("setBcrypt"); 
		setBcrypt.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setBcrypt.setMnemonic(KeyEvent.VK_B);
		setBcrypt.addActionListener(this);
		setKeyDerivation.add(setBcrypt);		
		
		
		JMenu setKDFParams = new JMenu(languageBundle.getString("set_params"));
		setKDFParams.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setKDFParams.setMnemonic(KeyEvent.VK_P);
		setKeyDerivation.add(setKDFParams);
		
		JMenuItem setCatenaParameters = new JMenuItem(languageBundle.getString("set_catena_params"));
		setCatenaParameters.setActionCommand("setCatenaParameters"); 
		setCatenaParameters.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setCatenaParameters.setMnemonic(KeyEvent.VK_P);
		setCatenaParameters.addActionListener(this);
		setKDFParams.add(setCatenaParameters);
		
		JMenuItem setArgon2Parameters = new JMenuItem(languageBundle.getString("set_argon2_params"));
		setArgon2Parameters.setActionCommand("setArgon2Parameters"); 
		setArgon2Parameters.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setArgon2Parameters.setMnemonic(KeyEvent.VK_P);
		setArgon2Parameters.addActionListener(this);
		setKDFParams.add(setArgon2Parameters);
		
		JMenuItem setScryptParameters = new JMenuItem(languageBundle.getString("set_scrypt_params"));
		setScryptParameters.setActionCommand("setScryptParameters"); 
		setScryptParameters.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setScryptParameters.setMnemonic(KeyEvent.VK_S);
		setScryptParameters.addActionListener(this);
		setKDFParams.add(setScryptParameters);
		
		JMenuItem setPomeloParameters = new JMenuItem(languageBundle.getString("set_pomelo_params"));
		setPomeloParameters.setActionCommand("setPomeloParameters"); 
		setPomeloParameters.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setPomeloParameters.setMnemonic(KeyEvent.VK_P);
		setPomeloParameters.addActionListener(this);
		setKDFParams.add(setPomeloParameters);
		
		JMenuItem setBcryptParameters = new JMenuItem(languageBundle.getString("set_bcrypt_params"));
		setBcryptParameters.setActionCommand("setBcryptParameters"); 
		setBcryptParameters.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setBcryptParameters.setMnemonic(KeyEvent.VK_B);
		setBcryptParameters.addActionListener(this);
		setKDFParams.add(setBcryptParameters);	

		// =   =   =   =   =  Cipher =   =   =   =   =   = 
		
		JMenu setCipherAlgo = new JMenu(languageBundle.getString("set_cipher")); 
		setCipherAlgo.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setCipherAlgo.setMnemonic(KeyEvent.VK_C);
		cryptMenu.add(setCipherAlgo);

		JMenuItem setTwofish = new JMenuItem(languageBundle.getString("set_twofish"));
		setTwofish.setActionCommand("setTwofish"); 
		setTwofish.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setTwofish.setMnemonic(KeyEvent.VK_T);
		setTwofish.addActionListener(this);
		setCipherAlgo.add(setTwofish);		
		
		JMenuItem setAES256 = new JMenuItem(languageBundle.getString("set_aes"));
		setAES256.setActionCommand("setAES"); 
		setAES256.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setAES256.setMnemonic(KeyEvent.VK_A);
		setAES256.addActionListener(this);
		setCipherAlgo.add(setAES256);
		
		JMenuItem setShacal2 = new JMenuItem(languageBundle.getString("set_shacal2"));
		setShacal2.setActionCommand("setShacal2"); 
		setShacal2.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setShacal2.setMnemonic(KeyEvent.VK_H);
		setShacal2.addActionListener(this);
		setCipherAlgo.add(setShacal2);		
		
		JMenuItem setSerpent = new JMenuItem(languageBundle.getString("set_serpent"));
		setSerpent.setActionCommand("setSerpent"); 
		setSerpent.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setSerpent.setMnemonic(KeyEvent.VK_S);
		setSerpent.addActionListener(this);
		setCipherAlgo.add(setSerpent);
		
		JMenuItem setThreefish256 = new JMenuItem(languageBundle.getString("set_threefish256"));
		setThreefish256.setActionCommand("setThreefish256"); 
		setThreefish256.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setThreefish256.setMnemonic(KeyEvent.VK_H);
		setThreefish256.addActionListener(this);
		setCipherAlgo.add(setThreefish256);

		JMenuItem setThreefish512 = new JMenuItem(languageBundle.getString("set_threefish512"));
		setThreefish512.setActionCommand("setThreefish512"); 
		setThreefish512.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setThreefish512.setMnemonic(KeyEvent.VK_R);
		setThreefish512.addActionListener(this);
		setCipherAlgo.add(setThreefish512);

		JMenuItem setThreefish1024 = new JMenuItem(languageBundle.getString("set_threefish1024"));
		setThreefish1024.setActionCommand("setThreefish1024"); 
		setThreefish1024.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setThreefish1024.setMnemonic(KeyEvent.VK_E);
		setThreefish1024.addActionListener(this);
		setCipherAlgo.add(setThreefish1024);		
		
		JMenuItem setLEA = new JMenuItem("LEA");
		setLEA.setActionCommand("setLEA"); 
		setLEA.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setLEA.setMnemonic(KeyEvent.VK_L);
		setLEA.addActionListener(this);
		setCipherAlgo.add(setLEA);	
		
		JMenuItem setRC6 = new JMenuItem("RC6");
		setRC6.setActionCommand("setRC6"); 
		setRC6.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setRC6.setMnemonic(KeyEvent.VK_R);
		setRC6.addActionListener(this);
		setCipherAlgo.add(setRC6);	
		
		JMenuItem setDSTU7624 = new JMenuItem("DSTU7624 (Kalyna)");
		setDSTU7624.setActionCommand("setDSTU"); 
		setDSTU7624.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setDSTU7624.setMnemonic(KeyEvent.VK_D);
		setDSTU7624.addActionListener(this);
		setCipherAlgo.add(setDSTU7624);	

		// =   =   =   =   =  Hash =   =   =   =   =   = 
		
		JMenu setHashAlgo = new JMenu(languageBundle.getString("set_hash")); 
		setHashAlgo.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setHashAlgo.setMnemonic(KeyEvent.VK_H);
		cryptMenu.add(setHashAlgo);

		JMenuItem setBLAKE512 = new JMenuItem(languageBundle.getString("set_blake512"));
		setBLAKE512.setActionCommand("setBlake512"); 
		setBLAKE512.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setBLAKE512.setMnemonic(KeyEvent.VK_B);
		setBLAKE512.addActionListener(this);
		setHashAlgo.add(setBLAKE512); 		
		
		JMenuItem setSkein512 = new JMenuItem(languageBundle.getString("set_skein512"));
		setSkein512.setActionCommand("setSkein512"); 
		setSkein512.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setSkein512.setMnemonic(KeyEvent.VK_S);
		setSkein512.addActionListener(this);
		setHashAlgo.add(setSkein512);	
		
		JMenuItem setWhirlpool = new JMenuItem(languageBundle.getString("set_whirlpool"));
		setWhirlpool.setActionCommand("setWhirlpool"); 
		setWhirlpool.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setWhirlpool.setMnemonic(KeyEvent.VK_W);
		setWhirlpool.addActionListener(this);
		setHashAlgo.add(setWhirlpool);
		
		JMenuItem setKeccak = new JMenuItem(languageBundle.getString("set_sha3") + " (Keccak)");
		setKeccak.setActionCommand("setKeccak"); 
		setKeccak.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setKeccak.setMnemonic(KeyEvent.VK_K);
		setKeccak.addActionListener(this);
		setHashAlgo.add(setKeccak);
		
		JMenuItem setSHA512 = new JMenuItem(languageBundle.getString("set_sha512"));
		setSHA512.setActionCommand("setSha512"); 
		setSHA512.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setSHA512.setMnemonic(KeyEvent.VK_H);
		setSHA512.addActionListener(this);
		setHashAlgo.add(setSHA512);
		
		JMenuItem setBlake3 = new JMenuItem("BLAKE3");
		setBlake3.setActionCommand("setBLAKE3"); 
		setBlake3.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setBlake3.setMnemonic(KeyEvent.VK_3);
		setBlake3.addActionListener(this);
		setHashAlgo.add(setBlake3);
		
		
		//================== Extras ======================================

		JMenu extrasMenu = new JMenu(languageBundle.getString("extras"));
		extrasMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		extrasMenu.setMnemonic(KeyEvent.VK_E);
		this.add(extrasMenu);		
		

		JMenuItem pdfToPng = new JMenuItem(languageBundle.getString("convert_pdf"));
		pdfToPng.setActionCommand("pdfToPng"); 
		pdfToPng.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		pdfToPng.setMnemonic(KeyEvent.VK_P);
		pdfToPng.addActionListener(this);
		extrasMenu.add(pdfToPng);
		

		JMenuItem zipFolder = new JMenuItem(languageBundle.getString("zip_folder"));
		zipFolder.setActionCommand("zipFolder"); 
		zipFolder.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		zipFolder.setMnemonic(KeyEvent.VK_Z);
		zipFolder.addActionListener(this);
		extrasMenu.add(zipFolder);
			
		JMenuItem randomItem = new JMenuItem(languageBundle.getString("random_password"));
		randomItem.setActionCommand("randomPassword");
		randomItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		randomItem.setMnemonic(KeyEvent.VK_R);
		randomItem.addActionListener(this);
		extrasMenu.add(randomItem);
		
		// = = = = =  Language = = = = = = 
		
		JMenu setLanguage = new JMenu(languageBundle.getString("set_language")); 
		setLanguage.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setLanguage.setMnemonic(KeyEvent.VK_L);
		extrasMenu.add(setLanguage);
		
		JMenuItem setDE = new JMenuItem(languageBundle.getString("set_de"));
		setDE.setActionCommand("setDE"); 
		setDE.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setDE.setMnemonic(KeyEvent.VK_D);
		setDE.addActionListener(this);
		setLanguage.add(setDE);

		JMenuItem setEN = new JMenuItem(languageBundle.getString("set_en"));
		setEN.setActionCommand("setEN"); 
		setEN.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		setEN.setMnemonic(KeyEvent.VK_E);
		setEN.addActionListener(this);
		setLanguage.add(setEN);

		
		//================== Help ======================================		

		JMenu helpMenu = new JMenu(languageBundle.getString("help"));
		helpMenu.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		helpMenu.setMnemonic(KeyEvent.VK_H);
		this.add(helpMenu);
		
		
		JMenuItem webHelp = new JMenuItem(languageBundle.getString("web_help"));
		webHelp.setActionCommand("webHelp");
		webHelp.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		webHelp.setMnemonic(KeyEvent.VK_P);
		webHelp.addActionListener(this);
		helpMenu.add(webHelp); 

		JMenuItem problemHelp = new JMenuItem(languageBundle.getString("problem_help"));
		problemHelp.setActionCommand("problemHelp");
		problemHelp.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		problemHelp.setMnemonic(KeyEvent.VK_P);
		problemHelp.addActionListener(this);
		helpMenu.add(problemHelp); 
		
		JMenuItem aboutLicense = new JMenuItem(languageBundle.getString("about_license"));
		aboutLicense.setActionCommand("aboutLicense");
		aboutLicense.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		aboutLicense.setMnemonic(KeyEvent.VK_L);
		aboutLicense.addActionListener(this);
		helpMenu.add(aboutLicense);		
	}
	

	@Override
	public void actionPerformed(ActionEvent ape) {
		
		//JComponent source = (JComponent) ape.getSource();
		String command = ape.getActionCommand();
		TestLog.v(getClass(), "Pea settings: "  + command);
		
		//Menu
		if (command.equals("newProject")) {
			ProjectSelection proj = new ProjectSelection(MainView.getFrameLocation());
			Point p = MainView.getFrameLocation();
			proj.setLocation( (int)p.getX() + 100, (int)p.getY() + 60 );
			proj.setVisible(true);
		} else if (command.equals("randomPassword")) {			
			PasswordGeneratorDialog pg = new PasswordGeneratorDialog(MainView.getInstance(MainView.getFrameLocation()) );
			pg.setVisible(true);

		} else if (command.equals("quit")) {
			System.exit(0);
			
		} else if (command.equals("generalPeaSettings")) {
			
			GeneralPeaSettings imageSetting = new GeneralPeaSettings();
			imageSetting.setVisible(true);
			
		} else if (command.equals("setPeaLang")) {
						
			// TODO own class, improve GUI, properties for PeaFactory...
			peaLangDialog = new JDialog();
			
			JPanel p = (JPanel) peaLangDialog.getContentPane();
			p.setBorder(new EmptyBorder(5,5,5,5));

			p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));
			p.add(new JLabel(languageBundle.getString("set_language_of_pea") + ": "));
			p.add(Box.createVerticalStrut(10));
			
			// get bundles from resource directory in file system:
			File resDir = new File(System.getProperty("user.dir") + File.separator 
					+ "resources");
			File[] resList = resDir.listFiles();
			if (resList != null) {
				for (File d : resList){
					if (d.isDirectory()) {
						if (d.getName().length() == 2) { // country codes only
							File[] resDirDir = d.listFiles();
							if (resDirDir != null) {
								for (File f : resDirDir) {
									String fileName = f.getName();

									if (fileName.startsWith("PeaLanguagesBundle_")){
										String langString = fileName.substring("PeaLanguagesBundle_".length(), fileName.indexOf("."));
										// default: add lang								
										CreateJarFile.addLang(langString);
										//TestLog.v(Menu.class, "Add language: " + langString);					
										JCheckBox c = new JCheckBox(langString);		
										c.setSelected(true);
										c.addActionListener(this);
										c.setActionCommand("LANG_" + langString);									
										p.add(c);
									}						
								}
							}
						}
					}
				}
			}
			p.add(Box.createVerticalStrut(10));

			JButton langOk = new JButton("OK");
			langOk.addActionListener(this);
			langOk.setActionCommand("SET_PEA_LANG");
			peaLangDialog.add(langOk);

			peaLangDialog.pack();
			peaLangDialog.setLocation(MainView.getFrameLocation());

			peaLangDialog.setVisible(true);
			
		} else if (command.startsWith("LANG_")) {	
			if (ape.getSource() instanceof JCheckBox) {
				String lang = command.substring("LANG_".length(), command.length());
				System.out.println("Lang: " + lang);
				JCheckBox c = (JCheckBox) ape.getSource();
				if (c.isSelected()) {
					CreateJarFile.addLang(lang);
				} else {
					CreateJarFile.removeLang(lang);
				}
			}
		} else if (command.equals("SET_PEA_LANG")) {		
			peaLangDialog.setVisible(false);
			peaLangDialog.dispose();
			
		} else if (command.equals("setThoughtless")) {			
			securityLevel = 1;
			setSecurityLevel(1);
		} else if (command.equals("setLow")) {
			securityLevel = 2;
			setSecurityLevel(2);
		} else if (command.equals("setStandard")) {
			securityLevel = 3;
			setSecurityLevel(3);
		} else if (command.equals("setHigh")) {
			securityLevel = 4;
			setSecurityLevel(4);		
		} else if (command.equals("setParanoid")) {
			securityLevel = 5;
			setSecurityLevel(5);
						
		} else if (command.equals("setBcrypt")) {
			setSecurityLevel(securityLevel);
			KeyDerivation.setKdf( new BcryptKDF() );
		} else if (command.equals("setScrypt")) {
			setSecurityLevel(securityLevel);
			KeyDerivation.setKdf( new ScryptKDF() );
		} else if (command.equals("setDragonfly")) {
			setSecurityLevel(securityLevel);
			CatenaKDF.setVersionID("Dragonfly-Full");
			KeyDerivation.setKdf( new CatenaKDF() );
		} else if (command.equals("setButterfly")) {
			setSecurityLevel(securityLevel);
			CatenaKDF.setVersionID("Butterfly-Full");
			KeyDerivation.setKdf( new CatenaKDF() );
			
		} else if (command.equals("setArgon2id")){
			setSecurityLevel(securityLevel);
			Argon2KDF.setType(2);
			KeyDerivation.setKdf( new Argon2KDF() );
		} else if (command.equals("setArgon2i")){
			setSecurityLevel(securityLevel);
			Argon2KDF.setType(1);
			KeyDerivation.setKdf( new Argon2KDF() );
		} else if (command.equals("setArgon2d")){
			setSecurityLevel(securityLevel);
			Argon2KDF.setType(0);
			KeyDerivation.setKdf( new Argon2KDF() );
			
		} else if (command.equals("setPomelo")) {
			setSecurityLevel(securityLevel);
			KeyDerivation.setKdf( new PomeloKDF() );
			
		} else if (command.equals("setBcryptParameters")) {

			@SuppressWarnings("unused")
			BcryptSetting bcryptSetting = new BcryptSetting(SwingUtilities.getWindowAncestor(this));
			
		} else if (command.equals("setPomeloParameters")) {

			@SuppressWarnings("unused")
			PomeloSetting pomeloSetting = new PomeloSetting(SwingUtilities.getWindowAncestor(this));

		} else if (command.equals("setScryptParameters")) {
			
			@SuppressWarnings("unused")
			ScryptSetting scryptSetting = new ScryptSetting(SwingUtilities.getWindowAncestor(this));

		} else if (command.equals("setCatenaParameters")) {
			
			@SuppressWarnings("unused")
			CatenaSetting catenaSetting = new CatenaSetting(SwingUtilities.getWindowAncestor(this));

		} else if (command.equals("setArgon2Parameters")) {
			
			@SuppressWarnings("unused")
			Argon2Setting argon2Setting = new Argon2Setting(SwingUtilities.getWindowAncestor(this));
			 
		} else if (command.equals("setShacal2")) {
			CipherStuff.setCipherAlgo( new Shacal2Engine() ); 
		} else if (command.equals("setThreefish256")) {
			CipherStuff.setCipherAlgo( new ThreefishEngine(256) ); 
		} else if (command.equals("setThreefish512")) {
			CipherStuff.setCipherAlgo( new ThreefishEngine(512) ); 
		} else if (command.equals("setThreefish1024")) {
			CipherStuff.setCipherAlgo( new ThreefishEngine(1024) ); 
		} else if (command.equals("setTwofish")) {
			CipherStuff.setCipherAlgo( new TwofishEngine() ); 
		} else if (command.equals("setSerpent")) {
			CipherStuff.setCipherAlgo( new SerpentEngine() ); 
		} else if (command.equals("setAES")) {
			CipherStuff.setCipherAlgo( new AESEngine() ); 
		} else if (command.equals("setLEA")) {
			CipherStuff.setCipherAlgo( new LEAEngine() ); 
		} else if (command.equals("setRC6")) {
			CipherStuff.setCipherAlgo( new RC6Engine() ); 
		} else if (command.equals("setDSTU")) {
			CipherStuff.setCipherAlgo( new DSTU7624Engine(512) ); 

			
		// hash function:
		} else if (command.equals("setWhirlpool")) {
			HashStuff.setHashAlgo( new WhirlpoolDigest() ); 
			//HashStuff.setHMacAlgo( new WhirlpoolDigest() ); 
		} else if (command.equals("setKeccak")) {
			HashStuff.setHashAlgo( new SHA3Digest() );
			//HashStuff.setHMacAlgo( new SHA3Digest() ); 
		} else if (command.equals("setSha512")) {
			HashStuff.setHashAlgo( new SHA512Digest() ); 
			//HashStuff.setHMacAlgo( new SHA512Digest() ); 
		} else if (command.equals("setSkein512")) {
			HashStuff.setHashAlgo( new SkeinDigest(512, 512) );
			//HashStuff.setHMacAlgo( new SkeinDigest(512, 512) ); 
		} else if (command.equals("setBlake512")) {
			HashStuff.setHashAlgo( new Blake2bDigest() );
			//HashStuff.setHMacAlgo( new Blake2bDigest() ); 	
		} else if (command.equals("setBLAKE3")) {
			HashStuff.setHashAlgo( new Blake3Digest() );
			
		} else if (command.equals("setDE")) {
			PeaFactory.setI18n("de");
		} else if (command.equals("setEN")) {
			PeaFactory.setI18n("en");
			
		} else if (command.equals("pdfToPng")) {			
    		
			// setting for the images: type and scale
	        PDFConverterSetting setting = new PDFConverterSetting(PeaFactory.getFrame());
			
	        // select pdf file:
			PeaFileChooser chooser = new PeaFileChooser(this);
			chooser.customizeFileChooser("Convert PDF file", "select pdf file to convert" , 
					false);
			FileNameExtensionFilter pdfFilter = new FileNameExtensionFilter(
                    "PDF", "pdf");			
			chooser.setFileFilter(pdfFilter);       
			// chooser.setCurrentDirectory(new File("/home/Documents"));
			int retrival = chooser.showOpenDialog(this);

			if (retrival == JFileChooser.APPROVE_OPTION) {	
    	
				File selectedFile = chooser.getSelectedFile();
				
				try {
					// indicate that program is working
					MainView.showWaitCursor();			
		    		// set progressBar
					MainView.progressBar.setValue(10);
					MainView.progressBar.paint(MainView.progressBar.getGraphics());
	    		
					// initialize converter with file
		    		PDFConverter converter = new PDFConverter(selectedFile);
		    		// set format (PNG...) and scale from settings
		    		converter.setImageFormatName(setting.getImageFormat());
		    		converter.setScale(setting.getScale());
		    							
		    		//convert file to images
		    		converter.convertAll();

				} finally {
					MainView.resetCursor();
				}
	    		JOptionPane.showMessageDialog(
	    				SwingUtilities.getWindowAncestor(this), 
	    				PeaFactory.getLanguagesBundle().getString("pdf_to_image_ready") + "\n" 
	    				+ selectedFile.getPath() + "\n"
	    				+ PeaFactory.getLanguagesBundle().getString("find_images_along_with_pdf_file"), 
	    				"Info", 
	    				JOptionPane.INFORMATION_MESSAGE);

			}
		} else if (command.equals("zipFolder")){
			
	        // select pdf file:
			PeaFileChooser chooser = new PeaFileChooser(this);
			chooser.customizeFileChooser("Create zip file", "select folder" , 
					false);
			chooser.setFileSelectionMode( JFileChooser.FILES_AND_DIRECTORIES );       
			// chooser.setCurrentDirectory(new File("/home/Documents"));
			int retrival = chooser.showOpenDialog(this);

			if (retrival == JFileChooser.APPROVE_OPTION) {	

    	
				File selectedFile = chooser.getSelectedFile();
				String zipName = null;
				
                if (selectedFile.isFile()) {
                    return;
                } else {
                    chooser.approveSelection();
            	}
				
				try {
					zipName = JOptionPane.showInputDialog(
							PeaFactory.getFrame(), 
							PeaFactory.getLanguagesBundle().getString("type_name_of_zip")); 
					
					if (zipName == null || zipName.trim().length() == 0) {
						JOptionPane.showMessageDialog(PeaFactory.getFrame(), 
								PeaFactory.getLanguagesBundle().getString("must_select_file"));
						zipName = JOptionPane.showInputDialog(
								PeaFactory.getFrame(), 
								PeaFactory.getLanguagesBundle().getString("type_name_of_zip")); 
						if (zipName == null || zipName.trim().length() == 0) {
							return;
						}
					}
						
					// indicate that program is working
					MainView.showWaitCursor();			
		    		// set progressBar
					MainView.progressBar.setValue(50);
					MainView.progressBar.paint(MainView.progressBar.getGraphics());

					
					String success = ZipStuff.zipFolder(selectedFile.getAbsolutePath(), zipName, PeaFactory.getFrame());
					if (success != null) {
						JOptionPane.showMessageDialog(PeaFactory.getFrame(), 
								PeaFactory.getLanguagesBundle().getString("zip_creation_failed") + ": \n"
								+ success,
								PeaFactory.getLanguagesBundle().getString("error"), 
								JOptionPane.ERROR_MESSAGE);			
						
						MainView.resetCursor();
						MainView.progressBar.setValue(0);
						MainView.progressBar.paint(MainView.progressBar.getGraphics());

						return;
					}

				} finally {
					MainView.resetCursor();
					MainView.progressBar.setValue(0);
					MainView.progressBar.paint(MainView.progressBar.getGraphics());

				}
	    		JOptionPane.showMessageDialog(
	    				PeaFactory.getFrame(), 
	    				PeaFactory.getLanguagesBundle().getString("zip_ready") + "\n" 
	    				+ zipName + "\n"
	    				+ PeaFactory.getLanguagesBundle().getString("find_zip_along_with_folder"), 
	    				"Info", 
	    				JOptionPane.INFORMATION_MESSAGE);
		
			}

		} else if (command.equals("keyboard_info")) {
			@SuppressWarnings("unused")
			InfoDialog info = new InfoDialog(
					"Onscreen Keyboard", 
					null, 
					"keyboard");
		} else if (command.equals("file")) {
			@SuppressWarnings("unused")
			InfoDialog info = new InfoDialog(languageBundle.getString(
					"file_description"), 
					null, 
					"file");
		} else if (command.equals("webHelp")) {
			@SuppressWarnings("unused")
			InfoDialog info = new InfoDialog(
					languageBundle.getString("web_help"), 
					languageBundle.getString("web_dialog"), 
					null);			
		} else if (command.equals("problemHelp")) {
			@SuppressWarnings("unused")
			InfoDialog info = new InfoDialog(
					languageBundle.getString("problem_help"), 
					languageBundle.getString("problem_help_dialog"), 
					null);	
		} else if (command.equals("aboutLicense")) {
			@SuppressWarnings("unused")
			InfoDialog info = new InfoDialog(
					languageBundle.getString("about_license"), 
					languageBundle.getString("about_license_dialog"), 
					"GPL");
		} 
	}	
	
	//
	// this will set the Level for all KDF schemes
	private void setSecurityLevel(int securityLevel){
		if (securityLevel == 1) {
			//// ca. 40ms
			BcryptKDF.setRounds(6);
			
			CatenaKDF.setGarlicDragonfly(14);
			CatenaKDF.setLambdaDragonfly(2);
			CatenaKDF.setGarlicButterfly(12);
			CatenaKDF.setLambdaButterfly(4);
			
			PomeloKDF.setMemoryCost(10);
			PomeloKDF.setTimeCost(0);
			
			ScryptKDF.setcPUFactor(4096);
			ScryptKDF.setMemoryFactor(8);
			ScryptKDF.setParallelFactor(1);
		
			Argon2KDF.setLanes(2);
			Argon2KDF.setMemoryCost(32 * 1024);
			Argon2KDF.setIterations(3);
	
		}
		if (securityLevel == 2) {
			//// ca. 120ms
			BcryptKDF.setRounds(10);
			
			CatenaKDF.setGarlicDragonfly(16);
			CatenaKDF.setLambdaDragonfly(2);
			CatenaKDF.setGarlicButterfly(13);
			CatenaKDF.setLambdaButterfly(4);
			
			PomeloKDF.setMemoryCost(14);
			PomeloKDF.setTimeCost(0);
			
			ScryptKDF.setcPUFactor(16384);
			ScryptKDF.setMemoryFactor(8);
			ScryptKDF.setParallelFactor(1);
			
			Argon2KDF.setLanes(4);
			Argon2KDF.setMemoryCost(64 * 1024);
			Argon2KDF.setIterations(3);
		}
		if (securityLevel == 3) {
			//// ca. 450ms
			BcryptKDF.setRounds(12);			
			
			CatenaKDF.setGarlicDragonfly(18);
			CatenaKDF.setLambdaDragonfly(2);
			CatenaKDF.setGarlicButterfly(14);
			CatenaKDF.setLambdaButterfly(4);
			
			PomeloKDF.setMemoryCost(15);
			PomeloKDF.setTimeCost(0);
			
			ScryptKDF.setcPUFactor(16384);
			ScryptKDF.setMemoryFactor(32);
			ScryptKDF.setParallelFactor(1);			
			
			Argon2KDF.setLanes(4);
			Argon2KDF.setMemoryCost(128 * 1024);
			Argon2KDF.setIterations(3);
		}
		if (securityLevel == 4) {
			//// ca. 1,4s (bei 13: 800ms)
			BcryptKDF.setRounds(14);
			
			CatenaKDF.setGarlicDragonfly(20);
			CatenaKDF.setLambdaDragonfly(2);
			CatenaKDF.setGarlicButterfly(16);
			CatenaKDF.setLambdaButterfly(4);
			
			PomeloKDF.setMemoryCost(16);
			PomeloKDF.setTimeCost(1);
			
			ScryptKDF.setcPUFactor(16384);
			ScryptKDF.setMemoryFactor(64);
			ScryptKDF.setParallelFactor(1);					
			
			Argon2KDF.setLanes(4);
			Argon2KDF.setMemoryCost(256 * 1024);
			Argon2KDF.setIterations(4);
		}
		if (securityLevel == 5) {
			//// ca. 5,5s (bei 15: 2,7s, bei 17: 11s)
			BcryptKDF.setRounds(16);
			
			CatenaKDF.setGarlicDragonfly(22);
			CatenaKDF.setLambdaDragonfly(2);
			CatenaKDF.setGarlicButterfly(18);
			CatenaKDF.setLambdaButterfly(4);
			
			PomeloKDF.setMemoryCost(17);
			PomeloKDF.setTimeCost(2);
			
			ScryptKDF.setcPUFactor(16384);
			ScryptKDF.setMemoryFactor(256);
			ScryptKDF.setParallelFactor(1);			
			
			Argon2KDF.setLanes(8);
			Argon2KDF.setMemoryCost(512 * 1024);
			Argon2KDF.setIterations(6);
		}
		// the instances of KeyDerivation must be updated with the new parameters
		updateKDF();
	}
	// Set a new instance of the current instance
	private void updateKDF(){
		if (KeyDerivation.getKdf() instanceof ScryptKDF){
			KeyDerivation.setKdf(new ScryptKDF() );
		} else if (KeyDerivation.getKdf() instanceof BcryptKDF){
			KeyDerivation.setKdf(new BcryptKDF() );
		} else if (KeyDerivation.getKdf() instanceof CatenaKDF){
			KeyDerivation.setKdf(new CatenaKDF() );
		} else if (KeyDerivation.getKdf() instanceof PomeloKDF){
			KeyDerivation.setKdf(new PomeloKDF() );
		}
	}
}
