package cologne.eck.peafactory.gui;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.EmptyBorder;

import cologne.eck.peafactory.control.PeaFactory;



@SuppressWarnings("serial")
class PDFConverterSetting extends JDialog 
		implements ActionListener {
	
	private JSpinner spinner;
	
	private String imageFormat = "PNG";
	private int scale = 4;
	
	PDFConverterSetting(Window parent) {
	
		super(parent);
		this.setAlwaysOnTop(true);
		this.setModalityType(JDialog.ModalityType.TOOLKIT_MODAL);
		this.setIconImage(MainView.getImage());
		
		JPanel contentPane = (JPanel) this.getContentPane();
		contentPane.setBorder(new EmptyBorder(5,5,5,5));
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		
		JPanel typeLabelPanel= new JPanel();
		typeLabelPanel.setLayout(new BoxLayout(typeLabelPanel, BoxLayout.LINE_AXIS));
		JLabel typeLabel = new JLabel(PeaFactory.getLanguagesBundle().getString("set_image_type"));
		typeLabelPanel.add(typeLabel);
		typeLabelPanel.add(Box.createHorizontalGlue());
		contentPane.add(typeLabelPanel);
		
		JPanel imageTypeSelection = new JPanel();
		// JPEG, PNG, GIF, BMP and WBMP
		JRadioButton pngButton = new JRadioButton("PNG");
		pngButton.addActionListener(this);
		pngButton.setActionCommand("PNG");
		pngButton.setSelected(true);
		JRadioButton jpegButton = new JRadioButton("JPEG");
		jpegButton.addActionListener(this);
		jpegButton.setActionCommand("JPEG");
		JRadioButton gifButton = new JRadioButton("GIF");
		gifButton.addActionListener(this);
		gifButton.setActionCommand("GIF");
		JRadioButton bmpButton = new JRadioButton("BMP");
		bmpButton.addActionListener(this);
		bmpButton.setActionCommand("BMP");
/*		JRadioButton wempButton = new JRadioButton("WBMP");		
		wempButton.addActionListener(this);
		wempButton.setActionCommand("WBMP");*/
		
		//Group the radio buttons.
	    ButtonGroup group = new ButtonGroup();
	    group.add(pngButton);
	    group.add(jpegButton);
	    group.add(gifButton);
	    group.add(bmpButton);
//	    group.add(wempButton);
	    imageTypeSelection.add(pngButton);
	    imageTypeSelection.add(jpegButton);
	    imageTypeSelection.add(gifButton);
	    imageTypeSelection.add(bmpButton);
//	    imageTypeSelection.add(wempButton);
	    contentPane.add(imageTypeSelection);
	    
	    JPanel scaleLabelPanel= new JPanel();
	    scaleLabelPanel.setLayout(new BoxLayout(scaleLabelPanel,BoxLayout.LINE_AXIS));
	    JLabel scaleLabel = new JLabel(PeaFactory.getLanguagesBundle().getString("set_image_scale"));
	    scaleLabelPanel.add(scaleLabel);
	    scaleLabelPanel.add(Box.createHorizontalGlue());
	    contentPane.add(scaleLabelPanel);
	    
	    JPanel scaleSpinnerPanel = new JPanel();

	    SpinnerNumberModel model = new SpinnerNumberModel();
	    spinner = new JSpinner(model);
	    model.setMinimum(1);
	    model.setMaximum(12);
	    
	    spinner.setValue( 4 );
	    spinner.setPreferredSize(new Dimension(60, 30));
	    scaleSpinnerPanel.add(spinner);
	    contentPane.add(scaleSpinnerPanel);
	    
		JPanel okButtonPanel= new JPanel();
		okButtonPanel.setLayout(new BoxLayout(okButtonPanel, BoxLayout.LINE_AXIS));
	    JButton okButton = new JButton("ok");
	    okButton.addActionListener(this);
	    okButton.setActionCommand("ok");
		okButtonPanel.add(Box.createHorizontalGlue());
		okButtonPanel.add(okButton);
	    contentPane.add(okButtonPanel);
	    
	    this.pack();//setSize(new Dimension(350,220));
		this.setLocation(parent.getLocation());
		this.setVisible(true);	
	}

	@Override
	public void actionPerformed(ActionEvent ape) {
		String command = ape.getActionCommand();
		if (command.equals("ok")) {
			scale = ((Integer) ( (SpinnerNumberModel) spinner.getModel()).getValue());
			this.dispose();
		} else {
			imageFormat = command;
		}
		
	}

	/**
	 * @return the imageFormat
	 */
	public String getImageFormat() {
		return imageFormat;
	}

	/**
	 * @return the scale
	 */
	public int getScale() {
		return scale;
	}
}
