package cologne.eck.peafactory.gui;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/*
 * GUI for selection of project type, a type of a Password Encrypted Archive (pea)
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;

import javax.imageio.ImageIO;
import javax.swing.*;


import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.peafactory.control.PeaFactory;
import cologne.eck.peafactory.control.data_type.CalendarType;
import cologne.eck.peafactory.control.data_type.DataType;
import cologne.eck.peafactory.control.data_type.EditorType;
import cologne.eck.peafactory.control.data_type.FileType;
import cologne.eck.peafactory.control.data_type.ImageType;
import cologne.eck.peafactory.data.FactoryPropertyHandler;
import cologne.eck.peafactory.data.PeaFactoryProperties;
import cologne.eck.tools.MouseRandomCollector;
import cologne.eck.tools.TestLog;






@SuppressWarnings("serial")
public class ProjectSelection extends JDialog 
		implements ActionListener, ItemListener{
	
	private ProjectSelection projectSelection;
	
	private JRadioButton notesButton;
	//private JCheckBox simpleNotesCheck;


	private ResourceBundle languageBundle;
	
	public ProjectSelection(Point location) {
		
		projectSelection = this;

		// i18n
		languageBundle = PeaFactory.getLanguagesBundle();

		this.setAlwaysOnTop(false);
		this.setTitle(languageBundle.getString("project_selection_title"));
		// to close program if this closed after start
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		this.addWindowListener( new WindowAdapter(){
			public void windowClosing(WindowEvent we) {
				new FactoryPropertyHandler().updateAllPropValues();
				// close program if ManView is not instantiated or not visible (program start)
				// else: close this dialog
				if (PeaFactory.isFrameInstantiated() == false  || MainView.isDisplayed() == false) {					
					TestLog.v(getClass(), "Exit...");
					System.exit(0);
				} else {
					projectSelection.dispose();
				}	
		}});
		
		// use frame to collect random values: 
		this.addMouseMotionListener(new MouseRandomCollector());

		this.setIconImage(MainView.getImage());
		
/*		Border compoundBorder = BorderFactory.createCompoundBorder(
				BorderFactory.createMatteBorder(5, 5, 5, 5, new Color(242, 255, 243)), 
                BorderFactory.createMatteBorder(5, 5, 5, 5, new Color(218, 242, 220)));
*/
		
		
		JPanel contentPane = (JPanel) this.getContentPane();
		contentPane.setBorder(PeaBorderFactory.createBorder(false));//new EmptyBorder(10,10,10,10));
//		contentPane.setBorder(compoundBorder);
		
	    //--------------//
	    // COMPONENTS:  //
	    //--------------//		
		
		JLabel typeLabel = new JLabel();
		typeLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 16));
		typeLabel.setText(languageBundle.getString("selection_type_label"));
		
		BufferedImage image = null;
		 try {
			//image = ImageIO.read(new File("src" + File.separator + "resources" + File.separator + "pea-factory.png"));
			image = ImageIO.read(new File("resources" + File.separator + "peafactory.png"));

		} catch (IOException e) {
			TestLog.e(ProjectSelection.class, "ProjectSelection: image read failed");
		}	 
		 JLabel imageLabel = null;
		if (image != null) {
			Image scaledImage = image.getScaledInstance(256, 128, Image.SCALE_FAST);//92, 92, Image.SCALE_FAST);
			imageLabel = new JLabel(new ImageIcon(scaledImage));
		} else {
			imageLabel = new JLabel();
		}
		
	    
		JRadioButton fileButton = new JRadioButton();
		fileButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		fileButton.setText(languageBundle.getString("selection_file_button"));
		fileButton.setToolTipText(languageBundle.getString("tooltip_selection_file_button"));
	    fileButton.setMnemonic(KeyEvent.VK_F);
	    fileButton.setMargin(new Insets(10,10,0,10));//oben, links, unten, rechts
	    fileButton.setActionCommand("file");
	    fileButton.addItemListener(this);

		notesButton = new JRadioButton();
		notesButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		notesButton.setText(languageBundle.getString("selection_notes_button"));
		notesButton.setToolTipText(languageBundle.getString("tooltip_selection_notes_button"));
	    notesButton.setMnemonic(KeyEvent.VK_N);
	    notesButton.setMargin(new Insets(10,10,0,10));//oben, links, unten, rechts
	    notesButton.setActionCommand("notes");
	    notesButton.addItemListener(this);
	    
	    JRadioButton imageButton = new JRadioButton();
		imageButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		imageButton.setText(languageBundle.getString("selection_image_button"));
		imageButton.setToolTipText(languageBundle.getString("tooltip_selection_image_button"));
	    imageButton.setMnemonic(KeyEvent.VK_I);
	    imageButton.setMargin(new Insets(10,10,20,10));//oben, links, unten, rechts
	    imageButton.setActionCommand("image");
	    imageButton.addItemListener(this);	    	    
	    
	    JRadioButton calendarButton = new JRadioButton();
		calendarButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		calendarButton.setText(languageBundle.getString("selection_calendar_button"));
		calendarButton.setToolTipText(languageBundle.getString("tooltip_selection_calendar_button"));
	    calendarButton.setMnemonic(KeyEvent.VK_C);
	    calendarButton.setMargin(new Insets(10,10,20,10));//oben, links, unten, rechts
	    calendarButton.setActionCommand("calendar");
	    calendarButton.addItemListener(this);
	    
	    ButtonGroup typeGroup = new ButtonGroup();
	    typeGroup.add(notesButton);
	    typeGroup.add(imageButton);
	    typeGroup.add(fileButton);	    
	    typeGroup.add(calendarButton);

	    
	    fileButton.setSelected(true);//NullPointer if before pswChangeCheck is initialized
	    
	    JButton okButton = new JButton();
	    okButton.setText("ok");
	    okButton.addActionListener(this);
	    okButton.setActionCommand("ok");
	    okButton.setToolTipText(languageBundle.getString("tooltip_project_ok_button"));
	    	    
	    //-----------//
	    // LAYOUT:   //
	    //-----------//
	    
	    contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
	    
	    JPanel typeLabelPanel = new JPanel();
	    typeLabelPanel.setLayout(new BoxLayout(typeLabelPanel, BoxLayout.X_AXIS));
	    typeLabelPanel.add(typeLabel);
	    typeLabelPanel.add(Box.createHorizontalGlue());
	    contentPane.add(typeLabelPanel);
	    contentPane.add(Box.createVerticalStrut(10));
	   
		JPanel imagePanel = new JPanel();
		imagePanel.setLayout(new BoxLayout(imagePanel, BoxLayout.X_AXIS));
		imagePanel.add(Box.createHorizontalGlue());
		imagePanel.add(imageLabel);
//		imagePanel.add(Box.createHorizontalStrut(10));
		contentPane.add(imagePanel);
//		contentPane.add(Box.createVerticalStrut(10));
	    
	    JPanel fileButtonPanel = new JPanel();
	    fileButtonPanel.setLayout(new BoxLayout(fileButtonPanel, BoxLayout.X_AXIS));
	    fileButtonPanel.add(fileButton);
	    fileButtonPanel.add(Box.createHorizontalGlue());
	    contentPane.add(fileButtonPanel);	    
	    //contentPane.add(Box.createVerticalStrut(20));	    
	    
	    JPanel textButtonPanel = new JPanel();
	    textButtonPanel.setLayout(new BoxLayout(textButtonPanel, BoxLayout.X_AXIS));
	    textButtonPanel.add(notesButton);
	    textButtonPanel.add(Box.createHorizontalGlue());	    
	    contentPane.add(textButtonPanel); 
	    
	    
	    JPanel imageButtonPanel = new JPanel();
	    imageButtonPanel.setLayout(new BoxLayout(imageButtonPanel, BoxLayout.X_AXIS));
	    imageButtonPanel.add(imageButton);
	    imageButtonPanel.add(Box.createHorizontalGlue());
	    contentPane.add(imageButtonPanel);	    
	    
	    
	    JPanel calendarButtonPanel = new JPanel();
	    calendarButtonPanel.setLayout(new BoxLayout(calendarButtonPanel, BoxLayout.X_AXIS));
	    calendarButtonPanel.add(calendarButton);
	    calendarButtonPanel.add(Box.createHorizontalGlue());
	    contentPane.add(calendarButtonPanel);	
	    
	    
	    JPanel okButtonPanel = new JPanel();
	    okButtonPanel.setLayout(new BoxLayout(okButtonPanel, BoxLayout.X_AXIS));
	    okButtonPanel.add(Box.createHorizontalGlue());
	    okButtonPanel.add(okButton);
	    contentPane.add(okButtonPanel);
	    
	    if (location == null) {
	    	location = PeaFactoryProperties.getFrameLocation();
	    }	    	    
	    this.setLocation(location);
		pack();
	}

	@Override
	public void actionPerformed(ActionEvent ape) {
		
		String com = ape.getActionCommand();

		if (com.equals("ok")) {
			// set main frame visible:
			MainView.updateFrame(this.getLocationOnScreen());	
			this.dispose();
		} else {
			//
		}
	}

	//
	// logic of selection and deselection
	//
	@Override
	public void itemStateChanged(ItemEvent ie) {
		AbstractButton aButton = (AbstractButton)ie.getSource();
        int state = ie.getStateChange();
        String com = aButton.getActionCommand();
        PswDialogView.setInitializing(true);// otherwise NullPointer
        if (com.equals("file")) {
           	if (state == ItemEvent.SELECTED) {				
				DataType.setCurrentType( new FileType() );
        		PeaProperties.setFileType("file");        		

        	} else { // ItemEvent.DESELECTED
        		//
        	}
        } else if (com.equals("notes")) {
        	if (state == ItemEvent.SELECTED) {
				DataType.setCurrentType( new EditorType() );
				PeaProperties.setFileType("text file");        		

        	} else { // ItemEvent.DESELECTED
        	}
        } else if (com.equals("image")) {
        	if (state == ItemEvent.SELECTED) {
				DataType.setCurrentType( new ImageType() );
				PeaProperties.setFileType("image");       
        	} else { // ItemEvent.DESELECTED
        		//
        	}
        } else if (com.equals("calendar")) {
        	if (state == ItemEvent.SELECTED) {
				DataType.setCurrentType( new CalendarType() );
				PeaProperties.setFileType("calendar");        		
        	} else { // ItemEvent.DESELECTED
        		//
        	}
        }
	}
}