package cologne.eck.peafactory.gui.kdf_settings;


/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Settings for the password hashing scheme Argon2. 
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
//import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.KeyStroke;
import javax.swing.text.NumberFormatter;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CloseOnEscapeAction;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.kdf_setting.NumberOrEmptyFormatter;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.kdf.Argon2KDF;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.TestLog;



@SuppressWarnings("serial")
public class Argon2Setting extends JDialog implements ActionListener {
	
	private static String instance = "Argon2id";//default
	
	private Argon2Setting setting;

	private JFormattedTextField memoryField;
	private JFormattedTextField parallelField;
	private JFormattedTextField timeField;
	private JLabel memoryRecommendedLabel;
	private JLabel timeRecommendedLabel;
	private JLabel parallelRecommendedLabel;
	
	private JLabel errorLabel;
	
	private Window owner;
	
	public Argon2Setting(Window _owner) {
		super(_owner);//PeaFactory.getFrame() );
		this.owner = _owner;
		setting = this;
		//setting.setUndecorated(true);
		//setting.setAlwaysOnTop(true);		
		
		// Avoid keyboard trap: Close with ESC
		this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
    		.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "CLOSE_ON_ESCAPE");
		this.getRootPane().getActionMap().put("CLOSE_ON_ESCAPE", new CloseOnEscapeAction() );
		
		this.setIconImage(PswDialogView.getImage());

		JPanel argonPane = (JPanel) setting.getContentPane();//new JPanel();
		argonPane.setBorder(PeaBorderFactory.createBorder(false));
		argonPane.setLayout(new BoxLayout(argonPane, BoxLayout.Y_AXIS));
		
		JLabel scryptLabel = new JLabel("Settings for Argon2:");
		scryptLabel.setPreferredSize(new Dimension(250, 30));
		
		JLabel tipLabel1 = new JLabel("Settings for this session only");
		tipLabel1.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		tipLabel1.setPreferredSize(new Dimension(250, 20));
		argonPane.add(scryptLabel);
		argonPane.add(tipLabel1);

		
		JLabel instanceLabel = new JLabel("Select an instance of Argon2:");
		instanceLabel.setPreferredSize(new Dimension(250, 40));
		argonPane.add(instanceLabel);		
		
		JLabel instanceRecommendedLabel = new JLabel("recommended: Argon2id");
		instanceRecommendedLabel.setPreferredSize(new Dimension(250, 20));
		instanceRecommendedLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		argonPane.add(instanceRecommendedLabel);

	    
		JRadioButton idButton = new JRadioButton("Argon2id");
	    idButton.addActionListener(this);
	    idButton.setActionCommand("id");
	    argonPane.add(idButton);
	    
		JRadioButton dButton = new JRadioButton("Argon2d");
	    dButton.addActionListener(this);
	    dButton.setActionCommand("d");
	    argonPane.add(dButton);
	    
		JRadioButton iButton = new JRadioButton("Argon2i");
	    iButton.addActionListener(this);
	    iButton.setActionCommand("i");
	    argonPane.add(iButton);
		
	    
	    //Group the radio buttons.
	    ButtonGroup group = new ButtonGroup();
	    group.add(iButton);
	    group.add(idButton);
	    group.add(dButton);
	    
	    group.setSelected(idButton.getModel(), true);
	    
		argonPane.add(Box.createVerticalStrut(20));

		NumberFormatter parallelFormatter = new NumberOrEmptyFormatter();//NumberFormat.getIntegerInstance());
		parallelFormatter.setMaximum(99);
		//numberFormatter.setOverwriteMode(true);
		
		
		JLabel parallelLabel = new JLabel("Parallelism parameter p (lanes): ");
		parallelLabel.setPreferredSize(new Dimension(220, 40));
		argonPane.add(parallelLabel);			
		parallelField = new JFormattedTextField(parallelFormatter);
		parallelField.setText("4");
		parallelField.setColumns(2);				
		parallelRecommendedLabel = new JLabel("recommended 4");
		parallelRecommendedLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		JPanel parallelPanel = new JPanel();
		parallelPanel.add(parallelField);
		parallelPanel.add(parallelRecommendedLabel);
		argonPane.add(parallelPanel);		
		
		//DecimalFormat df = new DecimalFormat();
		//df.format(value).replace(Character.toString(decSep), "");
		NumberFormatter memoryFormatter = new NumberOrEmptyFormatter();
				//new DecimalFormat());//NumberFormat.getIntegerInstance());
		memoryFormatter.setMaximum(99999);

		JLabel memoryLabel = new JLabel("Memory cost parameter m (MiB): ");
		memoryLabel.setPreferredSize(new Dimension(220, 40));
		argonPane.add(memoryLabel);			
		memoryField = new JFormattedTextField(memoryFormatter);
		memoryField.setText("128");
		memoryField.setColumns(5);
		memoryRecommendedLabel = new JLabel("recommended >= 128");
		memoryRecommendedLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		JPanel memoryPanel = new JPanel();
		memoryPanel.add(memoryField);
		memoryPanel.add(memoryRecommendedLabel);
		argonPane.add(memoryPanel);
		
		argonPane.add(Box.createVerticalStrut(10));
		
		NumberFormatter timeFormatter = new NumberOrEmptyFormatter();
		timeFormatter.setMaximum(99);

		JLabel timeLabel = new JLabel("Time cost parameter t: ");
		timeLabel.setPreferredSize(new Dimension(220, 40));
		argonPane.add(timeLabel);			
		timeField = new JFormattedTextField(timeFormatter);
		timeField.setText("3");
		timeField.setColumns(2);
		timeRecommendedLabel = new JLabel("recommended 3");
		timeRecommendedLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		JPanel timePanel = new JPanel();
		timePanel.add(timeField);
		timePanel.add(timeRecommendedLabel);
		argonPane.add(timePanel);		
		
		argonPane.add(Box.createVerticalStrut(10));
		
		errorLabel = new JLabel("");
		errorLabel.setForeground(Color.RED);
		argonPane.add(errorLabel);
		
		JButton scryptOkButton = new JButton("ok");
		scryptOkButton.setActionCommand("newSetting");
		scryptOkButton.addActionListener(this);
		argonPane.add(scryptOkButton);
		setting.pack();
		
		setting.setLocationRelativeTo(owner);//.setLocation(MainView.getFrameLocation());
		setting.setVisible(true);
	}



	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();
		boolean setDefaults = false;
		if (owner instanceof PswDialogView) {
			setDefaults = true;
		}
		
		if (command.equals("newSetting")) {			
			
			if (setDefaults == true) {
				String appendix = instance.substring("Argon2".length(), instance.length());
				TestLog.v(Argon2Setting.class, "Argon2 appendix: " + appendix);
				PeaProperties.setDefaultKDFInstanceAppendix(appendix);
			} else {
				if (instance.equals("Argon2id")){
					Argon2KDF.setType(2);
				} else if (instance.equals("Argon2d")){
					Argon2KDF.setType(0);
				} else if (instance.equals("Argon2i")){
					Argon2KDF.setType(1);
				}
			}
			// check parameters
			int m = 0, t = 0, p = 0;
			try {
				// replace "." in memory field
				String memText = memoryField.getText();
				memText = memText.replace(".",  "");
				m = Integer.parseInt(memText);
				t = Integer.parseInt(timeField.getText());
				p = Integer.parseInt(parallelField.getText());

			} catch (Exception e) {
				errorLabel.setText("invalid cost parameters");
				pack();
				return;
			}
			if (m <= 0 || t <= 0 || p <= 0) {
				errorLabel.setText("invalid cost parameters");
				pack();
				return;
			}
			if (setDefaults == true) {
				// set as default value
				PeaProperties.setDefaultTCost(t);
				PeaProperties.setDefaultMCost(m * 1024);// Argon2 uses KiB, used here: MiB
				PeaProperties.setDefaultParallel(p);
			} else {
				Argon2KDF.setIterations(t);
				Argon2KDF.setMemoryCost(m * 1024);// Argon2 uses KiB, used here: MiB
				Argon2KDF.setLanes(p);
			}

			if (setDefaults == true) {
				// set this KDF with new parameters as KDF to use				
				PeaProperties.setDefaultKDF(new Argon2KDF());
			} else {
				KeyDerivation.setKdf( new Argon2KDF() );
//				KeyDerivation.setVersionString(versionString);
			}
			// apply:
			if (setDefaults == true) {				// password was not yet typed
				String defaultAlgos = AlgoParamHandler.writeString(true);
				PeaProperties.setDefaultAlgoParamString(defaultAlgos);
				TestLog.o(Argon2Setting.class, "New default algorithms and parameters: " + AlgoParamHandler.getReadableString(defaultAlgos, false));
				new PropertyHandler().setOneProperty("default_algo_param", defaultAlgos);
				
			} else {
				// ask to apply changes or make further settings
				if (PeaControl.getDialog() != null &&
						PeaControl.getDialog().getLockFrame() != null 
						&& PeaControl.getDialog().getLockFrame() instanceof PeaLockFrame) {
					String errorMessage = ((PeaLockFrame) PeaControl.getDialog().getLockFrame()).changePasswordOrApplyChanges(
						(PeaLockFrame) PeaControl.getDialog().getLockFrame(), null);
					if (errorMessage != null) {
						TestLog.ve(Argon2Setting.class, "Error occurred when changing KDF Argon2 parameters", 5);
						PeaDialog.showMessage(PeaControl.getDialog().getLockFrame(), 
								errorMessage, 
								PeaProperties.getVmBridge().translate("error"), 0);
					}
				} else { // called in PeaFactory
					
				}
			}			
			setting.dispose();
			
		} else { // RadioButton for instance
			instance = "Argon2" + ape.getActionCommand();
		}
	}
/*	public static void main(String[] args){
	
		JFrame f = new JFrame();
		f.setSize(new Dimension(100,100));
		f.setLocation(500,  300);
		f.setVisible(true);
		Argon2Setting c = new Argon2Setting(f);
		c.setVisible(true);
	} */

}
