package cologne.eck.peafactory.gui.kdf_settings;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Dialog to change parameters of the Bcrypt password hashing scheme.
 */

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.*;

import cologne.eck.all_peas.gui.CloseOnEscapeAction;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.all_peas.gui.kdf_setting.AllOtherAdvancedPanel;
import cologne.eck.peafactory.crypto.kdf.BcryptKDF;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.peafactory.gui.MainView;


@SuppressWarnings("serial")
public class BcryptSetting extends JDialog implements ActionListener {

	private BcryptSetting bcryptSetting;

	private AllOtherAdvancedPanel aoa = null;

	public
	BcryptSetting(Window owner) {

		super(owner);

		bcryptSetting = this;
		bcryptSetting.setAlwaysOnTop(true);		

		this.setIconImage(MainView.getImage());

		// Avoid keyboard trap: Close with ESC
		this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
		.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "CLOSE_ON_ESCAPE");
		this.getRootPane().getActionMap().put("CLOSE_ON_ESCAPE", new CloseOnEscapeAction() );

		JPanel bcryptPane = (JPanel) bcryptSetting.getContentPane();
		bcryptPane.setBorder(PeaBorderFactory.createBorder(false));
		bcryptPane.setLayout(new BoxLayout(bcryptPane, BoxLayout.Y_AXIS));

		aoa = new AllOtherAdvancedPanel(new BcryptKDF());
		bcryptPane.add(aoa);

		JButton bcryptOkButton = new JButton("ok");
		bcryptOkButton.setActionCommand("newBcryptSetting");
		bcryptOkButton.addActionListener(this);
		bcryptPane.add(bcryptOkButton);
		bcryptSetting.setSize(new Dimension(350,220));
		bcryptSetting.setLocationRelativeTo(owner);
		bcryptSetting.setVisible(true);	
	}



	@Override
	public void actionPerformed(ActionEvent ape) {

		BcryptKDF.setRounds( aoa.getParams()[0]);
		// set this KDF with new parameters as KDF to use
		KeyDerivation.setKdf(new BcryptKDF());
		bcryptSetting.dispose();	
	}
}
