package cologne.eck.peafactory.gui.kdf_settings;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Settings for the password hashing scheme Catena. 
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.KeyStroke;
import javax.swing.text.NumberFormatter;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CloseOnEscapeAction;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.kdf_setting.NumberOrEmptyFormatter;
import cologne.eck.peafactory.crypto.AlgoParamHandler;
import cologne.eck.peafactory.crypto.kdf.CatenaKDF;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.TestLog;



@SuppressWarnings("serial")
public class CatenaSetting extends JDialog implements ActionListener {
	
	private static String instance = "Dragonfly-Full";
	
	private CatenaSetting setting;

	private JFormattedTextField memoryField;
	private JFormattedTextField timeField;
	private JLabel memoryRecommendedLabel;
	private JLabel timeRecommendedLabel;
	
	private JLabel errorLabel;
	
	private Window owner;
	
	public CatenaSetting(Window _owner) {
		super(_owner);
		this.owner = _owner;
		setting = this;
		
		// Avoid keyboard trap: Close with ESC
		this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
    		.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "CLOSE_ON_ESCAPE");
		this.getRootPane().getActionMap().put("CLOSE_ON_ESCAPE", new CloseOnEscapeAction() );
		//setting.setUndecorated(true);
//		setting.setAlwaysOnTop(true);		
		
		this.setIconImage(PswDialogView.getImage());

		JPanel scryptPane = (JPanel) setting.getContentPane();//new JPanel();
		scryptPane.setBorder(PeaBorderFactory.createBorder(false));
		scryptPane.setLayout(new BoxLayout(scryptPane, BoxLayout.Y_AXIS));
		
		JLabel scryptLabel = new JLabel("Settings for CATENA:");
		scryptLabel.setPreferredSize(new Dimension(250, 30));
		
		JLabel tipLabel1 = new JLabel("Settings for this session only");
		tipLabel1.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		tipLabel1.setPreferredSize(new Dimension(250, 20));
		scryptPane.add(scryptLabel);
		scryptPane.add(tipLabel1);

		
		JLabel instanceLabel = new JLabel("Select an instance of Catena:");
		instanceLabel.setPreferredSize(new Dimension(250, 40));
		scryptPane.add(instanceLabel);		
		
		JLabel instanceRecommendedLabel = new JLabel("recommended: Dragonfly-Full");
		instanceRecommendedLabel.setPreferredSize(new Dimension(250, 20));
		instanceRecommendedLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		scryptPane.add(instanceRecommendedLabel);

	    
		JRadioButton dragonflyFullButton = new JRadioButton("Dragonfly-Full");
	    dragonflyFullButton.setMnemonic(KeyEvent.VK_U);
	    dragonflyFullButton.addActionListener(this);
	    dragonflyFullButton.setActionCommand("Dragonfly-Full");
	    scryptPane.add(dragonflyFullButton);
	    
		JRadioButton butterflyFullButton = new JRadioButton("Butterfly-Full");
	    butterflyFullButton.setMnemonic(KeyEvent.VK_U);
	    butterflyFullButton.addActionListener(this);
	    butterflyFullButton.setActionCommand("Butterfly-Full");
	    scryptPane.add(butterflyFullButton);
	    
		JRadioButton dragonflyButton = new JRadioButton("Dragonfly");
	    dragonflyButton.setMnemonic(KeyEvent.VK_B);
	    dragonflyButton.addActionListener(this);
	    dragonflyButton.setActionCommand("Dragonfly");
	    scryptPane.add(dragonflyButton);
		
		JRadioButton butterflyButton = new JRadioButton("Butterfly");
	    butterflyButton.setMnemonic(KeyEvent.VK_B);
	    butterflyButton.addActionListener(this);
	    butterflyButton.setActionCommand("Butterfly");
	    scryptPane.add(butterflyButton);
	    
	    //Group the radio buttons.
	    ButtonGroup group = new ButtonGroup();
	    group.add(dragonflyButton);
	    group.add(dragonflyFullButton);
	    group.add(butterflyButton);
	    group.add(butterflyFullButton);
	    
	    group.setSelected(dragonflyFullButton.getModel(), true);
	    
		scryptPane.add(Box.createVerticalStrut(20));

		JLabel memoryLabel = new JLabel("Memory cost parameter GARLIC: ");
		memoryLabel.setPreferredSize(new Dimension(220, 40));
		scryptPane.add(memoryLabel);			
		NumberFormatter memoryFormatter = new NumberOrEmptyFormatter();
		memoryFormatter.setMaximum(99);
		memoryField = new JFormattedTextField(memoryFormatter);

		memoryField.setText("18");
		memoryField.setColumns(2);
		memoryField.setDragEnabled(true);

		memoryRecommendedLabel = new JLabel("recommended >= 18");
		memoryRecommendedLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		JPanel memoryPanel = new JPanel();
		memoryPanel.add(memoryField);
		memoryPanel.add(memoryRecommendedLabel);
		scryptPane.add(memoryPanel);
		
		scryptPane.add(Box.createVerticalStrut(10));

		JLabel timeLabel = new JLabel("Time cost parameter LAMBDA: ");
		timeLabel.setPreferredSize(new Dimension(220, 40));
		scryptPane.add(timeLabel);			
		NumberFormatter timeFormatter = new NumberOrEmptyFormatter();
		timeFormatter.setMaximum(999);
		timeField = new JFormattedTextField(timeFormatter);
		timeField.setText("2");
		timeField.setColumns(3);
				
		timeRecommendedLabel = new JLabel("recommended 2");
		timeRecommendedLabel.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 11));
		JPanel timePanel = new JPanel();
		timePanel.add(timeField);
		timePanel.add(timeRecommendedLabel);
		scryptPane.add(timePanel);		
		
		scryptPane.add(Box.createVerticalStrut(10));
		
		errorLabel = new JLabel("");
		errorLabel.setForeground(Color.RED);
		scryptPane.add(errorLabel);
		
		JButton scryptOkButton = new JButton("ok");
		scryptOkButton.setActionCommand("newSetting");
		scryptOkButton.addActionListener(this);
		scryptPane.add(scryptOkButton);
		setting.pack();
		setting.setLocationRelativeTo(owner);
		setting.setVisible(true);
	}



	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();
		boolean setDefaults = false;
		if (owner instanceof PswDialogView) {
			setDefaults = true;
		}
		
		if (command.equals("newSetting")) {
			// check parameters
			int garlic = Integer.parseInt(memoryField.getText());							  
			int lambda = Integer.parseInt(timeField.getText());
			if (garlic > 63) {
				errorLabel.setText("invalid memory cost parameter: must be < 64");
				pack();
				return;
			}
			if (garlic == 0 || lambda == 0) {
				errorLabel.setText("invalid cost parameters");
				pack();
				return;
			}
			if (setDefaults == true) {
				PeaProperties.setDefaultKDFInstanceAppendix(instance);
				PeaProperties.setDefaultMCost(garlic);
				PeaProperties.setDefaultTCost(lambda);
				PeaProperties.setDefaultParallel(0);
				PeaProperties.setDefaultKDF(new CatenaKDF());
			} else {
				if (instance.equals("Dragonfly-Full") || instance.equals("Dragonfly")){
					CatenaKDF.setVersionID(instance);
					CatenaKDF.setGarlicDragonfly(garlic);
					CatenaKDF.setLambdaDragonfly(lambda);
				} else if (instance.equals("Butterfly-Full") || instance.equals("Butterfly")){
					CatenaKDF.setVersionID(instance);
					CatenaKDF.setGarlicButterfly(garlic);
					CatenaKDF.setLambdaButterfly(lambda);
				}
				// set KDF with new parameters as KDF to use
				KeyDerivation.setKdf( new CatenaKDF() );
//				KeyDerivation.setVersionString(x);
			}

			
			if (setDefaults == true) {
				// password not yet set...
				String defaultAlgos = AlgoParamHandler.writeString(true);
				PeaProperties.setDefaultAlgoParamString(defaultAlgos);
				TestLog.o(CatenaSetting.class, "New default algorithms and parameters: " + AlgoParamHandler.getReadableString(defaultAlgos, false));
				new PropertyHandler().setOneProperty("default_algo_param", defaultAlgos);
			} else { // in lockFrame
				// ask to apply changes or make further settings
				if (PeaControl.getDialog() != null &&
						PeaControl.getDialog().getLockFrame() != null 
						&& PeaControl.getDialog().getLockFrame() instanceof PeaLockFrame) {
					String errorMessage = ((PeaLockFrame) PeaControl.getDialog().getLockFrame()).changePasswordOrApplyChanges(
						(PeaLockFrame) PeaControl.getDialog().getLockFrame(), null);
					if (errorMessage != null) {
						PeaDialog.showMessage(PeaControl.getDialog().getLockFrame(), 
								errorMessage, 
								PeaProperties.getVmBridge().translate("error"), 0);
					}
				} else { // called in PeaFactory
					
				}
			}			
			setting.dispose();
			
		} else { // RadioButton for instance
			instance = ape.getActionCommand();
			int currentGarlic = Integer.parseInt(memoryField.getText());
			int currentLambda = Integer.parseInt(timeField.getText());
			if (currentGarlic > 22 || currentLambda > 4){
				errorLabel.setText("Warning: Long execution time");
			} else {
				errorLabel.setText("");
			}
			
			if (instance.equals("Dragonfly")){
				timeRecommendedLabel.setText("recommended 2"); // lambda: 2
				memoryRecommendedLabel.setText("recommended >= 18"); // garlic			
				timeField.setText("2");
				memoryField.setText("18");
			} else if (instance.equals("Dragonfly-Full")){
				timeRecommendedLabel.setText("recommended 2"); // lambda: 2
				memoryRecommendedLabel.setText("recommended >= 18"); // garlic		
				timeField.setText("2");
				memoryField.setText("18");
			} else if (instance.equals("Butterfly")){
				timeRecommendedLabel.setText("recommended 4"); // lambda: 4
				memoryRecommendedLabel.setText("recommended >= 14"); // garlic		
				timeField.setText("4");
				memoryField.setText("14");
			} else if (instance.equals("Butterfly-Full")){
				timeRecommendedLabel.setText("recommended 4"); // lambda: 4
				memoryRecommendedLabel.setText("recommended >= 14"); // garlic	
				timeField.setText("4");
				memoryField.setText("14");
			}
		}
	}
/*	public static void main(String[] args){
		CatenaSetting c = new CatenaSetting();
		c.setVisible(true);
	} */
}
