package cologne.eck.peafactory.gui.kdf_settings;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Settings for the password hashing scheme Pomelo. 
 */

import java.awt.Color;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.KeyStroke;

import cologne.eck.all_peas.gui.CloseOnEscapeAction;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.all_peas.gui.kdf_setting.AllOtherAdvancedPanel;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.peafactory.crypto.kdf.PomeloKDF;
import cologne.eck.peafactory.gui.MainView;


@SuppressWarnings("serial")
public class PomeloSetting extends JDialog implements ActionListener {
	
	private PomeloSetting pomeloSetting;
	
	private AllOtherAdvancedPanel aoa = null;
	
	private JLabel errorLabel;
	
	public PomeloSetting(Window owner) {
		super(owner);
		
		// Avoid keyboard trap: Close with ESC
		this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
    		.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "CLOSE_ON_ESCAPE");
		this.getRootPane().getActionMap().put("CLOSE_ON_ESCAPE", new CloseOnEscapeAction() );
		
		pomeloSetting = this;
		//pomeloSetting.setUndecorated(true);
		pomeloSetting.setAlwaysOnTop(true);		
		
		this.setIconImage(MainView.getImage());

		JPanel pane = (JPanel) pomeloSetting.getContentPane();//new JPanel();
		pane.setBorder(PeaBorderFactory.createBorder(false));
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
		
		aoa = new AllOtherAdvancedPanel(new PomeloKDF() );
		pane.add(aoa);

		errorLabel = new JLabel("");
		errorLabel.setForeground(Color.RED);
		pane.add(errorLabel);
				
		JButton okButton = new JButton("ok");
		okButton.setActionCommand("newSetting");
		okButton.addActionListener(this);
		pane.add(okButton);
		pomeloSetting.pack();
		pomeloSetting.setLocationRelativeTo(owner);
		pomeloSetting.setVisible(true);
	}

	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();
		
		if (command.equals("newSetting")) {
			int[] params = aoa.getParams();

			int t_cost = params[0];
			int m_cost = params[1];
						
			if (t_cost < 0 || t_cost > 25) {
				errorLabel.setText("Invalid time cost parameter");
				return;
			}
			if (m_cost < 0 || m_cost > 25) {
				errorLabel.setText("Invalid memory cost parameter");
				return;
			}			
			if ( ((t_cost + m_cost) < 5 )|| (t_cost + m_cost > 25)){
				errorLabel.setText("Invalid parameters: memory cost + time cost must be > 5 and < 25");
				pack();
				return;
			}
						
			PomeloKDF.setMemoryCost( m_cost);
			PomeloKDF.setTimeCost( t_cost );
			// set this KDF with new parameters as KDF to use
			KeyDerivation.setKdf(new PomeloKDF() );
			pomeloSetting.dispose();
		}
	}
}
