package cologne.eck.peafactory.gui.kdf_settings;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Settings for the password hashing scheme Scrypt. 
 * Changing the parameters should result in displaying the required
 * memory. 
 */


import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CloseOnEscapeAction;
import cologne.eck.all_peas.gui.PeaBorderFactory;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.kdf_setting.AllOtherAdvancedPanel;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.peafactory.crypto.kdf.ScryptKDF;
import cologne.eck.peafactory.gui.MainView;


@SuppressWarnings("serial")
public class ScryptSetting extends JDialog implements ActionListener {
	
	private ScryptSetting scryptSetting;
	
	private AllOtherAdvancedPanel aoa = null;	
	
	public
	ScryptSetting(Window owner) {
		super(owner );
		scryptSetting = this;
		//scryptSetting.setUndecorated(true);
		scryptSetting.setAlwaysOnTop(true);		
		
		// Avoid keyboard trap: Close with ESC
		this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
    		.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "CLOSE_ON_ESCAPE");
		this.getRootPane().getActionMap().put("CLOSE_ON_ESCAPE", new CloseOnEscapeAction() );
		
		this.setIconImage(MainView.getImage());

		JPanel scryptPane = (JPanel) scryptSetting.getContentPane();//new JPanel();
		scryptPane.setBorder(PeaBorderFactory.createBorder(false));
		scryptPane.setLayout(new BoxLayout(scryptPane, BoxLayout.Y_AXIS));
		aoa = new AllOtherAdvancedPanel(new ScryptKDF());
		scryptPane.add(aoa);
						
		JButton scryptOkButton = new JButton("ok");
		scryptOkButton.setActionCommand("newScryptSetting");
		scryptOkButton.addActionListener(this);
		scryptPane.add(scryptOkButton);
		scryptSetting.pack();
		scryptSetting.setLocationRelativeTo(owner);
		scryptSetting.setVisible(true);
	}
	
	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();
		
			if (command.equals("newScryptSetting")) {
			
			int[] params = aoa.getParams();
			
			// check if parameters are valid:
			boolean setSuccess = new ScryptKDF().setParameters(params);
			if (setSuccess == false) {
				PeaDialog.showMessage(SwingUtilities.getWindowAncestor(this), 
						PeaProperties.getVmBridge().translate("invalid") + ":\n" + PeaProperties.getVmBridge().translate("parameter"), 
						PeaProperties.getVmBridge().translate("error"), 0);
				return;
			}
			
			ScryptKDF.setcPUFactor( params[0] );
			ScryptKDF.setMemoryFactor( params[1] );			
			ScryptKDF.setParallelFactor( params[2] );			

			// set this KDF with new parameters as KDF to use
			KeyDerivation.setKdf(new ScryptKDF());
			scryptSetting.dispose();
		}
	}
}
