package cologne.eck.tools;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;


public class LanguageBundleHelper {
	
	/**
	 * 0. Set folder and laguageFile for PEA (extra for Calendar Lock PEA)
	 * 
	 * 1. Create the keys and values file from default language bundle
	 *    by createKeyValues()
	 *    
	 * 2. Manually translate the values block by block in languages and store in props files:
	 *    xx.props e.g. tr.props
	 *    
	 * 3. Create languages bundles in new languages 
	 *    by createLanguage("xx")
	 *    
	 * 4. Translate the HTML info files by LanguageHTMLHelper.class
	 * 
	 * 5. Copy the language bundles and HTML info files in eck/peafactory/src/lang
	 *    in folder named with language e.g. "tr"
	 *    
	 * 6. Upload these files
	 * 
	 */
	
	private static final int MAX_SIZE = 5000;// Block to translate
	
	/**
	 * Translations must be stored as file xx.props in this folder: 
	 * 
	 */
//	private static String folder = System.getProperty("user.dir") + File.separator + "all_peas_lang" + File.separator;
//	private static 	File languageFile = new File(System.getProperty("user.dir") + File.separator + "resources" + File.separator + "PeaLanguagesBundle.properties");
	
	// Calendar Lock PEA
	private static String folder = System.getProperty("user.dir") + File.separator + "cal_pea_lang" + File.separator;
	private static 	File languageFile = new File(System.getProperty("user.dir") + File.separator + "resources" + File.separator + "CalendarLanguageBundle.properties");

	private static String valueFileName = folder + "values.txt";

	private static String keyFileName = folder + "keys.txt";
	
	public static void main(String[] args) {
		
		// 1. Create new keys and values
//		createKeyValues();
		
		// 2. Translate manually block by block TODO
		// xx.props e.g. tr.props

		// 3. Create bundles
		createLanguageFile("es");
		createLanguageFile("fr");
		createLanguageFile("ru");
		createLanguageFile("tr"); 
		
		createLanguageFile("pt");
		createLanguageFile("it");
		createLanguageFile("cz");
		createLanguageFile("ro");
		createLanguageFile("nl");
		
		createLanguageFile("cn");
		createLanguageFile("gr");
		createLanguageFile("lv");
		createLanguageFile("jp"); 
		//createLanguageFile("ar"); 
		
		// TODO to be read in java 8:
		// open in e.g. LibreOffice, paste in eclipse (chars are escaped automatically)
	}
	

	
	/**
	 * Create the files for keys and values: 
	 * keys.txt and values.txt
	 * The file values.txt must be translated manually
	 * block by block into the languages, which must then be stored in
	 * xx.props e.g. tr.props
	 * 
	 * @return	true for success
	 */
	private static boolean createKeyValues() {
		
		StringBuilder keys = new StringBuilder();
		StringBuilder values = new StringBuilder();
		int sizeToTranslate = 0;
		
		if (languageFile.exists()) {
			System.out.println("file found");
			try (BufferedReader br = new BufferedReader(new FileReader(languageFile))) {
			    String line;
			    while ((line = br.readLine()) != null) {
			       // process the line.
			    	if (line.startsWith("#")) {
			    		
			    	} else if (line.trim().isEmpty()) {
			    		
			    	} else {			    		
			    		int partIndex = line.indexOf("=");
			    		if (partIndex == -1) {
			    			System.err.println("Missing = in line: " + line);
			    		} else {
			    			String key = line.substring(0, partIndex);			    				
			    			String val = line.substring(partIndex + 1);
			    			while (val.endsWith("\\")){ // next line belongs to value
			    				if (br.readLine() != null) {
			    					val = val.trim() + " " + br.readLine().trim();
			    				} else {
			    					TestLog.ve(LanguageBundleHelper.class, "Reader is null", 5);
			    				}
			    			}
			    			sizeToTranslate += val.length();
			    			if (sizeToTranslate <= MAX_SIZE) {
			    				if (val.trim().length() > 1) { // no mnemonics
			    					keys.append( key.trim() + "\n");
			    					values.append(val.trim() + "\n");// without =
			    				}
			    			} else {
			    				// new line 
			    				keys.append( "\n\n");
			    				values.append("\n\n");
			    				sizeToTranslate = val.length(); // reset counter
			    				keys.append( key.trim() + "\n");
			    				values.append(val.trim() + "\n");// without =
			    			}
			    		}
			    	}
			    }
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		//System.out.println(new String(values));
		boolean valueSuccess = WriteResources.writeText( new String(values), valueFileName);
		boolean keySuccess = WriteResources.writeText(new String(keys), keyFileName);
		if (valueSuccess == true && keySuccess == true) {
			System.out.println("Created: values.txt, keys.txt: \n" + valueFileName + "\n" + keyFileName);
			return true;
		} else {
			System.err.println("Creating keys and value failed...");
			return false;
		}
	}
	
	/**
	 * Create a language bundle file from a manually translated
	 * file xx.props e.g. tr.props and the file keys.txt
	 * 
	 * 
	 * @param lang	the language to create a bundle
	 */
	private static void createLanguageFile(String lang){
	/*	BufferedWriter writer = null;
	    try {
			writer = new BufferedWriter(new FileWriter(System.getProperty("user.dir") + File.separator + "PeaLanguagesBundle_" +lang + ".properties"));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} */
	    BufferedReader keyReader = null;
		try {
			keyReader = new BufferedReader(new FileReader(new File(keyFileName)));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (keyReader == null) {
			return;
		}
	    BufferedReader valueReader = null;
		try {
			valueReader = new BufferedReader(new FileReader(new File(folder + lang + ".props")));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		StringBuilder resultBuilder = new StringBuilder();
	    String keyLine = null;
	    String valLine = null;
	    try {
			while ((keyLine = keyReader.readLine()) != null && (valLine = valueReader.readLine()) != null) {
				if (keyLine.trim().length() == 0) {
					if (valLine == null || valLine.trim().length() == 0) {
						System.out.println("Empty line..." + " - language: " + lang);
						continue;
					} else {
						System.err.println("Missing key for value " + valLine + " - language: " + lang);
					}
				}
				if (valLine == null || valLine.trim().length() == 0) {
					System.err.println("Missing value for key " + keyLine + " - language: " + lang + " - " + keyLine + " = " + valLine);
					valLine = " ";
				}
				String line = keyLine + " = " + valLine + "\n";
				//System.out.println(line);
				resultBuilder.append(line + "\n");
			    }
			keyReader.close();
			if (valueReader != null) {
				valueReader.close();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	    String languageFileName = languageFile.getName();
	    String newLanguageFileName = folder + 
	    		languageFileName.substring(0, languageFileName.indexOf("."))
	    + "_" + lang + ".properties";
	    System.out.println("Create: " + newLanguageFileName);
	    File file = new File(newLanguageFileName);
	    //File file = new File(folder + "PeaLanguagesBundle_" +lang + ".properties");
	    Boolean success = WriteResources.writeText(new String(resultBuilder), file.getAbsolutePath());
	    if (success == false){
	    	System.err.println(   	WriteResources.getLastErrorMessage() );
	    } else {
	    	System.out.println("Created " + file.getAbsolutePath());
	    }
	}
}
